@@ -1,7 +1,7 @@
 /*
  * wee-completion.c - completion for WeeChat commands
  *
- * Copyright (C) 2003-2017 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
  * Copyright (C) 2006 Emmanuel Bouthenot <kolter@openics.org>
  *
  * This file is part of WeeChat, the extensible chat client.
@@ -17,7 +17,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -36,6 +36,7 @@
 #include "weechat.h"
 #include "wee-arraylist.h"
 #include "wee-config.h"
+#include "wee-eval.h"
 #include "wee-hashtable.h"
 #include "wee-hook.h"
 #include "wee-list.h"
@@ -199,6 +200,97 @@ completion_list_add_buffers_plugins_names_cb (const void *pointer, void *data,
     return WEECHAT_RC_OK;
 }
 
+/*
+ * Adds a buffer local variable to completions list.
+ */
+
+void
+completion_list_map_buffer_local_variable_cb (void *data,
+                                              struct t_hashtable *hashtable,
+                                              const void *key, const void *value)
+{
+    /* make C compiler happy */
+    (void) hashtable;
+    (void) value;
+
+    gui_completion_list_add ((struct t_gui_completion *)data,
+                             (const char *)key,
+                             0, WEECHAT_LIST_POS_SORT);
+}
+
+/*
+ * Adds buffer local variables to completion list.
+ */
+
+int
+completion_list_add_buffer_local_variables_cb (const void *pointer, void *data,
+                                               const char *completion_item,
+                                               struct t_gui_buffer *buffer,
+                                               struct t_gui_completion *completion)
+{
+    /* make C compiler happy */
+    (void) pointer;
+    (void) data;
+    (void) completion_item;
+    (void) buffer;
+
+    hashtable_map (completion->buffer->local_variables,
+                   &completion_list_map_buffer_local_variable_cb,
+                   completion);
+
+    return WEECHAT_RC_OK;
+}
+
+/*
+ * Adds buffer local variable value to completion list.
+ */
+
+int
+completion_list_add_buffer_local_variable_value_cb (const void *pointer, void *data,
+                                                    const char *completion_item,
+                                                    struct t_gui_buffer *buffer,
+                                                    struct t_gui_completion *completion)
+{
+    char **argv;
+    int argc, arg_index;
+    const char *ptr_value;
+
+    /* make C compiler happy */
+    (void) pointer;
+    (void) data;
+    (void) completion_item;
+    (void) buffer;
+
+    if (!completion->args)
+        return WEECHAT_RC_OK;
+
+    argv = string_split (completion->args, " ", NULL,
+                         WEECHAT_STRING_SPLIT_STRIP_LEFT
+                         | WEECHAT_STRING_SPLIT_STRIP_RIGHT
+                         | WEECHAT_STRING_SPLIT_COLLAPSE_SEPS,
+                         0, &argc);
+    if (!argv)
+        return WEECHAT_RC_OK;
+
+    if (argc > 0)
+    {
+        arg_index = completion->base_command_arg_index - 2;
+        if ((arg_index < 1) || (arg_index > argc - 1))
+            arg_index = argc - 1;
+        ptr_value = hashtable_get (completion->buffer->local_variables,
+                                   argv[arg_index]);
+        if (ptr_value)
+        {
+            gui_completion_list_add (completion,
+                                     ptr_value,
+                                     0, WEECHAT_LIST_POS_SORT);
+        }
+    }
+    string_free_split (argv);
+
+    return WEECHAT_RC_OK;
+}
+
 /*
  * Adds buffer properties (that can be set) to completion list.
  */
@@ -435,6 +527,7 @@ completion_list_add_filename_cb (const void *pointer, void *data,
     char home[3] = { '~', DIR_SEPARATOR_CHAR, '\0' };
     char *ptr_home, *pos, buf[PATH_MAX], *real_prefix, *prefix, *path_dir;
     char *path_base, *dir_name;
+    const char *pos_args;
     int length_path_base;
     DIR *dp;
     struct dirent *entry;
@@ -443,11 +536,14 @@ completion_list_add_filename_cb (const void *pointer, void *data,
     /* make C compiler happy */
     (void) pointer;
     (void) data;
-    (void) completion_item;
     (void) buffer;
 
     completion->add_space = 0;
 
+    pos_args = (completion_item) ? strchr (completion_item, ':') : NULL;
+    if (pos_args)
+        pos_args++;
+
     ptr_home = getenv ("HOME");
 
     real_prefix = NULL;
@@ -466,7 +562,18 @@ completion_list_add_filename_cb (const void *pointer, void *data,
         if (!completion->base_word[0]
             || completion->base_word[0] != DIR_SEPARATOR_CHAR)
         {
-            real_prefix = strdup (weechat_home);
+            real_prefix = NULL;
+            if (pos_args && pos_args[0])
+            {
+                real_prefix = eval_expression (pos_args, NULL, NULL, NULL);
+                if (real_prefix && !real_prefix[0])
+                {
+                    free (real_prefix);
+                    real_prefix = NULL;
+                }
+            }
+            if (!real_prefix)
+                real_prefix = strdup (weechat_home);
             prefix = strdup ("");
         }
         else
@@ -603,7 +710,7 @@ completion_list_add_commands_cb (const void *pointer, void *data,
     (void) data;
     (void) buffer;
 
-    pos = strchr (completion_item, ':');
+    pos = (completion_item) ? strchr (completion_item, ':') : NULL;
     if (pos)
         pos++;
 
@@ -616,7 +723,7 @@ completion_list_add_commands_cb (const void *pointer, void *data,
         {
             if (pos)
             {
-                snprintf (str_command, sizeof(str_command),
+                snprintf (str_command, sizeof (str_command),
                           "%s%s",
                           pos,
                           HOOK_COMMAND(ptr_hook, command));
@@ -964,13 +1071,17 @@ completion_list_add_plugins_commands_cb (const void *pointer, void *data,
     if (!completion->args)
         return WEECHAT_RC_OK;
 
-    argv = string_split (completion->args, " ", 0, 0, &argc);
+    argv = string_split (completion->args, " ", NULL,
+                         WEECHAT_STRING_SPLIT_STRIP_LEFT
+                         | WEECHAT_STRING_SPLIT_STRIP_RIGHT
+                         | WEECHAT_STRING_SPLIT_COLLAPSE_SEPS,
+                         0, &argc);
     if (!argv)
         return WEECHAT_RC_OK;
 
     if (argc > 0)
     {
-        pos = strchr (completion_item, ':');
+        pos = (completion_item) ? strchr (completion_item, ':') : NULL;
         if (pos)
             pos++;
 
@@ -1000,7 +1111,7 @@ completion_list_add_plugins_commands_cb (const void *pointer, void *data,
             {
                 if (pos)
                 {
-                    snprintf (str_command, sizeof(str_command),
+                    snprintf (str_command, sizeof (str_command),
                               "%s%s",
                               pos,
                               HOOK_COMMAND(ptr_hook, command));
@@ -1017,7 +1128,7 @@ completion_list_add_plugins_commands_cb (const void *pointer, void *data,
         }
     }
 
-    string_free_split(argv);
+    string_free_split (argv);
 
     return WEECHAT_RC_OK;
 }
@@ -1267,7 +1378,7 @@ completion_list_add_weechat_commands_cb (const void *pointer, void *data,
     (void) data;
     (void) buffer;
 
-    pos = strchr (completion_item, ':');
+    pos = (completion_item) ? strchr (completion_item, ':') : NULL;
     if (pos)
         pos++;
 
@@ -1281,7 +1392,7 @@ completion_list_add_weechat_commands_cb (const void *pointer, void *data,
         {
             if (pos)
             {
-                snprintf (str_command, sizeof(str_command),
+                snprintf (str_command, sizeof (str_command),
                           "%s%s",
                           pos,
                           HOOK_COMMAND(ptr_hook, command));
@@ -1649,7 +1760,11 @@ completion_list_add_env_value_cb (const void *pointer, void *data,
 
     if (completion->args)
     {
-        argv = string_split (completion->args, " ", 0, 0, &argc);
+        argv = string_split (completion->args, " ", NULL,
+                             WEECHAT_STRING_SPLIT_STRIP_LEFT
+                             | WEECHAT_STRING_SPLIT_STRIP_RIGHT
+                             | WEECHAT_STRING_SPLIT_COLLAPSE_SEPS,
+                             0, &argc);
         if (!argv)
             return WEECHAT_RC_OK;
 
@@ -1687,6 +1802,12 @@ completion_init ()
     hook_completion (NULL, "buffers_plugins_names", /* formerly "%B" */
                      N_("names of buffers (including plugins names)"),
                      &completion_list_add_buffers_plugins_names_cb, NULL, NULL);
+    hook_completion (NULL, "buffer_local_variables",
+                     N_("buffer local variables"),
+                     &completion_list_add_buffer_local_variables_cb, NULL, NULL);
+    hook_completion (NULL, "buffer_local_variable_value",
+                     N_("value of a buffer local variable"),
+                     &completion_list_add_buffer_local_variable_value_cb, NULL, NULL);
     hook_completion (NULL, "buffer_properties_set",
                      N_("properties that can be set on a buffer"),
                      &completion_list_add_buffer_properties_set_cb, NULL, NULL);
@@ -1706,7 +1827,9 @@ completion_init ()
                      N_("configuration files"),
                      &completion_list_add_config_files_cb, NULL, NULL);
     hook_completion (NULL, "filename", /* formerly "%f" */
-                     N_("filename"),
+                     N_("filename; "
+                        "optional argument: default path (evaluated, "
+                        "see /help eval)"),
                      &completion_list_add_filename_cb, NULL, NULL);
     hook_completion (NULL, "filters_names", /* formerly "%F" */
                      N_("names of filters"),
