@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2017 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
  * Copyright (C) 2005-2006 Emmanuel Bouthenot <kolter@openics.org>
  *
  * This file is part of WeeChat, the extensible chat client.
@@ -15,7 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef WEECHAT_CONFIG_H
@@ -58,6 +58,8 @@ enum t_config_look_nick_color_hash
 {
     CONFIG_LOOK_NICK_COLOR_HASH_DJB2 = 0,
     CONFIG_LOOK_NICK_COLOR_HASH_SUM,
+    CONFIG_LOOK_NICK_COLOR_HASH_DJB2_32,
+    CONFIG_LOOK_NICK_COLOR_HASH_SUM_32,
 };
 
 enum t_config_look_prefix_align
@@ -149,6 +151,7 @@ extern struct t_config_option *config_look_buffer_search_force_default;
 extern struct t_config_option *config_look_buffer_search_regex;
 extern struct t_config_option *config_look_buffer_search_where;
 extern struct t_config_option *config_look_buffer_time_format;
+extern struct t_config_option *config_look_buffer_time_same;
 extern struct t_config_option *config_look_color_basic_force_bold;
 extern struct t_config_option *config_look_color_inactive_buffer;
 extern struct t_config_option *config_look_color_inactive_message;
@@ -185,6 +188,7 @@ extern struct t_config_option *config_look_hotlist_short_names;
 extern struct t_config_option *config_look_hotlist_sort;
 extern struct t_config_option *config_look_hotlist_suffix;
 extern struct t_config_option *config_look_hotlist_unique_numbers;
+extern struct t_config_option *config_look_hotlist_update_on_buffer_switch;
 extern struct t_config_option *config_look_input_cursor_scroll;
 extern struct t_config_option *config_look_input_share;
 extern struct t_config_option *config_look_input_share_overwrite;
@@ -203,6 +207,7 @@ extern struct t_config_option *config_look_mouse;
 extern struct t_config_option *config_look_mouse_timer_delay;
 extern struct t_config_option *config_look_nick_color_force;
 extern struct t_config_option *config_look_nick_color_hash;
+extern struct t_config_option *config_look_nick_color_hash_salt;
 extern struct t_config_option *config_look_nick_color_stop_chars;
 extern struct t_config_option *config_look_nick_prefix;
 extern struct t_config_option *config_look_nick_suffix;
@@ -221,6 +226,7 @@ extern struct t_config_option *config_look_prefix_buffer_align_max;
 extern struct t_config_option *config_look_prefix_buffer_align_more;
 extern struct t_config_option *config_look_prefix_buffer_align_more_after;
 extern struct t_config_option *config_look_prefix_same_nick;
+extern struct t_config_option *config_look_prefix_same_nick_middle;
 extern struct t_config_option *config_look_prefix_suffix;
 extern struct t_config_option *config_look_quote_nick_prefix;
 extern struct t_config_option *config_look_quote_nick_suffix;
@@ -228,6 +234,7 @@ extern struct t_config_option *config_look_quote_time_format;
 extern struct t_config_option *config_look_read_marker;
 extern struct t_config_option *config_look_read_marker_always_show;
 extern struct t_config_option *config_look_read_marker_string;
+extern struct t_config_option *config_look_read_marker_update_on_buffer_switch;
 extern struct t_config_option *config_look_save_config_on_exit;
 extern struct t_config_option *config_look_save_config_with_fsync;
 extern struct t_config_option *config_look_save_layout_on_exit;
@@ -320,6 +327,7 @@ extern struct t_config_option *config_completion_partial_completion_command;
 extern struct t_config_option *config_completion_partial_completion_command_arg;
 extern struct t_config_option *config_completion_partial_completion_count;
 extern struct t_config_option *config_completion_partial_completion_other;
+extern struct t_config_option *config_completion_partial_completion_templates;
 
 extern struct t_config_option *config_history_display_default;
 extern struct t_config_option *config_history_max_buffer_lines_minutes;
@@ -340,6 +348,7 @@ extern struct t_config_option *config_plugin_save_config_on_unload;
 
 extern int config_length_nick_prefix_suffix;
 extern int config_length_prefix_same_nick;
+extern int config_length_prefix_same_nick_middle;
 extern int config_emphasized_attributes;
 extern regex_t *config_highlight_regex;
 extern char ***config_highlight_tags;
@@ -354,6 +363,8 @@ extern int config_word_chars_input_count;
 extern char **config_nick_colors;
 extern int config_num_nick_colors;
 extern struct t_hashtable *config_hashtable_nick_color_force;
+extern char *config_buffer_time_same_evaluated;
+extern struct t_hashtable *config_hashtable_completion_partial_templates;
 
 extern void config_set_nick_colors ();
 extern struct t_config_option *config_weechat_debug_get (const char *plugin_name);
