@@ -1,7 +1,7 @@
 /*
  * wee-debug.c - debug functions
  *
- * Copyright (C) 2003-2017 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -34,9 +34,7 @@
 #include <curl/curl.h>
 #include <zlib.h>
 
-#ifdef HAVE_GNUTLS
 #include <gnutls/gnutls.h>
-#endif
 
 #include "weechat.h"
 #include "wee-backtrace.h"
@@ -54,6 +52,7 @@
 #include "../gui/gui-bar-item.h"
 #include "../gui/gui-buffer.h"
 #include "../gui/gui-chat.h"
+#include "../gui/gui-completion.h"
 #include "../gui/gui-filter.h"
 #include "../gui/gui-hotlist.h"
 #include "../gui/gui-key.h"
@@ -92,6 +91,7 @@ debug_dump (int crash)
 
     gui_window_print_log ();
     gui_buffer_print_log ();
+    gui_completion_print_log ();
     gui_layout_print_log ();
     gui_key_print_log (NULL);
     gui_filter_print_log ();
@@ -167,10 +167,14 @@ debug_sigsegv ()
         stderr,
         "***\n"
         "*** Please help WeeChat developers to fix this bug:\n"
-        "***   1. If you have a core file, please run:  gdb /path/to/weechat core\n"
-        "***      then issue command: \"bt full\" and send result to developers\n"
-        "***      (see user's guide for more info about report of crashes).\n"
-        "***   2. Otherwise send backtrace (below), only if it is a complete trace.\n"
+        "***\n"
+        "***   1. If you have a core file, please run: gdb /path/to/weechat core\n"
+        "***      then issue command: \"bt full\" and send result to developers.\n"
+        "***      See the user's guide for more info about enabling the core files\n"
+        "***      and reporting crashes:\n"
+        "***      https://weechat.org/doc/stable/user#report_crashes\n"
+        "***\n"
+        "***   2. Otherwise send the backtrace (below), only if it is a complete trace.\n"
         "***      Keep the crash log file, just in case developers ask you some info\n"
         "***      (be careful, private info like passwords may be in this file).\n\n");
 
@@ -534,7 +538,6 @@ debug_libs_cb (const void *pointer, void *data,
 #endif /* GCRYPT_VERSION */
 
     /* display gnutls version */
-#ifdef HAVE_GNUTLS
 #ifdef GNUTLS_VERSION
     gui_chat_printf (NULL, "    gnutls: %s%s",
                      GNUTLS_VERSION,
@@ -543,9 +546,6 @@ debug_libs_cb (const void *pointer, void *data,
     gui_chat_printf (NULL, "    gnutls: (?)%s",
                      (weechat_no_gnutls) ? " (not initialized)" : "");
 #endif /* GNUTLS_VERSION */
-#else
-    gui_chat_printf (NULL, "    gnutls: (not available)");
-#endif /* HAVE_GNUTLS */
 
     /* display curl version */
 #ifdef LIBCURL_VERSION
@@ -577,8 +577,12 @@ debug_directories ()
 
     gui_chat_printf (NULL, "");
     gui_chat_printf (NULL, _("Directories:"));
-    gui_chat_printf (NULL, "  home: %s (%s: %s)",
-                     weechat_home, _("default"), WEECHAT_HOME);
+    gui_chat_printf (NULL, "  home: %s%s%s",
+                     weechat_home,
+                     (weechat_home_temp) ? " " : "",
+                     (weechat_home_temp) ? _("(TEMPORARY, deleted on exit)") : "");
+    gui_chat_printf (NULL, _("        (default: %s)"),
+                     WEECHAT_HOME);
     gui_chat_printf (NULL, "  lib: %s", WEECHAT_LIBDIR);
     gui_chat_printf (NULL, "  lib (extra): %s",
                      (extra_libdir && extra_libdir[0]) ? extra_libdir : "-");
@@ -611,13 +615,13 @@ debug_display_time_elapsed (struct timeval *time1, struct timeval *time2,
     if (display)
     {
         gui_chat_printf (NULL,
-                         "debug: time[%s] -> %lld:%02lld:%02lld.%06d",
+                         "debug: time[%s] -> %lld:%02lld:%02lld.%06lld",
                          (message) ? message : "?",
                          diff_hour, diff_min, diff_sec, diff_usec);
     }
     else
     {
-        log_printf ("debug: time[%s] -> %lld:%02lld:%02lld.%06d",
+        log_printf ("debug: time[%s] -> %lld:%02lld:%02lld.%06lld",
                     (message) ? message : "?",
                     diff_hour, diff_min, diff_sec, diff_usec);
     }
