@@ -1,7 +1,7 @@
 /*
  * wee-network.c - network functions
  *
- * Copyright (C) 2003-2014 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
  * Copyright (C) 2005-2010 Emmanuel Bouthenot <kolter@openics.org>
  * Copyright (C) 2010 Gu1ll4um3r0m41n <aeroxteam@gmail.com>
  * Copyright (C) 2012 Simon Arlott
@@ -19,7 +19,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -39,9 +39,12 @@
 #include <string.h>
 #include <sys/types.h>
 #include <sys/socket.h>
+#include <poll.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
+#include <arpa/nameser.h>
 #include <netdb.h>
+#include <resolv.h>
 #include <errno.h>
 #include <gcrypt.h>
 #include <sys/time.h>
@@ -49,9 +52,7 @@
 #include <sys/uio.h>
 #endif
 
-#ifdef HAVE_GNUTLS
 #include <gnutls/gnutls.h>
-#endif
 
 #include "weechat.h"
 #include "wee-network.h"
@@ -65,9 +66,7 @@
 
 int network_init_gnutls_ok = 0;
 
-#ifdef HAVE_GNUTLS
 gnutls_certificate_credentials_t gnutls_xcred; /* GnuTLS client credentials */
-#endif
 
 
 /*
@@ -77,12 +76,12 @@ gnutls_certificate_credentials_t gnutls_xcred; /* GnuTLS client credentials */
 void
 network_init_gcrypt ()
 {
-    if (!weechat_no_gcrypt)
-    {
-        gcry_check_version (GCRYPT_VERSION);
-        gcry_control (GCRYCTL_DISABLE_SECMEM, 0);
-        gcry_control (GCRYCTL_INITIALIZATION_FINISHED, 0);
-    }
+    if (weechat_no_gcrypt)
+        return;
+
+    gcry_check_version (GCRYPT_VERSION);
+    gcry_control (GCRYCTL_DISABLE_SECMEM, 0);
+    gcry_control (GCRYCTL_INITIALIZATION_FINISHED, 0);
 }
 
 /*
@@ -92,7 +91,6 @@ network_init_gcrypt ()
 void
 network_set_gnutls_ca_file ()
 {
-#ifdef HAVE_GNUTLS
     char *ca_path, *ca_path2;
 
     if (weechat_no_gnutls)
@@ -110,7 +108,6 @@ network_set_gnutls_ca_file ()
         }
         free (ca_path);
     }
-#endif
 }
 
 /*
@@ -120,29 +117,24 @@ network_set_gnutls_ca_file ()
 void
 network_init_gnutls ()
 {
-#ifdef HAVE_GNUTLS
     if (!weechat_no_gnutls)
     {
         gnutls_global_init ();
         gnutls_certificate_allocate_credentials (&gnutls_xcred);
 
         network_set_gnutls_ca_file ();
-#if LIBGNUTLS_VERSION_NUMBER >= 0x02090a
-        /* for gnutls >= 2.9.10 */
+#if LIBGNUTLS_VERSION_NUMBER >= 0x02090a /* 2.9.10 */
         gnutls_certificate_set_verify_function (gnutls_xcred,
                                                 &hook_connect_gnutls_verify_certificates);
-#endif
-#if LIBGNUTLS_VERSION_NUMBER >= 0x020b00
-        /* for gnutls >= 2.11.0 */
+#endif /* LIBGNUTLS_VERSION_NUMBER >= 0x02090a */
+#if LIBGNUTLS_VERSION_NUMBER >= 0x020b00 /* 2.11.0 */
         gnutls_certificate_set_retrieve_function (gnutls_xcred,
                                                   &hook_connect_gnutls_set_certificates);
 #else
-        /* for gnutls < 2.11.0 */
         gnutls_certificate_client_set_retrieve_function (gnutls_xcred,
                                                          &hook_connect_gnutls_set_certificates);
-#endif
+#endif /* LIBGNUTLS_VERSION_NUMBER >= 0x020b00 */
     }
-#endif /* HAVE_GNUTLS */
 
     network_init_gnutls_ok = 1;
 }
@@ -156,13 +148,11 @@ network_end ()
 {
     if (network_init_gnutls_ok)
     {
-#ifdef HAVE_GNUTLS
         if (!weechat_no_gnutls)
         {
             gnutls_certificate_free_credentials (gnutls_xcred);
-            gnutls_global_deinit();
+            gnutls_global_deinit ();
         }
-#endif
         network_init_gnutls_ok = 0;
     }
 }
@@ -246,7 +236,7 @@ int
 network_pass_httpproxy (struct t_proxy *proxy, int sock, const char *address,
                         int port)
 {
-    char buffer[256], authbuf[128], authbuf_base64[512], *username, *password;
+    char buffer[4096], authbuf[128], authbuf_base64[512], *username, *password;
     int length;
 
     if (CONFIG_STRING(proxy->options[PROXY_OPTION_USERNAME])
@@ -267,7 +257,8 @@ network_pass_httpproxy (struct t_proxy *proxy, int sock, const char *address,
         snprintf (authbuf, sizeof (authbuf), "%s:%s", username, password);
         free (username);
         free (password);
-        string_encode_base64 (authbuf, strlen (authbuf), authbuf_base64);
+        if (string_base64_encode (authbuf, strlen (authbuf), authbuf_base64) < 0)
+            return 0;
         length = snprintf (buffer, sizeof (buffer),
                            "CONNECT %s:%d HTTP/1.0\r\nProxy-Authorization: "
                            "Basic %s\r\n\r\n",
@@ -313,13 +304,15 @@ network_resolve (const char *hostname, char *ip, int *version)
 
     res = NULL;
 
+    res_init ();
+
     if (getaddrinfo (hostname, NULL, NULL, &res) != 0)
         return 0;
 
     if (!res)
         return 0;
 
-    if (getnameinfo (res->ai_addr, res->ai_addrlen, ipbuffer, sizeof(ipbuffer),
+    if (getnameinfo (res->ai_addr, res->ai_addrlen, ipbuffer, sizeof (ipbuffer),
                      NULL, 0, NI_NUMERICHOST) != 0)
     {
         freeaddrinfo (res);
@@ -342,7 +335,7 @@ network_resolve (const char *hostname, char *ip, int *version)
 /*
  * Establishes a connection and authenticates with a socks4 proxy.
  *
- * The socks4 protocol is explained here: http://en.wikipedia.org/wiki/SOCKS
+ * The socks4 protocol is explained here: https://en.wikipedia.org/wiki/SOCKS
  *
  * WARNING: this function is blocking, it must be called only in a forked
  * process.
@@ -492,9 +485,9 @@ network_pass_socks5proxy (struct t_proxy *proxy, int sock, const char *address,
     }
 
     /* authentication successful then giving address/port to connect */
-    addr_len = strlen(address);
+    addr_len = strlen (address);
     addr_buffer_len = 4 + 1 + addr_len + 2;
-    addr_buffer = malloc (addr_buffer_len * sizeof(*addr_buffer));
+    addr_buffer = malloc (addr_buffer_len * sizeof (*addr_buffer));
     if (!addr_buffer)
         return 0;
     addr_buffer[0] = 5;   /* version 5 */
@@ -613,7 +606,7 @@ network_pass_proxy (const char *proxy, int sock, const char *address, int port)
 int
 network_connect (int sock, const struct sockaddr *addr, socklen_t addrlen)
 {
-    fd_set write_fds;
+    struct pollfd poll_fd;
     int ready, value;
     socklen_t len;
 
@@ -623,15 +616,19 @@ network_connect (int sock, const struct sockaddr *addr, socklen_t addrlen)
     if (errno != EINPROGRESS)
         return 0;
 
-    /* for non-blocking sockets, the connect() may fail with EINPROGRESS,
+    /*
+     * for non-blocking sockets, the connect() may fail with EINPROGRESS,
      * if this happens, we wait for writability on socket and check
      * the option SO_ERROR, which is 0 if connect is OK (see man connect)
      */
     while (1)
     {
-        FD_ZERO (&write_fds);
-        FD_SET (sock, &write_fds);
-        ready = select (sock + 1, NULL, &write_fds, NULL, NULL);
+        poll_fd.fd = sock;
+        poll_fd.events = POLLOUT;
+        poll_fd.revents = 0;
+        ready = poll (&poll_fd, 1, -1);
+        if (ready < 0)
+            break;
         if (ready > 0)
         {
             len = sizeof (value);
@@ -668,6 +665,9 @@ network_connect_to (const char *proxy, struct sockaddr *address,
     sock = -1;
     proxy_addrinfo = NULL;
 
+    if (!address || (address_length == 0))
+        return -1;
+
     ptr_proxy = NULL;
     if (proxy && proxy[0])
     {
@@ -694,6 +694,7 @@ network_connect_to (const char *proxy, struct sockaddr *address,
         hints.ai_flags = AI_NUMERICSERV;
         snprintf (str_port, sizeof (str_port), "%d",
                  CONFIG_INTEGER(ptr_proxy->options[PROXY_OPTION_PORT]));
+        res_init ();
         if (getaddrinfo (CONFIG_STRING(ptr_proxy->options[PROXY_OPTION_ADDRESS]),
                          str_port, &hints, &proxy_addrinfo) != 0)
         {
@@ -747,16 +748,12 @@ network_connect_child (struct t_hook *hook_connect)
     char status_without_string[1 + 5 + 1];
     const char *error;
     int rc, length, num_written;
-    int sock, set, flags;
-#ifdef HOOK_CONNECT_MAX_SOCKETS
-    int j;
-#else
+    int sock, set, flags, j;
     struct msghdr msg;
     struct cmsghdr *cmsg;
     char msg_buf[CMSG_SPACE(sizeof (sock))];
     struct iovec iov[1];
     char iov_data[1] = { 0 };
-#endif
     /*
      * indicates that something is wrong with whichever group of
      * servers is being tried first after connecting, so start at
@@ -803,7 +800,8 @@ network_connect_child (struct t_hook *hook_connect)
     hints.ai_socktype = SOCK_STREAM;
 #ifdef AI_ADDRCONFIG
     hints.ai_flags = AI_ADDRCONFIG;
-#endif
+#endif /* AI_ADDRCONFIG */
+    res_init ();
     if (ptr_proxy)
     {
         hints.ai_family = (CONFIG_BOOLEAN(ptr_proxy->options[PROXY_OPTION_IPV6])) ?
@@ -871,7 +869,7 @@ network_connect_child (struct t_hook *hook_connect)
         hints.ai_socktype = SOCK_STREAM;
 #ifdef AI_ADDRCONFIG
         hints.ai_flags = AI_ADDRCONFIG;
-#endif
+#endif /* AI_ADDRCONFIG */
         rc = getaddrinfo (HOOK_CONNECT(hook_connect, local_hostname),
                           NULL, &hints, &res_local);
         if (rc != 0)
@@ -977,7 +975,7 @@ network_connect_child (struct t_hook *hook_connect)
             if (tmp_num_groups >= retry)
             {
                 /* shuffle while adding */
-                rand_num = tmp_host + (rand() % ((i + 1) - tmp_host));
+                rand_num = tmp_host + (rand () % ((i + 1) - tmp_host));
                 if (rand_num == i)
                     res_reorder[i++] = ptr_res;
                 else
@@ -1008,7 +1006,7 @@ network_connect_child (struct t_hook *hook_connect)
             if (tmp_num_groups < retry)
             {
                 /* shuffle while adding */
-                rand_num = tmp_host + (rand() % ((i + 1) - tmp_host));
+                rand_num = tmp_host + (rand () % ((i + 1) - tmp_host));
                 if (rand_num == i)
                     res_reorder[i++] = ptr_res;
                 else
@@ -1042,38 +1040,41 @@ network_connect_child (struct t_hook *hook_connect)
     {
         ptr_res = res_reorder[i];
 
-#ifdef HOOK_CONNECT_MAX_SOCKETS
-        /* use pre-created socket pool */
-        sock = -1;
-        for (j = 0; j < HOOK_CONNECT_MAX_SOCKETS; j++)
+        if (hook_socketpair_ok)
         {
-            if (ptr_res->ai_family == AF_INET)
+            /* create a socket */
+            sock = socket (ptr_res->ai_family,
+                           ptr_res->ai_socktype,
+                           ptr_res->ai_protocol);
+        }
+        else
+        {
+            /* use pre-created socket pool */
+            sock = -1;
+            for (j = 0; j < HOOK_CONNECT_MAX_SOCKETS; j++)
             {
-                sock = HOOK_CONNECT(hook_connect, sock_v4[j]);
-                if (sock != -1)
+                if (ptr_res->ai_family == AF_INET)
                 {
-                    HOOK_CONNECT(hook_connect, sock_v4[j]) = -1;
-                    break;
+                    sock = HOOK_CONNECT(hook_connect, sock_v4[j]);
+                    if (sock != -1)
+                    {
+                        HOOK_CONNECT(hook_connect, sock_v4[j]) = -1;
+                        break;
+                    }
                 }
-            }
-            else if (ptr_res->ai_family == AF_INET6)
-            {
-                sock = HOOK_CONNECT(hook_connect, sock_v6[j]);
-                if (sock != -1)
+                else if (ptr_res->ai_family == AF_INET6)
                 {
-                    HOOK_CONNECT(hook_connect, sock_v6[j]) = -1;
-                    break;
+                    sock = HOOK_CONNECT(hook_connect, sock_v6[j]);
+                    if (sock != -1)
+                    {
+                        HOOK_CONNECT(hook_connect, sock_v6[j]) = -1;
+                        break;
+                    }
                 }
             }
+            if (sock < 0)
+                continue;
         }
-        if (sock < 0)
-            continue;
-#else
-        /* create a socket */
-        sock = socket (ptr_res->ai_family,
-                       ptr_res->ai_socktype,
-                       ptr_res->ai_protocol);
-#endif
         if (sock < 0)
         {
             status_str[0] = '0' + WEECHAT_HOOK_CONNECT_SOCKET_ERROR;
@@ -1181,30 +1182,36 @@ network_connect_child (struct t_hook *hook_connect)
         }
 
         /* send the socket to the parent process */
-#ifndef HOOK_CONNECT_MAX_SOCKETS
-        memset (&msg, 0, sizeof (msg));
-        msg.msg_control = msg_buf;
-        msg.msg_controllen = sizeof (msg_buf);
+        if (hook_socketpair_ok)
+        {
+            memset (&msg, 0, sizeof (msg));
+            msg.msg_control = msg_buf;
+            msg.msg_controllen = sizeof (msg_buf);
 
-        /* send 1 byte of data (not required on Linux, required by BSD/OSX) */
-        memset (iov, 0, sizeof (iov));
-        iov[0].iov_base = iov_data;
-        iov[0].iov_len = 1;
-        msg.msg_iov = iov;
-        msg.msg_iovlen = 1;
+            /*
+             * send 1 byte of data
+             * (not required on Linux, required by BSD/macOS)
+             */
+            memset (iov, 0, sizeof (iov));
+            iov[0].iov_base = iov_data;
+            iov[0].iov_len = 1;
+            msg.msg_iov = iov;
+            msg.msg_iovlen = 1;
 
-        cmsg = CMSG_FIRSTHDR(&msg);
-        cmsg->cmsg_level = SOL_SOCKET;
-        cmsg->cmsg_type = SCM_RIGHTS;
-        cmsg->cmsg_len = CMSG_LEN(sizeof (sock));
-        memcpy(CMSG_DATA(cmsg), &sock, sizeof (sock));
-        msg.msg_controllen = cmsg->cmsg_len;
-        num_written = sendmsg (HOOK_CONNECT(hook_connect, child_send), &msg, 0);
-        (void) num_written;
-#else
-        num_written = write (HOOK_CONNECT(hook_connect, child_write), &sock, sizeof (sock));
-        (void) num_written;
-#endif
+            cmsg = CMSG_FIRSTHDR(&msg);
+            cmsg->cmsg_level = SOL_SOCKET;
+            cmsg->cmsg_type = SCM_RIGHTS;
+            cmsg->cmsg_len = CMSG_LEN(sizeof (sock));
+            memcpy (CMSG_DATA(cmsg), &sock, sizeof (sock));
+            msg.msg_controllen = cmsg->cmsg_len;
+            num_written = sendmsg (HOOK_CONNECT(hook_connect, child_send), &msg, 0);
+            (void) num_written;
+        }
+        else
+        {
+            num_written = write (HOOK_CONNECT(hook_connect, child_write), &sock, sizeof (sock));
+            (void) num_written;
+        }
     }
     else
     {
@@ -1231,19 +1238,22 @@ end:
  */
 
 int
-network_connect_child_timer_cb (void *arg_hook_connect, int remaining_calls)
+network_connect_child_timer_cb (const void *pointer, void *data,
+                                int remaining_calls)
 {
     struct t_hook *hook_connect;
 
     /* make C compiler happy */
+    (void) data;
     (void) remaining_calls;
 
-    hook_connect = (struct t_hook *)arg_hook_connect;
+    hook_connect = (struct t_hook *)pointer;
 
     HOOK_CONNECT(hook_connect, hook_child_timer) = NULL;
 
     (void) (HOOK_CONNECT(hook_connect, callback))
-        (hook_connect->callback_data,
+        (hook_connect->callback_pointer,
+         hook_connect->callback_data,
          WEECHAT_HOOK_CONNECT_TIMEOUT,
          0, -1, NULL, NULL);
     unhook (hook_connect);
@@ -1258,17 +1268,18 @@ network_connect_child_timer_cb (void *arg_hook_connect, int remaining_calls)
  * finish).
  */
 
-#ifdef HAVE_GNUTLS
 int
-network_connect_gnutls_handshake_fd_cb (void *arg_hook_connect, int fd)
+network_connect_gnutls_handshake_fd_cb (const void *pointer, void *data,
+                                        int fd)
 {
     struct t_hook *hook_connect;
     int rc, direction, flags;
 
     /* make C compiler happy */
+    (void) data;
     (void) fd;
 
-    hook_connect = (struct t_hook *)arg_hook_connect;
+    hook_connect = (struct t_hook *)pointer;
 
     rc = gnutls_handshake (*HOOK_CONNECT(hook_connect, gnutls_sess));
 
@@ -1287,8 +1298,10 @@ network_connect_gnutls_handshake_fd_cb (void *arg_hook_connect, int fd)
     }
     else if (rc != GNUTLS_E_SUCCESS)
     {
+        unhook (HOOK_CONNECT(hook_connect, handshake_hook_fd));
         (void) (HOOK_CONNECT(hook_connect, callback))
-            (hook_connect->callback_data,
+            (hook_connect->callback_pointer,
+             hook_connect->callback_data,
              WEECHAT_HOOK_CONNECT_GNUTLS_HANDSHAKE_ERROR, rc,
              HOOK_CONNECT(hook_connect, sock),
              gnutls_strerror (rc),
@@ -1299,7 +1312,7 @@ network_connect_gnutls_handshake_fd_cb (void *arg_hook_connect, int fd)
     {
         fcntl (HOOK_CONNECT(hook_connect, sock), F_SETFL,
                HOOK_CONNECT(hook_connect, handshake_fd_flags));
-#if LIBGNUTLS_VERSION_NUMBER < 0x02090a
+#if LIBGNUTLS_VERSION_NUMBER < 0x02090a /* 2.9.10 */
         /*
          * gnutls only has the gnutls_certificate_set_verify_function()
          * function since version 2.9.10. We need to call our verify
@@ -1307,8 +1320,10 @@ network_connect_gnutls_handshake_fd_cb (void *arg_hook_connect, int fd)
          */
         if (hook_connect_gnutls_verify_certificates (*HOOK_CONNECT(hook_connect, gnutls_sess)) != 0)
         {
+            unhook (HOOK_CONNECT(hook_connect, handshake_hook_fd));
             (void) (HOOK_CONNECT(hook_connect, callback))
-                (hook_connect->callback_data,
+                (hook_connect->callback_pointer,
+                 hook_connect->callback_data,
                  WEECHAT_HOOK_CONNECT_GNUTLS_HANDSHAKE_ERROR, rc,
                  HOOK_CONNECT(hook_connect, sock),
                  "Error in the certificate.",
@@ -1316,80 +1331,77 @@ network_connect_gnutls_handshake_fd_cb (void *arg_hook_connect, int fd)
             unhook (hook_connect);
             return WEECHAT_RC_OK;
         }
-#endif
+#endif /* LIBGNUTLS_VERSION_NUMBER < 0x02090a */
         unhook (HOOK_CONNECT(hook_connect, handshake_hook_fd));
         (void) (HOOK_CONNECT(hook_connect, callback))
-                (hook_connect->callback_data,
-                 WEECHAT_HOOK_CONNECT_OK, 0,
-                 HOOK_CONNECT(hook_connect, sock),
-                 NULL, HOOK_CONNECT(hook_connect, handshake_ip_address));
+            (hook_connect->callback_pointer,
+             hook_connect->callback_data,
+             WEECHAT_HOOK_CONNECT_OK, 0,
+             HOOK_CONNECT(hook_connect, sock),
+             NULL, HOOK_CONNECT(hook_connect, handshake_ip_address));
         unhook (hook_connect);
     }
 
     return WEECHAT_RC_OK;
 }
-#endif
 
 /*
  * Timer callback for timeout of handshake.
  */
 
-#ifdef HAVE_GNUTLS
 int
-network_connect_gnutls_handshake_timer_cb (void *arg_hook_connect,
+network_connect_gnutls_handshake_timer_cb (const void *pointer,
+                                           void *data,
                                            int remaining_calls)
 {
     struct t_hook *hook_connect;
 
     /* make C compiler happy */
+    (void) data;
     (void) remaining_calls;
 
-    hook_connect = (struct t_hook *)arg_hook_connect;
+    hook_connect = (struct t_hook *)pointer;
 
     HOOK_CONNECT(hook_connect, handshake_hook_timer) = NULL;
 
+    unhook (HOOK_CONNECT(hook_connect, handshake_hook_fd));
     (void) (HOOK_CONNECT(hook_connect, callback))
-            (hook_connect->callback_data,
-             WEECHAT_HOOK_CONNECT_GNUTLS_HANDSHAKE_ERROR,
-             GNUTLS_E_EXPIRED,
-             HOOK_CONNECT(hook_connect, sock),
-             gnutls_strerror (GNUTLS_E_EXPIRED),
-             HOOK_CONNECT(hook_connect, handshake_ip_address));
+        (hook_connect->callback_pointer,
+         hook_connect->callback_data,
+         WEECHAT_HOOK_CONNECT_GNUTLS_HANDSHAKE_ERROR,
+         GNUTLS_E_EXPIRED,
+         HOOK_CONNECT(hook_connect, sock),
+         gnutls_strerror (GNUTLS_E_EXPIRED),
+         HOOK_CONNECT(hook_connect, handshake_ip_address));
     unhook (hook_connect);
 
     return WEECHAT_RC_OK;
 }
-#endif
 
 /*
  * Reads connection progress from child process.
  */
 
 int
-network_connect_child_read_cb (void *arg_hook_connect, int fd)
+network_connect_child_read_cb (const void *pointer, void *data, int fd)
 {
     struct t_hook *hook_connect;
     char buffer[1], buf_size[6], *cb_error, *cb_ip_address, *error;
     int num_read;
     long size_msg;
-#ifdef HAVE_GNUTLS
     int rc, direction;
-#endif
-    int sock;
-#ifdef HOOK_CONNECT_MAX_SOCKETS
-    int i;
-#else
+    int sock, i;
     struct msghdr msg;
     struct cmsghdr *cmsg;
     char msg_buf[CMSG_SPACE(sizeof (sock))];
     struct iovec iov[1];
     char iov_data[1];
-#endif
 
     /* make C compiler happy */
+    (void) data;
     (void) fd;
 
-    hook_connect = (struct t_hook *)arg_hook_connect;
+    hook_connect = (struct t_hook *)pointer;
 
     cb_error = NULL;
     cb_ip_address = NULL;
@@ -1427,48 +1439,53 @@ network_connect_child_read_cb (void *arg_hook_connect, int fd)
                 }
             }
 
-#ifndef HOOK_CONNECT_MAX_SOCKETS
-            /* receive the socket from the child process */
-            memset (&msg, 0, sizeof (msg));
-            msg.msg_control = msg_buf;
-            msg.msg_controllen = sizeof (msg_buf);
-
-            /* recv 1 byte of data (not required on Linux, required by BSD/OSX) */
-            memset (iov, 0, sizeof (iov));
-            iov[0].iov_base = iov_data;
-            iov[0].iov_len = 1;
-            msg.msg_iov = iov;
-            msg.msg_iovlen = 1;
-
-            if (recvmsg (HOOK_CONNECT(hook_connect, child_recv), &msg, 0) >= 0)
+            if (hook_socketpair_ok)
             {
-                cmsg = CMSG_FIRSTHDR(&msg);
-                if (cmsg != NULL
-                    && cmsg->cmsg_level == SOL_SOCKET
-                    && cmsg->cmsg_type == SCM_RIGHTS
-                    && cmsg->cmsg_len >= sizeof (sock))
+                /* receive the socket from the child process */
+                memset (&msg, 0, sizeof (msg));
+                msg.msg_control = msg_buf;
+                msg.msg_controllen = sizeof (msg_buf);
+
+                /*
+                 * recv 1 byte of data
+                 * (not required on Linux, required by BSD/macOS)
+                 */
+                memset (iov, 0, sizeof (iov));
+                iov[0].iov_base = iov_data;
+                iov[0].iov_len = 1;
+                msg.msg_iov = iov;
+                msg.msg_iovlen = 1;
+
+                if (recvmsg (HOOK_CONNECT(hook_connect, child_recv), &msg, 0) >= 0)
                 {
-                    memcpy(&sock, CMSG_DATA(cmsg), sizeof (sock));
+                    cmsg = CMSG_FIRSTHDR(&msg);
+                    if (cmsg != NULL
+                        && cmsg->cmsg_level == SOL_SOCKET
+                        && cmsg->cmsg_type == SCM_RIGHTS
+                        && cmsg->cmsg_len >= sizeof (sock))
+                    {
+                        memcpy (&sock, CMSG_DATA(cmsg), sizeof (sock));
+                    }
                 }
             }
-#else
-            num_read = read (HOOK_CONNECT(hook_connect, child_read), &sock, sizeof (sock));
-            (void) num_read;
-
-            /* prevent unhook process from closing the socket */
-            for (i = 0; i < HOOK_CONNECT_MAX_SOCKETS; i++)
+            else
             {
-                if (HOOK_CONNECT(hook_connect, sock_v4[i]) == sock)
-                    HOOK_CONNECT(hook_connect, sock_v4[i]) = -1;
+                num_read = read (HOOK_CONNECT(hook_connect, child_read), &sock, sizeof (sock));
+                (void) num_read;
 
-                if (HOOK_CONNECT(hook_connect, sock_v6[i]) == sock)
-                    HOOK_CONNECT(hook_connect, sock_v6[i]) = -1;
+                /* prevent unhook process from closing the socket */
+                for (i = 0; i < HOOK_CONNECT_MAX_SOCKETS; i++)
+                {
+                    if (HOOK_CONNECT(hook_connect, sock_v4[i]) == sock)
+                        HOOK_CONNECT(hook_connect, sock_v4[i]) = -1;
+
+                    if (HOOK_CONNECT(hook_connect, sock_v6[i]) == sock)
+                        HOOK_CONNECT(hook_connect, sock_v6[i]) = -1;
+                }
             }
-#endif
 
             HOOK_CONNECT(hook_connect, sock) = sock;
 
-#ifdef HAVE_GNUTLS
             if (HOOK_CONNECT(hook_connect, gnutls_sess))
             {
                 /*
@@ -1505,19 +1522,20 @@ network_connect_child_read_cb (void *arg_hook_connect, int fd)
                                  HOOK_CONNECT(hook_connect, sock),
                                  (!direction ? 1 : 0), (direction  ? 1 : 0), 0,
                                  &network_connect_gnutls_handshake_fd_cb,
-                                 hook_connect);
+                                 hook_connect, NULL);
                     HOOK_CONNECT(hook_connect, handshake_hook_timer) =
                         hook_timer (hook_connect->plugin,
                                     CONFIG_INTEGER(config_network_gnutls_handshake_timeout) * 1000,
                                     0, 1,
                                     &network_connect_gnutls_handshake_timer_cb,
-                                    hook_connect);
+                                    hook_connect, NULL);
                     return WEECHAT_RC_OK;
                 }
                 else if (rc != GNUTLS_E_SUCCESS)
                 {
                     (void) (HOOK_CONNECT(hook_connect, callback))
-                        (hook_connect->callback_data,
+                        (hook_connect->callback_pointer,
+                         hook_connect->callback_data,
                          WEECHAT_HOOK_CONNECT_GNUTLS_HANDSHAKE_ERROR,
                          rc, sock,
                          gnutls_strerror (rc),
@@ -1529,7 +1547,7 @@ network_connect_child_read_cb (void *arg_hook_connect, int fd)
                 }
                 fcntl (HOOK_CONNECT(hook_connect, sock), F_SETFL,
                        HOOK_CONNECT(hook_connect, handshake_fd_flags));
-#if LIBGNUTLS_VERSION_NUMBER < 0x02090a
+#if LIBGNUTLS_VERSION_NUMBER < 0x02090a /* 2.9.10 */
                 /*
                  * gnutls only has the gnutls_certificate_set_verify_function()
                  * function since version 2.9.10. We need to call our verify
@@ -1538,7 +1556,8 @@ network_connect_child_read_cb (void *arg_hook_connect, int fd)
                 if (hook_connect_gnutls_verify_certificates (*HOOK_CONNECT(hook_connect, gnutls_sess)) != 0)
                 {
                     (void) (HOOK_CONNECT(hook_connect, callback))
-                        (hook_connect->callback_data,
+                        (hook_connect->callback_pointer,
+                         hook_connect->callback_data,
                          WEECHAT_HOOK_CONNECT_GNUTLS_HANDSHAKE_ERROR,
                          rc, sock,
                          "Error in the certificate.",
@@ -1548,9 +1567,8 @@ network_connect_child_read_cb (void *arg_hook_connect, int fd)
                         free (cb_ip_address);
                     return WEECHAT_RC_OK;
                 }
-#endif
+#endif /* LIBGNUTLS_VERSION_NUMBER < 0x02090a */
             }
-#endif
         }
         else
         {
@@ -1581,15 +1599,19 @@ network_connect_child_read_cb (void *arg_hook_connect, int fd)
             }
         }
         (void) (HOOK_CONNECT(hook_connect, callback))
-            (hook_connect->callback_data, buffer[0] - '0', 0,
+            (hook_connect->callback_pointer,
+             hook_connect->callback_data,
+             buffer[0] - '0', 0,
              sock, cb_error, cb_ip_address);
         unhook (hook_connect);
     }
     else
     {
         (void) (HOOK_CONNECT(hook_connect, callback))
-            (hook_connect->callback_data, WEECHAT_HOOK_CONNECT_MEMORY_ERROR,
-             0, sock, cb_error, cb_ip_address);
+            (hook_connect->callback_pointer,
+             hook_connect->callback_data,
+             WEECHAT_HOOK_CONNECT_MEMORY_ERROR,
+             0, sock, "child_read_cb", NULL);
         unhook (hook_connect);
     }
 
@@ -1608,37 +1630,46 @@ network_connect_child_read_cb (void *arg_hook_connect, int fd)
 void
 network_connect_with_fork (struct t_hook *hook_connect)
 {
-    int child_pipe[2], rc;
-#ifdef HOOK_CONNECT_MAX_SOCKETS
-    int i;
-#else
-    int child_socket[2];
-#endif
-#ifdef HAVE_GNUTLS
+    int child_pipe[2], child_socket[2], rc, i;
+    char str_error[1024];
     const char *pos_error;
-#endif
     pid_t pid;
 
-#ifdef HAVE_GNUTLS
     /* initialize GnuTLS if SSL asked */
     if (HOOK_CONNECT(hook_connect, gnutls_sess))
     {
         if (gnutls_init (HOOK_CONNECT(hook_connect, gnutls_sess), GNUTLS_CLIENT) != GNUTLS_E_SUCCESS)
         {
             (void) (HOOK_CONNECT(hook_connect, callback))
-                (hook_connect->callback_data,
+                (hook_connect->callback_pointer,
+                 hook_connect->callback_data,
                  WEECHAT_HOOK_CONNECT_GNUTLS_INIT_ERROR,
                  0, -1, NULL, NULL);
             unhook (hook_connect);
             return;
         }
+        rc = gnutls_server_name_set (*HOOK_CONNECT(hook_connect, gnutls_sess),
+                                     GNUTLS_NAME_DNS,
+                                     HOOK_CONNECT(hook_connect, address),
+                                     strlen (HOOK_CONNECT(hook_connect, address)));
+        if (rc != GNUTLS_E_SUCCESS)
+        {
+            (void) (HOOK_CONNECT(hook_connect, callback))
+                (hook_connect->callback_pointer,
+                 hook_connect->callback_data,
+                 WEECHAT_HOOK_CONNECT_GNUTLS_INIT_ERROR,
+                 0, -1, _("set server name indication (SNI) failed"), NULL);
+            unhook (hook_connect);
+            return;
+        }
         rc = gnutls_priority_set_direct (*HOOK_CONNECT(hook_connect, gnutls_sess),
                                          HOOK_CONNECT(hook_connect, gnutls_priorities),
                                          &pos_error);
         if (rc != GNUTLS_E_SUCCESS)
         {
             (void) (HOOK_CONNECT(hook_connect, callback))
-                (hook_connect->callback_data,
+                (hook_connect->callback_pointer,
+                 hook_connect->callback_data,
                  WEECHAT_HOOK_CONNECT_GNUTLS_INIT_ERROR,
                  0, -1, _("invalid priorities"), NULL);
             unhook (hook_connect);
@@ -1650,50 +1681,58 @@ network_connect_with_fork (struct t_hook *hook_connect)
         gnutls_transport_set_ptr (*HOOK_CONNECT(hook_connect, gnutls_sess),
                                   (gnutls_transport_ptr_t) ((unsigned long) HOOK_CONNECT(hook_connect, sock)));
     }
-#endif
 
     /* create pipe for child process */
     if (pipe (child_pipe) < 0)
     {
         (void) (HOOK_CONNECT(hook_connect, callback))
-            (hook_connect->callback_data,
+            (hook_connect->callback_pointer,
+             hook_connect->callback_data,
              WEECHAT_HOOK_CONNECT_MEMORY_ERROR,
-             0, -1, NULL, NULL);
+             0, -1, "pipe", NULL);
         unhook (hook_connect);
         return;
     }
     HOOK_CONNECT(hook_connect, child_read) = child_pipe[0];
     HOOK_CONNECT(hook_connect, child_write) = child_pipe[1];
 
-#ifndef HOOK_CONNECT_MAX_SOCKETS
-    /* create socket for child process */
-    if (socketpair (AF_LOCAL, SOCK_DGRAM, 0, child_socket) < 0)
+    if (hook_socketpair_ok)
     {
-        (void) (HOOK_CONNECT(hook_connect, callback))
-            (hook_connect->callback_data,
-             WEECHAT_HOOK_CONNECT_MEMORY_ERROR,
-             0, -1, NULL, NULL);
-        unhook (hook_connect);
-        return;
+        /* create socket for child process */
+        if (socketpair (AF_LOCAL, SOCK_DGRAM, 0, child_socket) < 0)
+        {
+            (void) (HOOK_CONNECT(hook_connect, callback))
+                (hook_connect->callback_pointer,
+                 hook_connect->callback_data,
+                 WEECHAT_HOOK_CONNECT_MEMORY_ERROR,
+                 0, -1, "socketpair", NULL);
+            unhook (hook_connect);
+            return;
+        }
+        HOOK_CONNECT(hook_connect, child_recv) = child_socket[0];
+        HOOK_CONNECT(hook_connect, child_send) = child_socket[1];
     }
-    HOOK_CONNECT(hook_connect, child_recv) = child_socket[0];
-    HOOK_CONNECT(hook_connect, child_send) = child_socket[1];
-#else
-    for (i = 0; i < HOOK_CONNECT_MAX_SOCKETS; i++)
+    else
     {
-        HOOK_CONNECT(hook_connect, sock_v4[i]) = socket (AF_INET, SOCK_STREAM, 0);
-        HOOK_CONNECT(hook_connect, sock_v6[i]) = socket (AF_INET6, SOCK_STREAM, 0);
+        for (i = 0; i < HOOK_CONNECT_MAX_SOCKETS; i++)
+        {
+            HOOK_CONNECT(hook_connect, sock_v4[i]) = socket (AF_INET, SOCK_STREAM, 0);
+            HOOK_CONNECT(hook_connect, sock_v6[i]) = socket (AF_INET6, SOCK_STREAM, 0);
+        }
     }
-#endif
 
     switch (pid = fork ())
     {
         /* fork failed */
         case -1:
+            snprintf (str_error, sizeof (str_error),
+                      "fork error: %s",
+                      strerror (errno));
             (void) (HOOK_CONNECT(hook_connect, callback))
-                (hook_connect->callback_data,
+                (hook_connect->callback_pointer,
+                 hook_connect->callback_data,
                  WEECHAT_HOOK_CONNECT_MEMORY_ERROR,
-                 0, -1, NULL, NULL);
+                 0, -1, str_error, NULL);
             unhook (hook_connect);
             return;
         /* child process */
@@ -1701,9 +1740,8 @@ network_connect_with_fork (struct t_hook *hook_connect)
             rc = setuid (getuid ());
             (void) rc;
             close (HOOK_CONNECT(hook_connect, child_read));
-#ifndef HOOK_CONNECT_MAX_SOCKETS
-            close (HOOK_CONNECT(hook_connect, child_recv));
-#endif
+            if (hook_socketpair_ok)
+                close (HOOK_CONNECT(hook_connect, child_recv));
             network_connect_child (hook_connect);
             _exit (EXIT_SUCCESS);
     }
@@ -1711,18 +1749,20 @@ network_connect_with_fork (struct t_hook *hook_connect)
     HOOK_CONNECT(hook_connect, child_pid) = pid;
     close (HOOK_CONNECT(hook_connect, child_write));
     HOOK_CONNECT(hook_connect, child_write) = -1;
-#ifndef HOOK_CONNECT_MAX_SOCKETS
-    close (HOOK_CONNECT(hook_connect, child_send));
-    HOOK_CONNECT(hook_connect, child_send) = -1;
-#endif
+    if (hook_socketpair_ok)
+    {
+        close (HOOK_CONNECT(hook_connect, child_send));
+        HOOK_CONNECT(hook_connect, child_send) = -1;
+    }
     HOOK_CONNECT(hook_connect, hook_child_timer) = hook_timer (hook_connect->plugin,
                                                                CONFIG_INTEGER(config_network_connection_timeout) * 1000,
                                                                0, 1,
                                                                &network_connect_child_timer_cb,
-                                                               hook_connect);
+                                                               hook_connect,
+                                                               NULL);
     HOOK_CONNECT(hook_connect, hook_fd) = hook_fd (hook_connect->plugin,
                                                    HOOK_CONNECT(hook_connect, child_read),
                                                    1, 0, 0,
                                                    &network_connect_child_read_cb,
-                                                   hook_connect);
+                                                   hook_connect, NULL);
 }
