@@ -1,7 +1,7 @@
 /*
  * wee-upgrade.c - save/restore session data of WeeChat core
  *
- * Copyright (C) 2003-2017 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -32,7 +32,7 @@
 #include "wee-upgrade.h"
 #include "wee-hook.h"
 #include "wee-infolist.h"
-#include "wee-secure.h"
+#include "wee-secure-buffer.h"
 #include "wee-string.h"
 #include "wee-util.h"
 #include "../gui/gui-buffer.h"
@@ -446,6 +446,12 @@ upgrade_weechat_read_buffer (struct t_infolist *infolist)
     /* full name */
     gui_buffer_build_full_name (ptr_buffer);
 
+    /* old full name */
+    if (ptr_buffer->old_full_name)
+        free (ptr_buffer->old_full_name);
+    str = infolist_string (infolist, "old_full_name");
+    ptr_buffer->old_full_name = (str) ? strdup (str) : NULL;
+
     /* short name */
     if (ptr_buffer->short_name)
         free (ptr_buffer->short_name);
@@ -511,6 +517,8 @@ upgrade_weechat_read_buffer (struct t_infolist *infolist)
         infolist_integer (infolist, "input_get_unknown_commands");
     ptr_buffer->input_get_empty =
         infolist_integer (infolist, "input_get_empty");
+    ptr_buffer->input_multiline =
+        infolist_integer (infolist, "input_multiline");
 
     if (infolist_integer (infolist, "input_buffer_alloc") > 0)
     {
@@ -631,7 +639,8 @@ upgrade_weechat_read_buffer_line (struct t_infolist *infolist)
     switch (upgrade_current_buffer->type)
     {
         case GUI_BUFFER_TYPE_FORMATTED:
-            new_line = gui_line_add (upgrade_current_buffer,
+            new_line = gui_line_new (upgrade_current_buffer,
+                                     -1,
                                      infolist_time (infolist, "date"),
                                      infolist_time (infolist, "date_printed"),
                                      infolist_string (infolist, "tags"),
@@ -639,6 +648,7 @@ upgrade_weechat_read_buffer_line (struct t_infolist *infolist)
                                      infolist_string (infolist, "message"));
             if (new_line)
             {
+                gui_line_add (new_line);
                 new_line->data->highlight = infolist_integer (infolist,
                                                               "highlight");
                 if (infolist_integer (infolist, "last_read_line"))
@@ -646,9 +656,12 @@ upgrade_weechat_read_buffer_line (struct t_infolist *infolist)
             }
             break;
         case GUI_BUFFER_TYPE_FREE:
-            gui_line_add_y (upgrade_current_buffer,
-                            infolist_integer (infolist, "y"),
-                            infolist_string (infolist, "message"));
+            new_line = gui_line_new (upgrade_current_buffer,
+                                     infolist_integer (infolist, "y"),
+                                     0, 0, NULL, NULL,
+                                     infolist_string (infolist, "message"));
+            if (new_line)
+                gui_line_add_y (new_line);
             break;
         case GUI_BUFFER_NUM_TYPES:
             break;
@@ -854,6 +867,8 @@ upgrade_weechat_load ()
     gui_color_buffer_assign ();
     gui_color_buffer_display ();
 
+    gui_buffer_user_set_callbacks ();
+
     secure_buffer_assign ();
     secure_buffer_display ();
 
