@@ -1,7 +1,7 @@
 /*
  * wee-util.c - some useful functions
  *
- * Copyright (C) 2003-2017 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -16,9 +16,17 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
+/* for P_tmpdir in stdio.h */
+#ifndef __USE_XOPEN
+#define __USE_XOPEN
+#endif
+
+/* for nftw() */
+#define _XOPEN_SOURCE 700
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -39,6 +47,7 @@
 #include <signal.h>
 #include <ctype.h>
 #include <time.h>
+#include <ftw.h>
 
 #include "weechat.h"
 #include "wee-util.h"
@@ -48,6 +57,7 @@
 #include "wee-utf8.h"
 #include "../gui/gui-chat.h"
 #include "../gui/gui-window.h"
+#include "../plugins/plugin.h"
 
 
 #ifdef HAVE_SYS_RESOURCE_H
@@ -143,8 +153,8 @@ util_setrlimit_resource (const char *resource_name, long limit)
             if (limit < -1)
             {
                 gui_chat_printf (NULL,
-                                 _("%sError: invalid limit for resource \"%s\": "
-                                   "%s (must be >= -1)"),
+                                 _("%sInvalid limit for resource \"%s\": %s "
+                                   "(must be >= -1)"),
                                  gui_chat_prefix[GUI_CHAT_PREFIX_ERROR],
                                  resource_name, str_limit);
                 return;
@@ -166,8 +176,8 @@ util_setrlimit_resource (const char *resource_name, long limit)
             else
             {
                 gui_chat_printf (NULL,
-                                 _("%sError: unable to set resource limit "
-                                   "\"%s\" to %s: error %d %s"),
+                                 _("%sUnable to set resource limit \"%s\" to "
+                                   "%s: error %d %s"),
                                  gui_chat_prefix[GUI_CHAT_PREFIX_ERROR],
                                  resource_name, str_limit,
                                  errno, strerror (errno));
@@ -177,7 +187,7 @@ util_setrlimit_resource (const char *resource_name, long limit)
     }
 
     gui_chat_printf (NULL,
-                     _("%sError: unknown resource limit \"%s\" (see /help "
+                     _("%sUnknown resource limit \"%s\" (see /help "
                        "weechat.startup.sys_rlimit)"),
                      gui_chat_prefix[GUI_CHAT_PREFIX_ERROR],
                      resource_name);
@@ -196,8 +206,11 @@ util_setrlimit ()
     int num_items, i;
     long number;
 
-    items = string_split (CONFIG_STRING(config_startup_sys_rlimit), ",", 0, 0,
-                          &num_items);
+    items = string_split (CONFIG_STRING(config_startup_sys_rlimit), ",", NULL,
+                          WEECHAT_STRING_SPLIT_STRIP_LEFT
+                          | WEECHAT_STRING_SPLIT_STRIP_RIGHT
+                          | WEECHAT_STRING_SPLIT_COLLAPSE_SEPS,
+                          0, &num_items);
     if (items)
     {
         for (i = 0; i < num_items; i++)
@@ -215,8 +228,8 @@ util_setrlimit ()
                 else
                 {
                     gui_chat_printf (NULL,
-                                     _("%sError: invalid limit for resource "
-                                       "\"%s\": %s (must be >= -1)"),
+                                     _("%sInvalid limit for resource \"%s\": "
+                                       "%s (must be >= -1)"),
                                      gui_chat_prefix[GUI_CHAT_PREFIX_ERROR],
                                      items[i], pos + 1);
                 }
@@ -325,6 +338,107 @@ util_get_time_string (const time_t *date)
     return text_time;
 }
 
+/*
+ * Returns difference between two times.
+ *
+ * The following variables are set, if pointer is not NULL:
+ *   - number of total seconds between the two times (basic subtraction)
+ *   - number of days/hours/minutes/seconds between the two times
+ */
+
+void
+util_get_time_diff (time_t time1, time_t time2,
+                    time_t *total_seconds,
+                    int *days, int *hours, int *minutes, int *seconds)
+{
+    time_t diff;
+
+    diff = time2 - time1;
+
+    if (total_seconds)
+        *total_seconds = diff;
+    if (days)
+        *days = diff / (60 * 60 * 24);
+    if (hours)
+        *hours = (diff % (60 * 60 * 24)) / (60 * 60);
+    if (minutes)
+        *minutes = ((diff % (60 * 60 * 24)) % (60 * 60)) / 60;
+    if (seconds)
+        *seconds = ((diff % (60 * 60 * 24)) % (60 * 60)) % 60;
+}
+
+/*
+ * Parses a string with a delay and optional unit, returns the delay in
+ * milliseconds.
+ *
+ * The delay is a number followed by a unit which can be:
+ *   - "ms": milliseconds
+ *   - "s": seconds
+ *   - "m": minutes
+ *   - "h": hours
+ *
+ * The default factor sets the default unit:
+ *   - 1: milliseconds
+ *   - 1000: seconds
+ *   - 60000: minutes
+ *   - 3600000: hours
+ *
+ * Returns the delay in milliseconds, -1 if error.
+ */
+
+long
+util_parse_delay (const char *string_delay, long default_factor)
+{
+    const char *pos;
+    char *str_number, *error;
+    long factor, delay;
+
+    if (!string_delay || !string_delay[0] || (default_factor < 1))
+        return -1;
+
+    factor = default_factor;
+
+    pos = string_delay;
+    while (pos[0] && isdigit ((unsigned char)pos[0]))
+    {
+        pos++;
+    }
+
+    if ((pos > string_delay) && pos[0])
+    {
+        str_number = string_strndup (string_delay, pos - string_delay);
+        if (strcmp (pos, "ms") == 0)
+            factor = 1;
+        else if (strcmp (pos, "s") == 0)
+            factor = 1000;
+        else if (strcmp (pos, "m") == 0)
+            factor = 1000 * 60;
+        else if (strcmp (pos, "h") == 0)
+            factor = 1000 * 60 * 60;
+        else
+            return -1;
+    }
+    else
+    {
+        str_number = strdup (string_delay);
+    }
+
+    if (!str_number)
+        return -1;
+
+    error = NULL;
+    delay = strtol (str_number, &error, 10);
+    if (!error || error[0] || (delay < 0))
+    {
+        free (str_number);
+        return -1;
+    }
+
+    free (str_number);
+
+    return delay * factor;
+}
+
 /*
  * Gets a signal number with a name; only some commonly used signal names are
  * supported here (see declaration of util_signals[]).
@@ -337,6 +451,9 @@ util_signal_search (const char *name)
 {
     int i;
 
+    if (!name)
+        return -1;
+
     for (i = 0; util_signals[i].name; i++)
     {
         if (string_strcasecmp (util_signals[i].name, name) == 0)
@@ -347,6 +464,28 @@ util_signal_search (const char *name)
     return -1;
 }
 
+/*
+ * Gets a signal name with a signal number; only some commonly used signal
+ * names are supported here (see declaration of util_signals[]).
+ *
+ * Returns the pointer to the signal name, NULL if not found.
+ */
+
+const char *
+util_signal_search_number (int signal_number)
+{
+    int i;
+
+    for (i = 0; util_signals[i].name; i++)
+    {
+        if (util_signals[i].signal == signal_number)
+            return util_signals[i].name;
+    }
+
+    /* signal not found */
+    return NULL;
+}
+
 /*
  * Catches a system signal.
  */
@@ -359,7 +498,50 @@ util_catch_signal (int signum, void (*handler)(int))
     sigemptyset (&act.sa_mask);
     act.sa_flags = 0;
     act.sa_handler = handler;
-    sigaction(signum, &act, NULL);
+    sigaction (signum, &act, NULL);
+}
+
+/*
+ * Returns the path to a temporary directory, the first valid directory in
+ * this list:
+ *   - content of environment variable "TMPDIR"
+ *   - P_tmpdir (from stdio.h)
+ *   - content of environment variable "HOME" (user home directory)
+ *   - "." (current directory)
+ */
+
+char *
+util_get_temp_dir()
+{
+    char *tmpdir;
+    struct stat buf;
+    int rc;
+
+    /* get directory from $TMPDIR */
+    tmpdir = getenv ("TMPDIR");
+    if (tmpdir && tmpdir[0])
+    {
+        rc = stat (tmpdir, &buf);
+        if ((rc == 0) && S_ISDIR(buf.st_mode))
+            return strdup (tmpdir);
+    }
+
+    /* get directory from P_tmpdir */
+    rc = stat (P_tmpdir, &buf);
+    if ((rc == 0) && S_ISDIR(buf.st_mode))
+        return strdup (P_tmpdir);
+
+    /* get directory from $HOME */
+    tmpdir = getenv ("HOME");
+    if (tmpdir && tmpdir[0])
+    {
+        rc = stat (tmpdir, &buf);
+        if ((rc == 0) && S_ISDIR(buf.st_mode))
+            return strdup (tmpdir);
+    }
+
+    /* fallback on current directory */
+    return strdup (".");
 }
 
 /*
@@ -482,6 +664,47 @@ util_mkdir_parents (const char *directory, int mode)
     return 1;
 }
 
+/*
+ * Unlinks a file or directory; callback called by function util_rmtree().
+ *
+ * Returns the return code of remove():
+ *   0: OK
+ *   -1: error
+ */
+
+int
+util_unlink_cb (const char *fpath, const struct stat *sb, int typeflag,
+                struct FTW *ftwbuf)
+{
+    /* make C compiler happy */
+    (void) sb;
+    (void) typeflag;
+    (void) ftwbuf;
+
+    return remove (fpath);
+}
+
+/*
+ * Removes a directory and all files inside recursively.
+ *
+ * Returns:
+ *   1: OK
+ *   0: error
+ */
+
+int
+util_rmtree (const char *directory)
+{
+    int rc;
+
+    if (!directory)
+        return 0;
+
+    rc = nftw (directory, util_unlink_cb, 64, FTW_DEPTH | FTW_PHYS);
+
+    return (rc == 0) ? 1 : 0;
+}
+
 /*
  * Finds files in a directory and executes a function on each file.
  */
@@ -562,7 +785,7 @@ util_search_full_lib_name_ext (const char *filename, const char *extension,
     {
         length = strlen (extra_libdir) + strlen (name_with_ext) +
             strlen (plugins_dir) + 16;
-        final_name = malloc(length);
+        final_name = malloc (length);
         if (!final_name)
         {
             free (name_with_ext);
@@ -719,7 +942,7 @@ util_file_get_content (const char *filename)
         if (!buffer2)
             goto error;
         buffer = buffer2;
-        count = fread (&buffer[fp], sizeof(char), 1024, f);
+        count = fread (&buffer[fp], sizeof (char), 1024, f);
         if (count <= 0)
             goto error;
         fp += count;
@@ -739,7 +962,7 @@ error:
     if (buffer)
 	free (buffer);
     if (f)
-	fclose(f);
+	fclose (f);
     return NULL;
 }
 
@@ -765,7 +988,11 @@ util_version_number (const char *version)
     int num_items, i, version_int[4], index_buf;
     long number;
 
-    items = string_split (version, ".", 0, 4, &num_items);
+    items = string_split (version, ".", NULL,
+                          WEECHAT_STRING_SPLIT_STRIP_LEFT
+                          | WEECHAT_STRING_SPLIT_STRIP_RIGHT
+                          | WEECHAT_STRING_SPLIT_COLLAPSE_SEPS,
+                          4, &num_items);
     for (i = 0; i < 4; i++)
     {
         version_int[i] = 0;
@@ -806,27 +1033,3 @@ util_version_number (const char *version)
     return (version_int[0] << 24) | (version_int[1] << 16)
         | (version_int[2] << 8) | version_int[3];
 }
-
-/*
- * Return uptime as number of days, hours, minutes, seconds.
- */
-
-void
-util_get_uptime (time_t *total_seconds, int *days,
-                 int *hours, int *minutes, int *seconds)
-{
-    time_t running_time;
-
-    running_time = time (NULL) - weechat_first_start_time;
-
-    if (total_seconds)
-        *total_seconds = running_time;
-    if (days)
-        *days = running_time / (60 * 60 * 24);
-    if (hours)
-        *hours = (running_time % (60 * 60 * 24)) / (60 * 60);
-    if (minutes)
-        *minutes = ((running_time % (60 * 60 * 24)) % (60 * 60)) / 60;
-    if (seconds)
-        *seconds = ((running_time % (60 * 60 * 24)) % (60 * 60)) % 60;
-}
