@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2017 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -14,7 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef WEECHAT_UTIL_H
@@ -36,16 +36,38 @@ struct t_util_signal
     int signal;                        /* signal number                     */
 };
 
+extern struct t_util_signal util_signals[];
+
+/* limits */
 extern void util_setrlimit ();
+
+/* timeval */
 extern int util_timeval_cmp (struct timeval *tv1, struct timeval *tv2);
 extern long long util_timeval_diff (struct timeval *tv1, struct timeval *tv2);
 extern void util_timeval_add (struct timeval *tv, long long interval);
+
+/* time */
 extern const char *util_get_time_string (const time_t *date);
+extern void util_get_time_diff (time_t time1, time_t time2,
+                                time_t *total_seconds,
+                                int *days, int *hours, int *minutes,
+                                int *seconds);
+
+/* delay */
+
+extern long util_parse_delay (const char *string_delay, long default_factor);
+
+/* signal */
 extern int util_signal_search (const char *name);
+extern const char *util_signal_search_number (int signal_number);
 extern void util_catch_signal (int signum, void (*handler)(int));
+
+/* files/directories */
+extern char *util_get_temp_dir();
 extern int util_mkdir_home (const char *directory, int mode);
 extern int util_mkdir (const char *directory, int mode);
 extern int util_mkdir_parents (const char *directory, int mode);
+extern int util_rmtree (const char *directory);
 extern void util_exec_on_files (const char *directory, int recurse_subdirs,
                                 int hidden_files,
                                 void (*callback)(void *data,
@@ -54,8 +76,8 @@ extern void util_exec_on_files (const char *directory, int recurse_subdirs,
 extern char *util_search_full_lib_name (const char *filename,
                                         const char *sys_directory);
 extern char *util_file_get_content (const char *filename);
+
+/* version */
 extern int util_version_number (const char *version);
-extern void util_get_uptime (time_t *total_seconds, int *days,
-                             int *hours, int *minutes, int *seconds);
 
 #endif /* WEECHAT_UTIL_H */
