@@ -17,7 +17,7 @@
  *
  * weechat.c - WeeChat main functions
  *
- * Copyright (C) 2003-2019 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -39,10 +39,12 @@
 #include "config.h"
 #endif
 
+#include <unistd.h>
 #include <errno.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#include <getopt.h>
 #include <sys/stat.h>
 #include <sys/time.h>
 #include <time.h>
@@ -74,14 +76,21 @@
 #include "../gui/gui-chat.h"
 #include "../gui/gui-color.h"
 #include "../gui/gui-completion.h"
+#include "../gui/gui-focus.h"
 #include "../gui/gui-key.h"
 #include "../gui/gui-layout.h"
 #include "../gui/gui-main.h"
 #include "../plugins/plugin.h"
 #include "../plugins/plugin-api.h"
 
+/* debug command line options */
+#define OPTION_NO_DLCLOSE 1000
+#define OPTION_NO_GNUTLS  1001
+#define OPTION_NO_GCRYPT  1002
 
 int weechat_headless = 0;              /* 1 if running headless (no GUI)    */
+int weechat_daemon = 0;                /* 1 if daemonized (no foreground)   */
+int weechat_log_stdout = 0;            /* 1 to log messages on stdout       */
 int weechat_debug_core = 0;            /* debug level for core              */
 char *weechat_argv0 = NULL;            /* WeeChat binary file name (argv[0])*/
 int weechat_upgrading = 0;             /* =1 if WeeChat is upgrading        */
@@ -93,6 +102,8 @@ struct timeval weechat_current_start_timeval; /* start time used to display */
 volatile sig_atomic_t weechat_quit = 0;   /* = 1 if quit request from user  */
 volatile sig_atomic_t weechat_quit_signal = 0; /* signal received,          */
                                        /* WeeChat must quit                 */
+volatile sig_atomic_t weechat_reload_signal = 0; /* signal received,        */
+                                       /* WeeChat must reload configuration */
 char *weechat_home = NULL;             /* home dir. (default: ~/.weechat)   */
 int weechat_home_temp = 0;             /* 1 if using a temporary home       */
 int weechat_home_delete_on_exit = 0;   /* 1 if home is deleted on exit      */
@@ -108,6 +119,7 @@ int weechat_no_gcrypt = 0;             /* remove init/deinit of gcrypt      */
                                        /* (useful with valgrind)            */
 struct t_weelist *weechat_startup_commands = NULL; /* startup commands      */
                                                    /* (option -r)           */
+int weechat_auto_connect = 1;          /* auto-connect to servers           */
 
 
 /*
@@ -186,8 +198,26 @@ weechat_display_usage ()
             stdout,
             _("                           (by default in headless mode "
               "WeeChat is blocking and does not run in background)\n"));
+        string_fprintf (
+            stdout,
+            _("      --stdout             display log messages on standard "
+              "output instead of writing them in log file\n"));
+        string_fprintf (
+            stdout,
+            _("                           (option ignored if option "
+              "\"--daemon\" is given)\n"));
         string_fprintf (stdout, "\n");
     }
+
+    /* debug options */
+    string_fprintf (
+        stdout,
+        _("Debug options (for tools like valgrind, DO NOT USE IN PRODUCTION):\n"
+          "      --no-dlclose         do not call function dlclose after "
+          "plugins are unloaded\n"
+          "      --no-gnutls          disable init/deinit of gnutls\n"
+          "      --no-gcrypt          disable init/deinit of gcrypt\n"));
+    string_fprintf (stdout, "\n");
 }
 
 /*
@@ -199,7 +229,27 @@ weechat_display_usage ()
 void
 weechat_parse_args (int argc, char *argv[])
 {
-    int i;
+    int opt;
+    struct option long_options[] = {
+        /* standard options */
+        { "no-connect",  no_argument,       NULL, 'a'  },
+        { "colors",      no_argument,       NULL, 'c'  },
+        { "dir",         required_argument, NULL, 'd'  },
+        { "temp-dir",    no_argument,       NULL, 't'  },
+        { "help",        no_argument,       NULL, 'h'  },
+        { "license",     no_argument,       NULL, 'l'  },
+        { "no-plugin",   no_argument,       NULL, 'p'  },
+        { "plugins",     required_argument, NULL, 'P'  },
+        { "run-command", required_argument, NULL, 'r'  },
+        { "no-script",   no_argument,       NULL, 's'  },
+        { "upgrade",     no_argument,       NULL, 'u'  },
+        { "version",     no_argument,       NULL, 'v'  },
+        /* debug options */
+        { "no-dlclose",  no_argument,       NULL, OPTION_NO_DLCLOSE },
+        { "no-gnutls",   no_argument,       NULL, OPTION_NO_GNUTLS  },
+        { "no-gcrypt",   no_argument,       NULL, OPTION_NO_GCRYPT  },
+        { NULL,          0,                 NULL, 0 },
+    };
 
     weechat_argv0 = (argv[0]) ? strdup (argv[0]) : NULL;
     weechat_upgrading = 0;
@@ -210,140 +260,114 @@ weechat_parse_args (int argc, char *argv[])
     weechat_force_plugin_autoload = NULL;
     weechat_plugin_no_dlclose = 0;
 
-    for (i = 1; i < argc; i++)
+    optind = 0;
+    opterr = 0;
+
+    while ((opt = getopt_long (argc, argv,
+                               ":acd:thlpP:r:sv",
+                               long_options, NULL)) != -1)
     {
-        if ((strcmp (argv[i], "-c") == 0)
-            || (strcmp (argv[i], "--colors") == 0))
+        switch (opt)
         {
-            gui_color_display_terminal_colors ();
-            weechat_shutdown (EXIT_SUCCESS, 0);
-        }
-        else if ((strcmp (argv[i], "-d") == 0)
-                 || (strcmp (argv[i], "--dir") == 0))
-        {
-            if (weechat_home_temp)
+            case 'a': /* -a / --no-connect */
+                /* option ignored, it will be used by plugins/scripts */
+                break;
+            case 'c': /* -c / --colors */
+                gui_color_display_terminal_colors ();
+                weechat_shutdown (EXIT_SUCCESS, 0);
+                break;
+            case 'd': /* -d / --dir */
                 weechat_home_temp = 0;
-            if (i + 1 < argc)
-            {
                 if (weechat_home)
                     free (weechat_home);
-                weechat_home = strdup (argv[++i]);
-            }
-            else
-            {
-                string_fprintf (stderr,
-                                _("Error: missing argument for \"%s\" option\n"),
-                                argv[i]);
-                weechat_shutdown (EXIT_FAILURE, 0);
-            }
-        }
-        else if ((strcmp (argv[i], "-t") == 0)
-                 || (strcmp (argv[i], "--temp-dir") == 0))
-        {
-            if (weechat_home)
-            {
-                free (weechat_home);
-                weechat_home = NULL;
-            }
-            weechat_home_temp = 1;
-        }
-        else if ((strcmp (argv[i], "-h") == 0)
-                || (strcmp (argv[i], "--help") == 0))
-        {
-            weechat_display_usage ();
-            weechat_shutdown (EXIT_SUCCESS, 0);
-        }
-        else if ((strcmp (argv[i], "-l") == 0)
-                 || (strcmp (argv[i], "--license") == 0))
-        {
-            weechat_display_copyright ();
-            string_fprintf (stdout, "\n");
-            string_fprintf (stdout, "%s%s", WEECHAT_LICENSE_TEXT);
-            weechat_shutdown (EXIT_SUCCESS, 0);
-        }
-        else if (strcmp (argv[i], "--no-dlclose") == 0)
-        {
-            /*
-             * Valgrind works better when dlclose() is not done after plugins
-             * are unloaded, it can display stack for plugins,* otherwise
-             * you'll see "???" in stack for functions of unloaded plugins.
-             * This option disables the call to dlclose(),
-             * it must NOT be used for other purposes!
-             */
-            weechat_plugin_no_dlclose = 1;
-        }
-        else if (strcmp (argv[i], "--no-gnutls") == 0)
-        {
-            /*
-             * Electric-fence is not working fine when gnutls loads
-             * certificates and Valgrind reports many memory errors with
-             * gnutls.
-             * This option disables the init/deinit of gnutls,
-             * it must NOT be used for other purposes!
-             */
-            weechat_no_gnutls = 1;
-        }
-        else if (strcmp (argv[i], "--no-gcrypt") == 0)
-        {
-            /*
-             * Valgrind reports many memory errors with gcrypt.
-             * This option disables the init/deinit of gcrypt,
-             * it must NOT be used for other purposes!
-             */
-            weechat_no_gcrypt = 1;
-        }
-        else if ((strcmp (argv[i], "-p") == 0)
-                 || (strcmp (argv[i], "--no-plugin") == 0))
-        {
-            if (weechat_force_plugin_autoload)
-                free (weechat_force_plugin_autoload);
-            weechat_force_plugin_autoload = strdup ("!*");
-        }
-        else if ((strcmp (argv[i], "-P") == 0)
-                 || (strcmp (argv[i], "--plugins") == 0))
-        {
-            if (i + 1 < argc)
-            {
+                weechat_home = strdup (optarg);
+                break;
+            case 't': /* -t / --temp-dir */
+                weechat_home_temp = 1;
+                if (weechat_home)
+                {
+                    free (weechat_home);
+                    weechat_home = NULL;
+                }
+                break;
+            case 'h': /* -h / --help */
+                weechat_display_usage ();
+                weechat_shutdown (EXIT_SUCCESS, 0);
+                break;
+            case 'l': /* -l / --license */
+                weechat_display_copyright ();
+                string_fprintf (stdout, "\n");
+                string_fprintf (stdout, "%s%s", WEECHAT_LICENSE_TEXT);
+                weechat_shutdown (EXIT_SUCCESS, 0);
+                break;
+            case 'p': /* -p / --no-plugin */
                 if (weechat_force_plugin_autoload)
                     free (weechat_force_plugin_autoload);
-                weechat_force_plugin_autoload = strdup (argv[++i]);
-            }
-            else
-            {
-                string_fprintf (stderr,
-                                _("Error: missing argument for \"%s\" option\n"),
-                                argv[i]);
-                weechat_shutdown (EXIT_FAILURE, 0);
-            }
-        }
-        else if ((strcmp (argv[i], "-r") == 0)
-                 || (strcmp (argv[i], "--run-command") == 0))
-        {
-            if (i + 1 < argc)
-            {
+                weechat_force_plugin_autoload = strdup ("!*");
+                break;
+            case 'P': /* -P / --plugins */
+                if (weechat_force_plugin_autoload)
+                    free (weechat_force_plugin_autoload);
+                weechat_force_plugin_autoload = strdup (optarg);
+                break;
+            case 'r': /* -r / --run-command */
                 if (!weechat_startup_commands)
                     weechat_startup_commands = weelist_new ();
-                weelist_add (weechat_startup_commands, argv[++i],
+                weelist_add (weechat_startup_commands, optarg,
                              WEECHAT_LIST_POS_END, NULL);
-            }
-            else
-            {
+                break;
+            case 's': /* -s / --no-script */
+                /* option ignored, it will be used by the scripting plugins */
+                break;
+            case 'u': /* --upgrade */
+                weechat_upgrading = 1;
+                break;
+            case 'v': /* -v / --version */
+                string_fprintf (stdout, version_get_version ());
+                fprintf (stdout, "\n");
+                weechat_shutdown (EXIT_SUCCESS, 0);
+                break;
+            case OPTION_NO_DLCLOSE: /* --no-dlclose */
+                /*
+                 * Valgrind works better when dlclose() is not done after
+                 * plugins are unloaded, it can display stack for plugins,*
+                 * otherwise you'll see "???" in stack for functions of
+                 * unloaded plugins.
+                 * This option disables the call to dlclose(),
+                 * it must NOT be used for other purposes!
+                 */
+                weechat_plugin_no_dlclose = 1;
+                break;
+            case OPTION_NO_GNUTLS: /* --no-gnutls */
+                /*
+                 * Electric-fence is not working fine when gnutls loads
+                 * certificates and Valgrind reports many memory errors with
+                 * gnutls.
+                 * This option disables the init/deinit of gnutls,
+                 * it must NOT be used for other purposes!
+                 */
+                weechat_no_gnutls = 1;
+                break;
+            case OPTION_NO_GCRYPT: /* --no-gcrypt */
+                /*
+                 * Valgrind reports many memory errors with gcrypt.
+                 * This option disables the init/deinit of gcrypt,
+                 * it must NOT be used for other purposes!
+                 */
+                weechat_no_gcrypt = 1;
+                break;
+            case ':':
                 string_fprintf (stderr,
                                 _("Error: missing argument for \"%s\" option\n"),
-                                argv[i]);
+                                argv[optind - 1]);
                 weechat_shutdown (EXIT_FAILURE, 0);
-            }
-        }
-        else if (strcmp (argv[i], "--upgrade") == 0)
-        {
-            weechat_upgrading = 1;
-        }
-        else if ((strcmp (argv[i], "-v") == 0)
-                 || (strcmp (argv[i], "--version") == 0))
-        {
-            string_fprintf (stdout, version_get_version ());
-            fprintf (stdout, "\n");
-            weechat_shutdown (EXIT_SUCCESS, 0);
+                break;
+            case '?':
+                /* ignore any unknown option; plugins can use them */
+                break;
+            default:
+                /* ignore any other error */
+                break;
         }
     }
 }
@@ -665,13 +689,13 @@ weechat_locale_check ()
 }
 
 /*
- * Callback for system signal SIGHUP: quits WeeChat.
+ * Callback for system signal SIGHUP: reloads configuration.
  */
 
 void
 weechat_sighup ()
 {
-    weechat_quit_signal = SIGHUP;
+    weechat_reload_signal = SIGHUP;
 }
 
 /*
@@ -798,8 +822,10 @@ weechat_init (int argc, char *argv[], void (*gui_init_cb)())
 
     if (weechat_upgrading)
     {
-        upgrade_weechat_load ();        /* upgrade with session file        */
-        weechat_upgrade_count++;        /* increase /upgrade count          */
+        if (upgrade_weechat_load ())    /* upgrade with session file        */
+            weechat_upgrade_count++;    /* increase /upgrade count          */
+        else
+            weechat_upgrading = 0;
     }
     weechat_startup_message ();         /* display WeeChat startup message  */
     gui_chat_print_lines_waiting_buffer (NULL); /* display lines waiting    */
