@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2018 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
  * Copyright (C) 2006 Emmanuel Bouthenot <kolter@openics.org>
  *
  * This file is part of WeeChat, the extensible chat client.
@@ -15,7 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef WEECHAT_H
@@ -53,7 +53,7 @@
 #endif /* !defined(_) */
 
 
-#define WEECHAT_COPYRIGHT_DATE   "(C) 2003-2018"
+#define WEECHAT_COPYRIGHT_DATE   "(C) 2003-2021"
 #define WEECHAT_WEBSITE          "https://weechat.org/"
 #define WEECHAT_WEBSITE_DOWNLOAD "https://weechat.org/download"
 
@@ -74,7 +74,7 @@
     "GNU General Public License for more details.\n" \
     "\n" \
     "You should have received a copy of the GNU General Public License\n" \
-    "along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.\n\n"
+    "along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.\n\n"
 
 /* directory separator, depending on OS */
 #ifdef _WIN32
@@ -100,6 +100,8 @@ struct t_weelist;
 
 /* global variables and functions */
 extern int weechat_headless;
+extern int weechat_daemon;
+extern int weechat_log_stdout;
 extern int weechat_debug_core;
 extern char *weechat_argv0;
 extern int weechat_upgrading;
@@ -109,12 +111,15 @@ extern struct timeval weechat_current_start_timeval;
 extern int weechat_upgrade_count;
 extern volatile sig_atomic_t weechat_quit;
 extern volatile sig_atomic_t weechat_quit_signal;
+extern volatile sig_atomic_t weechat_reload_signal;
 extern char *weechat_home;
+extern int weechat_home_temp;
 extern char *weechat_local_charset;
 extern int weechat_plugin_no_dlclose;
 extern int weechat_no_gnutls;
 extern int weechat_no_gcrypt;
 extern struct t_weelist *weechat_startup_commands;
+extern int weechat_auto_connect;
 
 extern void weechat_term_check ();
 extern void weechat_shutdown (int return_code, int crash);
