@@ -1,7 +1,7 @@
 /*
  * gui-curses-color.c - color functions for Curses GUI
  *
- * Copyright (C) 2003-2019 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -115,7 +115,7 @@ int gui_color_timer = 0;                      /* timer in seconds           */
 /*
  * Searches for a color by name.
  *
- * Return index of color in WeeChat colors table, -1 if not found.
+ * Returns index of color in WeeChat colors table, -1 if not found.
  */
 
 int
@@ -133,12 +133,33 @@ gui_color_search (const char *color_name)
     return -1;
 }
 
+/*
+ * Searches for a color by index.
+ *
+ * Returns name of color in WeeChat colors table, NULL if not found.
+ */
+
+const char *
+gui_color_search_index (int index)
+{
+    int i;
+
+    for (i = 0; gui_weechat_colors[i].string; i++)
+    {
+        if (i == index)
+            return gui_weechat_colors[i].string;
+    }
+
+    /* color not found */
+    return NULL;
+}
+
 /*
  * Get Curses attributes corresponding to extended attributes flags in a color.
  */
 
 int
-gui_color_get_extended_attrs (int color)
+gui_color_get_gui_attrs (int color)
 {
     int attributes;
 
@@ -156,6 +177,29 @@ gui_color_get_extended_attrs (int color)
     return attributes;
 }
 
+/*
+ * Get extended flags corresponding to Curses attributes in a color.
+ */
+
+int
+gui_color_get_extended_flags (int attrs)
+{
+    int flags;
+
+    flags = 0;
+
+    if (attrs & A_BOLD)
+        flags |= GUI_COLOR_EXTENDED_BOLD_FLAG;
+    if (attrs & A_REVERSE)
+        flags |= GUI_COLOR_EXTENDED_REVERSE_FLAG;
+    if (attrs & A_ITALIC)
+        flags |= GUI_COLOR_EXTENDED_ITALIC_FLAG;
+    if (attrs & A_UNDERLINE)
+        flags |= GUI_COLOR_EXTENDED_UNDERLINE_FLAG;
+
+    return flags;
+}
+
 /*
  * Assigns a WeeChat color (read from configuration).
  *
@@ -521,6 +565,8 @@ gui_color_get_name (int num_color)
 void
 gui_color_build (int number, int foreground, int background)
 {
+    char str_color[128];
+
     if (foreground < 0)
         foreground = 0;
     if (background < 0)
@@ -532,7 +578,6 @@ gui_color_build (int number, int foreground, int background)
         gui_color[number] = malloc (sizeof (*gui_color[number]));
         if (!gui_color[number])
             return;
-        gui_color[number]->string = malloc (4);
     }
 
     /* set foreground and attributes */
@@ -546,7 +591,7 @@ gui_color_build (int number, int foreground, int background)
         gui_color[number]->foreground = gui_weechat_colors[foreground & GUI_COLOR_EXTENDED_MASK].foreground;
         gui_color[number]->attributes = gui_weechat_colors[foreground & GUI_COLOR_EXTENDED_MASK].attributes;
     }
-    gui_color[number]->attributes |= gui_color_get_extended_attrs (foreground);
+    gui_color[number]->attributes |= gui_color_get_gui_attrs (foreground);
 
     /* set background */
     if (background & GUI_COLOR_EXTENDED_FLAG)
@@ -555,12 +600,10 @@ gui_color_build (int number, int foreground, int background)
         gui_color[number]->background = gui_weechat_colors[background & GUI_COLOR_EXTENDED_MASK].background;
 
     /* set string */
-    if (gui_color[number]->string)
-    {
-        snprintf (gui_color[number]->string, 4,
-                  "%c%02d",
-                  GUI_COLOR_COLOR_CHAR, number);
-    }
+    snprintf (str_color, sizeof (str_color),
+              "%c%02d",
+              GUI_COLOR_COLOR_CHAR, number);
+    gui_color[number]->string = strdup (str_color);
 }
 
 /*
@@ -570,8 +613,6 @@ gui_color_build (int number, int foreground, int background)
 void
 gui_color_init_vars ()
 {
-    int size;
-
     gui_color_term_has_colors = (has_colors ()) ? 1 : 0;
     gui_color_term_colors = 0;
     gui_color_term_color_pairs = 0;
@@ -593,12 +634,9 @@ gui_color_init_vars ()
         /* TODO: ncurses may support 65536, but short type used for pairs supports only 32768? */
         gui_color_num_pairs = (gui_color_term_color_pairs >= 32768) ?
             32767 : gui_color_term_color_pairs - 1;
-        size = (gui_color_term_colors + 2)
-            * (gui_color_term_colors + 2)
-            * sizeof (gui_color_pairs[0]);
-        gui_color_pairs = malloc (size);
-        if (gui_color_pairs)
-            memset (gui_color_pairs, 0, size);
+        gui_color_pairs = calloc (
+            (gui_color_term_colors + 2) * (gui_color_term_colors + 2),
+            sizeof (gui_color_pairs[0]));
         gui_color_pairs_used = 0;
 
         /* reserved for future usage */
@@ -623,10 +661,7 @@ gui_color_init_vars ()
         gui_color_term_color_pairs = 1;
         gui_color_term_can_change_color = 0;
         gui_color_num_pairs = 1;
-        size = sizeof (gui_color_pairs[0]);
-        gui_color_pairs = malloc (size);
-        if (gui_color_pairs)
-            memset (gui_color_pairs, 0, size);
+        gui_color_pairs = calloc (1, sizeof (gui_color_pairs[0]));
         gui_color_pairs_used = 0;
     }
 }
