@@ -1,7 +1,7 @@
 /*
  * gui-curses-main.c - main loop for Curses GUI
  *
- * Copyright (C) 2003-2014 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -60,7 +60,8 @@
 #include "gui-curses.h"
 
 
-int gui_signal_sigwinch_received = 0;  /* sigwinch signal (term resized)    */
+volatile sig_atomic_t gui_signal_sigwinch_received = 0;  /* sigwinch signal */
+                                       /* (terminal has been resized)       */
 int gui_term_cols = 0;                 /* number of columns in terminal     */
 int gui_term_lines = 0;                /* number of lines in terminal       */
 
@@ -73,45 +74,64 @@ int gui_term_lines = 0;                /* number of lines in terminal       */
  */
 
 void
-gui_main_get_password (const char *prompt1, const char *prompt2,
-                       const char *prompt3,
-                       char *password, int size)
+gui_main_get_password (const char **prompt, char *password, int size)
 {
-    int i, ch;
+    int line, i, ch;
+
+    memset (password, '\0', size);
+
+    if (weechat_headless)
+    {
+        password[0] = ' ';
+        return;
+    }
 
     initscr ();
     cbreak ();
     noecho ();
+    raw ();
 
-    clear();
+    clear ();
 
-    mvaddstr (0, 0, prompt1);
-    mvaddstr (1, 0, prompt2);
-    mvaddstr (2, 0, prompt3);
-    mvaddstr (3, 0, "=> ");
+    line = 0;
+
+    while (prompt[line])
+    {
+        mvaddstr (line, 0, prompt[line]);
+        line++;
+    }
+
+    mvaddstr (line, 0, "=> ");
     refresh ();
 
-    memset (password, '\0', size);
     i = 0;
     while (i < size - 1)
     {
         ch = getch ();
+        /* enter */
         if (ch == '\n')
             break;
+        /* terminal lost or ctrl-C */
+        if ((ch == ERR) || (ch == 3))
+        {
+            password[0] = 3;
+            i = 1;
+            break;
+        }
         if (ch == 127)
         {
             if (i > 0)
             {
                 i--;
                 password[i] = '\0';
-                mvaddstr (3, 3 + i, " ");
-                move (3, 3 + i);
+                mvaddstr (line, 3 + i, " ");
+                move (line, 3 + i);
             }
         }
         else
         {
             password[i] = ch;
-            mvaddstr (3, 3 + i, "*");
+            mvaddstr (line, 3 + i, "*");
             i++;
         }
         refresh ();
@@ -122,6 +142,16 @@ gui_main_get_password (const char *prompt1, const char *prompt2,
     endwin ();
 }
 
+/*
+ * Callback for system signal SIGINT: quits WeeChat.
+ */
+
+void
+gui_main_signal_sigint ()
+{
+    weechat_quit = 1;
+}
+
 /*
  * Initializes GUI.
  */
@@ -134,6 +164,11 @@ gui_main_init ()
     struct t_gui_bar_window *ptr_bar_win;
     char title[256];
 
+#ifdef WEECHAT_HEADLESS
+    /* allow Ctrl-C to quit WeeChat in headless mode */
+    util_catch_signal (SIGINT, &gui_main_signal_sigint);
+#endif /* WEECHAT_HEADLESS */
+
     initscr ();
 
     if (CONFIG_BOOLEAN(config_look_eat_newline_glitch))
@@ -169,7 +204,8 @@ gui_main_init ()
 
     /* create core buffer */
     ptr_buffer = gui_buffer_new (NULL, GUI_BUFFER_MAIN,
-                                 NULL, NULL, NULL, NULL);
+                                 NULL, NULL, NULL,
+                                 NULL, NULL, NULL);
     if (ptr_buffer)
     {
         gui_init_ok = 1;
@@ -228,6 +264,30 @@ gui_main_init ()
     gui_window_set_bracketed_paste_mode (CONFIG_BOOLEAN(config_look_paste_bracketed));
 }
 
+/*
+ * Returns signal name with a signal number.
+ *
+ * Note: result must be freed after use.
+ */
+
+char *
+gui_main_get_signal_name (int signal_number)
+{
+    const char *signal_name;
+    char str_signal[32];
+
+    signal_name = util_signal_search_number (signal_number);
+    if (!signal_name)
+        return NULL;
+
+    snprintf (str_signal, sizeof (str_signal),
+              "SIG%s",
+              signal_name);
+    string_toupper (str_signal);
+
+    return strdup (str_signal);
+}
+
 /*
  * Callback for system signal SIGWINCH: refreshes screen.
  */
@@ -236,10 +296,84 @@ void
 gui_main_signal_sigwinch ()
 {
     gui_signal_sigwinch_received = 1;
-
-    gui_window_ask_refresh (2);
 }
 
+/*
+ * Sends a WeeChat signal on a system signal received.
+ *
+ * Returns:
+ *   WEECHAT_RC_OK: the WeeChat handler must be executed
+ *   WEECHAT_RC_OK_EAT: signal eaten, the WeeChat handler must NOT be executed
+ */
+
+int
+gui_main_handle_signal (const char *signal_name)
+{
+    int rc;
+    char str_signal[32];
+
+    if (!signal_name)
+        return WEECHAT_RC_OK;
+
+    snprintf (str_signal, sizeof (str_signal), "signal_%s", signal_name);
+    string_tolower (str_signal);
+
+    rc = hook_signal_send (str_signal, WEECHAT_HOOK_SIGNAL_STRING, NULL);
+
+    return (rc == WEECHAT_RC_OK_EAT) ? WEECHAT_RC_OK_EAT : WEECHAT_RC_OK;
+}
+
+/*
+ * Callback for signals received that will make WeeChat reload configuration.
+ */
+
+void
+gui_main_handle_reload_signal ()
+{
+    char *signal_name;
+
+    signal_name = gui_main_get_signal_name (weechat_reload_signal);
+
+    if (gui_main_handle_signal (signal_name) != WEECHAT_RC_OK_EAT)
+    {
+        log_printf (_("Signal %s received, reloading configuration..."),
+                    signal_name);
+        command_reload_files ();
+    }
+
+    if (signal_name)
+        free (signal_name);
+
+    weechat_reload_signal = 0;
+}
+
+/*
+ * Callback for signals received that will make WeeChat quit.
+ */
+
+void
+gui_main_handle_quit_signals ()
+{
+    char *signal_name;
+
+    signal_name = gui_main_get_signal_name (weechat_quit_signal);
+
+    if (gui_main_handle_signal (signal_name) != WEECHAT_RC_OK_EAT)
+    {
+        if (!weechat_quit)
+        {
+            log_printf (_("Signal %s received, exiting WeeChat..."),
+                        signal_name);
+            (void) hook_signal_send ("quit", WEECHAT_HOOK_SIGNAL_STRING, NULL);
+            weechat_quit = 1;
+        }
+    }
+
+    if (signal_name)
+        free (signal_name);
+
+    weechat_quit_signal = 0;
+}
 /*
  * Displays infos about ncurses lib.
  */
@@ -252,15 +386,15 @@ gui_main_debug_libs ()
                      NCURSES_VERSION, NCURSES_VERSION_PATCH);
 #else
     gui_chat_printf (NULL, "    ncurses: (?)");
-#endif
+#endif /* defined(NCURSES_VERSION) && defined(NCURSES_VERSION_PATCH) */
 }
 
 /*
- * Refreshs for windows, buffers, bars.
+ * Refreshes for windows, buffers, bars.
  */
 
 void
-gui_main_refreshs ()
+gui_main_refreshes ()
 {
     struct t_gui_window *ptr_win;
     struct t_gui_buffer *ptr_buffer;
@@ -370,23 +504,30 @@ void
 gui_main_loop ()
 {
     struct t_hook *hook_fd_keyboard;
-    struct timeval tv_timeout;
-    fd_set read_fds, write_fds, except_fds;
-    int max_fd;
-    int ready;
+    int send_signal_sigwinch;
+
+    send_signal_sigwinch = 0;
 
     /* catch SIGWINCH signal: redraw screen */
-    util_catch_signal (SIGWINCH, &gui_main_signal_sigwinch);
+    if (!weechat_headless)
+        util_catch_signal (SIGWINCH, &gui_main_signal_sigwinch);
 
     /* hook stdin (read keyboard) */
-    hook_fd_keyboard = hook_fd (NULL, STDIN_FILENO, 1, 0, 0,
-                                &gui_key_read_cb, NULL);
+    if (weechat_headless)
+    {
+        hook_fd_keyboard = NULL;
+    }
+    else
+    {
+        hook_fd_keyboard = hook_fd (NULL, STDIN_FILENO, 1, 0, 0,
+                                    &gui_key_read_cb, NULL, NULL);
+    }
 
     gui_window_ask_refresh (1);
 
     while (!weechat_quit)
     {
-        /* execute hook timers */
+        /* execute timer hooks */
         hook_timer_exec ();
 
         /* auto reset of color pairs */
@@ -398,35 +539,42 @@ gui_main_loop ()
             gui_color_pairs_auto_reset_pending = 1;
         }
 
-        gui_main_refreshs ();
-        if (gui_window_refresh_needed && !gui_window_bare_display)
-            gui_main_refreshs ();
-
         if (gui_signal_sigwinch_received)
+        {
+            gui_window_ask_refresh (2);
+            gui_signal_sigwinch_received = 0;
+            send_signal_sigwinch = 1;
+        }
+
+        gui_main_refreshes ();
+        if (gui_window_refresh_needed && !gui_window_bare_display)
+            gui_main_refreshes ();
+
+        if (send_signal_sigwinch)
         {
             (void) hook_signal_send ("signal_sigwinch",
                                      WEECHAT_HOOK_SIGNAL_STRING, NULL);
-            gui_signal_sigwinch_received = 0;
+            send_signal_sigwinch = 0;
         }
 
         gui_color_pairs_auto_reset_pending = 0;
 
-        /* wait for keyboard or network activity */
-        FD_ZERO (&read_fds);
-        FD_ZERO (&write_fds);
-        FD_ZERO (&except_fds);
-        max_fd = hook_fd_set (&read_fds, &write_fds, &except_fds);
-        hook_timer_time_to_next (&tv_timeout);
-        ready = select (max_fd + 1, &read_fds, &write_fds, &except_fds,
-                        &tv_timeout);
-        if (ready > 0)
-        {
-            hook_fd_exec (&read_fds, &write_fds, &except_fds);
-        }
+        /* execute fd hooks */
+        hook_fd_exec ();
+
+        /* run process (with fork) */
+        hook_process_exec ();
+
+        /* handle signals received */
+        if (weechat_reload_signal > 0)
+            gui_main_handle_reload_signal ();
+        if (weechat_quit_signal > 0)
+            gui_main_handle_quit_signals ();
     }
 
     /* remove keyboard hook */
-    unhook (hook_fd_keyboard);
+    if (hook_fd_keyboard)
+        unhook (hook_fd_keyboard);
 }
 
 /*
@@ -442,14 +590,14 @@ gui_main_end (int clean_exit)
     if (clean_exit)
     {
         /*
-         * final refreshs, to see messages just before exiting
+         * final refreshes, to see messages just before exiting
          * (if we are upgrading, don't refresh anything!)
          */
         if (!weechat_upgrading)
         {
-            gui_main_refreshs ();
+            gui_main_refreshes ();
             if (gui_window_refresh_needed)
-                gui_main_refreshs ();
+                gui_main_refreshes ();
         }
 
         /* disable bracketed paste mode */
