@@ -1,7 +1,7 @@
 /*
  * gui-bar.c - bar functions (used by all GUI)
  *
- * Copyright (C) 2003-2017 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -50,11 +50,11 @@
 char *gui_bar_option_string[GUI_BAR_NUM_OPTIONS] =
 { "hidden", "priority", "type", "conditions", "position", "filling_top_bottom",
   "filling_left_right", "size", "size_max", "color_fg", "color_delim",
-  "color_bg", "separator", "items" };
+  "color_bg", "color_bg_inactive", "separator", "items" };
 char *gui_bar_option_default[GUI_BAR_NUM_OPTIONS] =
 { "0", "0", "0", "", "top", "horizontal",
   "vertical", "0", "0", "default", "default",
-  "default", "off", "" };
+  "default", "default", "off", "" };
 char *gui_bar_type_string[GUI_BAR_NUM_TYPES] =
 { "root", "window" };
 char *gui_bar_position_string[GUI_BAR_NUM_POSITIONS] =
@@ -387,7 +387,7 @@ gui_bar_check_conditions (struct t_gui_bar *bar,
     snprintf (str_modifier, sizeof (str_modifier),
               "bar_condition_%s", bar->name);
     snprintf (str_window, sizeof (str_window),
-              "0x%lx", (long unsigned int)(window));
+              "0x%lx", (unsigned long)(window));
     str_displayed = hook_modifier_exec (NULL,
                                         str_modifier,
                                         str_window,
@@ -704,7 +704,11 @@ gui_bar_set_items_array (struct t_gui_bar *bar, const char *items)
 
     if (items && items[0])
     {
-        tmp_array = string_split (items, ",", 0, 0, &count);
+        tmp_array = string_split (items, ",", NULL,
+                                  WEECHAT_STRING_SPLIT_STRIP_LEFT
+                                  | WEECHAT_STRING_SPLIT_STRIP_RIGHT
+                                  | WEECHAT_STRING_SPLIT_COLLAPSE_SEPS,
+                                  0, &count);
         if (count > 0)
         {
             bar->items_count = count;
@@ -716,8 +720,15 @@ gui_bar_set_items_array (struct t_gui_bar *bar, const char *items)
             bar->items_suffix = malloc (count * sizeof (*bar->items_suffix));
             for (i = 0; i < count; i++)
             {
-                bar->items_array[i] = string_split (tmp_array[i], "+", 0, 0,
-                                                    &(bar->items_subcount[i]));
+                bar->items_array[i] = string_split (
+                    tmp_array[i],
+                    "+",
+                    NULL,
+                    WEECHAT_STRING_SPLIT_STRIP_LEFT
+                    | WEECHAT_STRING_SPLIT_STRIP_RIGHT
+                    | WEECHAT_STRING_SPLIT_COLLAPSE_SEPS,
+                    0,
+                    &(bar->items_subcount[i]));
                 if (bar->items_subcount[i] > 0)
                 {
                     bar->items_buffer[i] = malloc (bar->items_subcount[i] *
@@ -962,7 +973,7 @@ gui_bar_config_check_size (const void *pointer, void *data,
     struct t_gui_bar *ptr_bar;
     long number;
     char *error;
-    int new_value;
+    int new_value, current_size;
 
     /* make C compiler happy */
     (void) pointer;
@@ -1002,14 +1013,16 @@ gui_bar_config_check_size (const void *pointer, void *data,
         if (new_value < 0)
             return 0;
 
-        if ((new_value > 0) &&
-            ((CONFIG_INTEGER(ptr_bar->options[GUI_BAR_OPTION_SIZE]) == 0)
-             || (new_value > CONFIG_INTEGER(ptr_bar->options[GUI_BAR_OPTION_SIZE]))))
+        if (!CONFIG_BOOLEAN(ptr_bar->options[GUI_BAR_OPTION_HIDDEN]))
         {
-            if (!CONFIG_BOOLEAN(ptr_bar->options[GUI_BAR_OPTION_HIDDEN])
-                && !gui_bar_check_size_add (ptr_bar,
-                                            new_value - CONFIG_INTEGER(ptr_bar->options[GUI_BAR_OPTION_SIZE])))
+            current_size = CONFIG_INTEGER(ptr_bar->options[GUI_BAR_OPTION_SIZE]);
+            if ((current_size == 0) && ptr_bar->bar_window)
+                current_size = ptr_bar->bar_window->current_size;
+            if ((new_value > 0) && (new_value > current_size)
+                && !gui_bar_check_size_add (ptr_bar, new_value - current_size))
+            {
                 return 0;
+            }
         }
 
         return 1;
@@ -1174,6 +1187,8 @@ gui_bar_set_name (struct t_gui_bar *bar, const char *name)
         config_file_option_rename (bar->options[GUI_BAR_OPTION_COLOR_DELIM], option_name);
         snprintf (option_name, length, "%s.color_bg", name);
         config_file_option_rename (bar->options[GUI_BAR_OPTION_COLOR_BG], option_name);
+        snprintf (option_name, length, "%s.color_bg_inactive", name);
+        config_file_option_rename (bar->options[GUI_BAR_OPTION_COLOR_BG_INACTIVE], option_name);
         snprintf (option_name, length, "%s.separator", name);
         config_file_option_rename (bar->options[GUI_BAR_OPTION_SEPARATOR], option_name);
         snprintf (option_name, length, "%s.items", name);
@@ -1264,6 +1279,12 @@ gui_bar_set (struct t_gui_bar *bar, const char *property, const char *value)
         gui_bar_refresh (bar);
         return 1;
     }
+    else if (string_strcasecmp (property, "color_bg_inactive") == 0)
+    {
+        config_file_option_set (bar->options[GUI_BAR_OPTION_COLOR_BG_INACTIVE], value, 1);
+        gui_bar_refresh (bar);
+        return 1;
+    }
     else if (string_strcasecmp (property, "separator") == 0)
     {
         config_file_option_set (bar->options[GUI_BAR_OPTION_SEPARATOR],
@@ -1463,6 +1484,17 @@ gui_bar_create_option (const char *bar_name, int index_option, const char *value
                 &gui_bar_config_change_color, NULL, NULL,
                 NULL, NULL, NULL);
             break;
+        case GUI_BAR_OPTION_COLOR_BG_INACTIVE:
+            ptr_option = config_file_new_option (
+                weechat_config_file, weechat_config_section_bar,
+                option_name, "color",
+                N_("background color for a bar with type \"window\" which is "
+                   "not displayed in the active window"),
+                NULL, 0, 0, value, NULL, 0,
+                NULL, NULL, NULL,
+                &gui_bar_config_change_color, NULL, NULL,
+                NULL, NULL, NULL);
+            break;
         case GUI_BAR_OPTION_SEPARATOR:
             ptr_option = config_file_new_option (
                 weechat_config_file, weechat_config_section_bar,
@@ -1570,6 +1602,7 @@ gui_bar_new_with_options (const char *name,
                           struct t_config_option *color_fg,
                           struct t_config_option *color_delim,
                           struct t_config_option *color_bg,
+                          struct t_config_option *color_bg_inactive,
                           struct t_config_option *separator,
                           struct t_config_option *items)
 {
@@ -1593,6 +1626,7 @@ gui_bar_new_with_options (const char *name,
     new_bar->options[GUI_BAR_OPTION_COLOR_FG] = color_fg;
     new_bar->options[GUI_BAR_OPTION_COLOR_DELIM] = color_delim;
     new_bar->options[GUI_BAR_OPTION_COLOR_BG] = color_bg;
+    new_bar->options[GUI_BAR_OPTION_COLOR_BG_INACTIVE] = color_bg_inactive;
     new_bar->options[GUI_BAR_OPTION_SEPARATOR] = separator;
     new_bar->options[GUI_BAR_OPTION_ITEMS] = items;
     new_bar->items_count = 0;
@@ -1641,15 +1675,16 @@ gui_bar_new (const char *name, const char *hidden, const char *priority,
              const char *filling_top_bottom, const char *filling_left_right,
              const char *size, const char *size_max,
              const char *color_fg, const char *color_delim,
-             const char *color_bg, const char *separators, const char *items)
+             const char *color_bg, const char *color_bg_inactive,
+             const char *separators, const char *items)
 {
     struct t_config_option *option_hidden, *option_priority, *option_type;
     struct t_config_option *option_conditions, *option_position;
     struct t_config_option *option_filling_top_bottom, *option_filling_left_right;
     struct t_config_option *option_size, *option_size_max;
     struct t_config_option *option_color_fg, *option_color_delim;
-    struct t_config_option *option_color_bg, *option_separator;
-    struct t_config_option *option_items;
+    struct t_config_option *option_color_bg, *option_color_bg_inactive;
+    struct t_config_option *option_separator, *option_items;
     struct t_gui_bar *new_bar;
 
     if (!name || !name[0])
@@ -1691,20 +1726,24 @@ gui_bar_new (const char *name, const char *hidden, const char *priority,
                                                 color_delim);
     option_color_bg = gui_bar_create_option (name, GUI_BAR_OPTION_COLOR_BG,
                                              color_bg);
+    option_color_bg_inactive = gui_bar_create_option (name, GUI_BAR_OPTION_COLOR_BG_INACTIVE,
+                                                      color_bg_inactive);
     option_separator = gui_bar_create_option (name, GUI_BAR_OPTION_SEPARATOR,
                                               (config_file_string_to_boolean (separators)) ?
                                                "on" : "off");
     option_items = gui_bar_create_option (name, GUI_BAR_OPTION_ITEMS,
                                           items);
-    new_bar = gui_bar_new_with_options (name, option_hidden,
-                                        option_priority, option_type,
-                                        option_conditions, option_position,
-                                        option_filling_top_bottom,
-                                        option_filling_left_right,
-                                        option_size, option_size_max,
-                                        option_color_fg, option_color_delim,
-                                        option_color_bg, option_separator,
-                                        option_items);
+    new_bar = gui_bar_new_with_options (
+        name, option_hidden,
+        option_priority, option_type,
+        option_conditions, option_position,
+        option_filling_top_bottom,
+        option_filling_left_right,
+        option_size, option_size_max,
+        option_color_fg, option_color_delim,
+        option_color_bg, option_color_bg_inactive,
+        option_separator,
+        option_items);
     if (!new_bar)
     {
         if (option_hidden)
@@ -1781,6 +1820,7 @@ gui_bar_use_temp_bars ()
                                       ptr_temp_bar->options[GUI_BAR_OPTION_COLOR_FG],
                                       ptr_temp_bar->options[GUI_BAR_OPTION_COLOR_DELIM],
                                       ptr_temp_bar->options[GUI_BAR_OPTION_COLOR_BG],
+                                      ptr_temp_bar->options[GUI_BAR_OPTION_COLOR_BG_INACTIVE],
                                       ptr_temp_bar->options[GUI_BAR_OPTION_SEPARATOR],
                                       ptr_temp_bar->options[GUI_BAR_OPTION_ITEMS]);
         }
@@ -1860,11 +1900,12 @@ gui_bar_create_default_input ()
                              "bottom",     /* position */
                              "horizontal", /* filling_top_bottom */
                              "vertical",   /* filling_left_right */
-                             "1",          /* size */
+                             "0",          /* size */
                              "0",          /* size_max */
                              "default",    /* color fg */
                              "cyan",       /* color delim */
                              "default",    /* color bg */
+                             "default",    /* color bg inactive */
                              "0",          /* separators */
                              gui_bar_default_items (GUI_BAR_DEFAULT_NAME_INPUT))) /* items */
             {
@@ -1902,6 +1943,7 @@ gui_bar_create_default_title ()
                          "default",    /* color fg */
                          "cyan",       /* color delim */
                          "blue",       /* color bg */
+                         "darkgray",   /* color bg inactive */
                          "0",          /* separators */
                          gui_bar_default_items (GUI_BAR_DEFAULT_NAME_TITLE))) /* items */
         {
@@ -1938,6 +1980,7 @@ gui_bar_create_default_status ()
                          "default",    /* color fg */
                          "cyan",       /* color delim */
                          "blue",       /* color bg */
+                         "darkgray",   /* color bg inactive */
                          "0",          /* separators */
                          gui_bar_default_items (GUI_BAR_DEFAULT_NAME_STATUS))) /* items */
         {
@@ -1974,6 +2017,7 @@ gui_bar_create_default_nicklist ()
                          "default",          /* color fg */
                          "cyan",             /* color delim */
                          "default",          /* color bg */
+                         "default",          /* color bg inactive */
                          "1",                /* separators */
                          gui_bar_default_items (GUI_BAR_DEFAULT_NAME_NICKLIST))) /* items */
         {
@@ -2308,6 +2352,8 @@ gui_bar_add_to_infolist (struct t_infolist *infolist,
         return 0;
     if (!infolist_new_var_string (ptr_item, "color_bg", gui_color_get_name (CONFIG_COLOR(bar->options[GUI_BAR_OPTION_COLOR_BG]))))
         return 0;
+    if (!infolist_new_var_string (ptr_item, "color_bg_inactive", gui_color_get_name (CONFIG_COLOR(bar->options[GUI_BAR_OPTION_COLOR_BG_INACTIVE]))))
+        return 0;
     if (!infolist_new_var_integer (ptr_item, "separator", CONFIG_INTEGER(bar->options[GUI_BAR_OPTION_SEPARATOR])))
         return 0;
     if (!infolist_new_var_string (ptr_item, "items", CONFIG_STRING(bar->options[GUI_BAR_OPTION_ITEMS])))
@@ -2387,6 +2433,9 @@ gui_bar_print_log ()
         log_printf ("  color_bg . . . . . . . : %d (%s)",
                     CONFIG_COLOR(ptr_bar->options[GUI_BAR_OPTION_COLOR_BG]),
                     gui_color_get_name (CONFIG_COLOR(ptr_bar->options[GUI_BAR_OPTION_COLOR_BG])));
+        log_printf ("  color_bg_inactive. . . : %d (%s)",
+                    CONFIG_COLOR(ptr_bar->options[GUI_BAR_OPTION_COLOR_BG_INACTIVE]),
+                    gui_color_get_name (CONFIG_COLOR(ptr_bar->options[GUI_BAR_OPTION_COLOR_BG_INACTIVE])));
         log_printf ("  separator. . . . . . . : %d",    CONFIG_INTEGER(ptr_bar->options[GUI_BAR_OPTION_SEPARATOR]));
         log_printf ("  items. . . . . . . . . : '%s'",  CONFIG_STRING(ptr_bar->options[GUI_BAR_OPTION_ITEMS]));
         log_printf ("  items_count. . . . . . : %d",    ptr_bar->items_count);
