@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2017 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -14,7 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef WEECHAT_GUI_BUFFER_H
@@ -86,6 +86,7 @@ struct t_gui_buffer
     int layout_number_merge_order;     /* order in merge for layout         */
     char *name;                        /* buffer name                       */
     char *full_name;                   /* plugin name + '.' + buffer name   */
+    char *old_full_name;               /* old full name (set before rename) */
     char *short_name;                  /* short buffer name                 */
     enum t_gui_buffer_type type;       /* buffer type (formatted, free, ..) */
     int notify;                        /* 0 = never                         */
@@ -132,9 +133,11 @@ struct t_gui_buffer
     int nicklist_max_length;           /* max length for a nick             */
     int nicklist_display_groups;       /* display groups ?                  */
     int nicklist_count;                /* number of nicks/groups            */
+    int nicklist_visible_count;        /* number of nicks/groups displayed  */
     int nicklist_groups_count;         /* number of groups                  */
+    int nicklist_groups_visible_count; /* number of groups displayed        */
     int nicklist_nicks_count;          /* number of nicks                   */
-    int nicklist_visible_count;        /* number of nicks/groups to display */
+    int nicklist_nicks_visible_count;  /* number of nicks displayed         */
     int (*nickcmp_callback)(const void *pointer, /* called to compare nicks */
                             void *data,          /* (search in nicklist)    */
                             struct t_gui_buffer *buffer,
@@ -156,6 +159,8 @@ struct t_gui_buffer
                                        /* input_callback                    */
     int input_get_empty;               /* 1 if empty input is sent to       */
                                        /* input_callback                    */
+    int input_multiline;               /* 1 if multiple lines are sent as   */
+                                       /* one message to input_callback     */
     char *input_buffer;                /* input buffer                      */
     int input_buffer_alloc;            /* input buffer: allocated size      */
     int input_buffer_size;             /* buffer size in bytes              */
@@ -235,6 +240,8 @@ extern int gui_buffers_visited_index;
 extern int gui_buffers_visited_count;
 extern int gui_buffers_visited_frozen;
 extern struct t_gui_buffer *gui_buffer_last_displayed;
+extern char *gui_buffer_reserved_names[];
+extern char *gui_buffer_type_string[];
 extern char *gui_buffer_notify_string[];
 extern char *gui_buffer_properties_get_integer[];
 extern char *gui_buffer_properties_get_string[];
@@ -243,11 +250,19 @@ extern char *gui_buffer_properties_set[];
 
 /* buffer functions */
 
+extern int gui_buffer_search_type (const char *type);
+extern int gui_buffer_search_notify (const char *notify);
 extern const char *gui_buffer_get_plugin_name (struct t_gui_buffer *buffer);
 extern const char *gui_buffer_get_short_name (struct t_gui_buffer *buffer);
 extern void gui_buffer_build_full_name (struct t_gui_buffer *buffer);
+extern void gui_buffer_local_var_add (struct t_gui_buffer *buffer,
+                                      const char *name,
+                                      const char *value);
+extern void gui_buffer_local_var_remove (struct t_gui_buffer *buffer,
+                                         const char *name);
 extern void gui_buffer_notify_set_all ();
 extern void gui_buffer_input_buffer_init (struct t_gui_buffer *buffer);
+extern int gui_buffer_is_reserved_name (const char *name);
 extern struct t_gui_buffer *gui_buffer_new (struct t_weechat_plugin *plugin,
                                             const char *name,
                                             int (*input_callback)(const void *pointer,
@@ -261,11 +276,11 @@ extern struct t_gui_buffer *gui_buffer_new (struct t_weechat_plugin *plugin,
                                                                   struct t_gui_buffer *buffer),
                                             const void *close_callback_pointer,
                                             void *close_callback_data);
+extern struct t_gui_buffer *gui_buffer_new_user (const char *name);
+extern void gui_buffer_user_set_callbacks ();
 extern int gui_buffer_valid (struct t_gui_buffer *buffer);
 extern char *gui_buffer_string_replace_local_var (struct t_gui_buffer *buffer,
                                                   const char *string);
-extern int gui_buffer_match_list_split (struct t_gui_buffer *buffer,
-                                        int num_buffers, char **buffers);
 extern int gui_buffer_match_list (struct t_gui_buffer *buffer,
                                   const char *string);
 extern void gui_buffer_set_plugin_for_upgrade (char *name,
