@@ -1,7 +1,7 @@
 /*
  * gui-cursor.c - functions for free movement of cursor (used by all GUI)
  *
- * Copyright (C) 2011-2015 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2011-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -74,6 +74,17 @@ gui_cursor_mode_toggle ()
     }
 }
 
+/*
+ * Stops cursor mode.
+ */
+
+void
+gui_cursor_mode_stop ()
+{
+    if (gui_cursor_mode)
+        gui_cursor_mode_toggle ();
+}
+
 /*
  * Sets debug for cursor mode.
  */
@@ -114,10 +125,10 @@ gui_cursor_display_debug_info ()
                   "chat: %d, word: \"%s\"",
                   gui_color_get_custom ("yellow,red"),
                   focus_info->x, focus_info->y,
-                  (long unsigned int)focus_info->window,
-                  (long unsigned int)focus_info->buffer,
+                  (unsigned long)focus_info->window,
+                  (unsigned long)focus_info->buffer,
                   (focus_info->buffer) ? (focus_info->buffer)->full_name : "-",
-                  (long unsigned int)focus_info->bar_window,
+                  (unsigned long)focus_info->bar_window,
                   (focus_info->bar_window) ? ((focus_info->bar_window)->bar)->name : "-",
                   (focus_info->bar_item) ? focus_info->bar_item : "-",
                   focus_info->bar_item_line,
