@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2017 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -14,13 +14,15 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef WEECHAT_GUI_NICK_H
 #define WEECHAT_GUI_NICK_H
 
-extern const char *gui_nick_find_color (const char *nickname);
-extern const char *gui_nick_find_color_name (const char *nickname);
+extern char *gui_nick_find_color_name (const char *nickname,
+                                       const char *colors);
+extern char *gui_nick_find_color (const char *nickname,
+                                  const char *colors);
 
 #endif /* WEECHAT_GUI_NICK_H */
