@@ -1,7 +1,7 @@
 /*
  * buflist-command.c - buflist command
  *
- * Copyright (C) 2003-2019 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -25,6 +25,7 @@
 #include "buflist.h"
 #include "buflist-bar-item.h"
 #include "buflist-command.h"
+#include "buflist-config.h"
 
 
 /*
@@ -45,6 +46,24 @@ buflist_command_buflist (const void *pointer, void *data,
     if (argc == 1)
         return WEECHAT_RC_OK;
 
+    if (weechat_strcasecmp (argv[1], "enable") == 0)
+    {
+        weechat_config_option_set (buflist_config_look_enabled, "on", 1);
+        return WEECHAT_RC_OK;
+    }
+
+    if (weechat_strcasecmp (argv[1], "disable") == 0)
+    {
+        weechat_config_option_set (buflist_config_look_enabled, "off", 1);
+        return WEECHAT_RC_OK;
+    }
+
+    if (weechat_strcasecmp (argv[1], "toggle") == 0)
+    {
+        weechat_config_option_set (buflist_config_look_enabled, "toggle", 1);
+        return WEECHAT_RC_OK;
+    }
+
     if (weechat_strcasecmp (argv[1], "bar") == 0)
     {
         buflist_add_bar ();
@@ -70,8 +89,11 @@ buflist_command_init ()
     weechat_hook_command (
         "buflist",
         N_("bar item with list of buffers"),
-        "bar || refresh",
-        N_("    bar: add the \"buflist\" bar\n"
+        "enable|disable|toggle || bar || refresh",
+        N_(" enable: enable buflist\n"
+           "disable: disable buflist\n"
+           " toggle: toggle buflist\n"
+           "    bar: add the \"buflist\" bar\n"
            "refresh: force the refresh of the bar items (buflist, buflist2 "
            "and buflist3)\n"
            "\n"
@@ -84,9 +106,14 @@ buflist_command_init ()
            "  - buflist.format.buffer_current: format for the current buffer\n"
            "\n"
            "The following variables can be used in these options:\n"
-           "  - bar item data (see hdata \"bar_item\" in API doc for a complete "
-           "list), for example:\n"
+           "  - bar item data (see hdata \"bar_item\" in API doc for a "
+           "complete list), for example:\n"
            "    - ${bar_item.name}\n"
+           "  - window data, where the bar item is displayed (there's no "
+           "window in root bars, see hdata \"window\" in API doc for a "
+           "complete list), for example:\n"
+           "    - ${window.number}\n"
+           "    - ${window.buffer.full_name}\n"
            "  - buffer data (see hdata \"buffer\" in API doc for a complete "
            "list), for example:\n"
            "    - ${buffer.number}\n"
@@ -142,6 +169,6 @@ buflist_command_init ()
            "\"private\" or \"highlight\"\n"
            "    - ${format_lag}: the lag for an IRC server buffer, empty if "
            "there's no lag (evaluation of option buflist.format.lag)"),
-        "bar || refresh",
+        "enable|disable|toggle || bar || refresh",
         &buflist_command_buflist, NULL, NULL);
 }
