@@ -1,7 +1,7 @@
 /*
  * buflist-config.c - buflist configuration options (file buflist.conf)
  *
- * Copyright (C) 2003-2017 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <stdlib.h>
@@ -44,6 +44,7 @@ struct t_config_option *buflist_config_look_nick_prefix;
 struct t_config_option *buflist_config_look_nick_prefix_empty;
 struct t_config_option *buflist_config_look_signals_refresh;
 struct t_config_option *buflist_config_look_sort;
+struct t_config_option *buflist_config_look_use_items;
 
 /* buflist config, format section */
 
@@ -61,8 +62,8 @@ struct t_config_option *buflist_config_format_number;
 
 struct t_hook **buflist_config_signals_refresh = NULL;
 int buflist_config_num_signals_refresh = 0;
-char **buflist_config_sort_fields = NULL;
-int buflist_config_sort_fields_count = 0;
+char **buflist_config_sort_fields[BUFLIST_BAR_NUM_ITEMS] = { NULL, NULL, NULL };
+int buflist_config_sort_fields_count[BUFLIST_BAR_NUM_ITEMS] = { 0, 0, 0 };
 char *buflist_config_format_buffer_eval = NULL;
 char *buflist_config_format_buffer_current_eval = NULL;
 char *buflist_config_format_hotlist_eval = NULL;
@@ -142,7 +143,7 @@ buflist_config_hook_signals_refresh ()
     char **all_signals, **signals;
     const char *ptr_signals_refresh;
     struct t_arraylist *signals_list;
-    int count, i;
+    int count, list_size, i;
 
     all_signals = weechat_string_dyn_alloc (256);
     if (!all_signals)
@@ -151,23 +152,26 @@ buflist_config_hook_signals_refresh ()
     ptr_signals_refresh = weechat_config_string (
         buflist_config_look_signals_refresh);
 
-    weechat_string_dyn_concat (all_signals, BUFLIST_CONFIG_SIGNALS_REFRESH);
+    weechat_string_dyn_concat (all_signals, BUFLIST_CONFIG_SIGNALS_REFRESH, -1);
     if (ptr_signals_refresh && ptr_signals_refresh[0])
     {
-        weechat_string_dyn_concat (all_signals, ",");
-        weechat_string_dyn_concat (
-            all_signals,
-            weechat_config_string (buflist_config_look_signals_refresh));
+        weechat_string_dyn_concat (all_signals, ",", -1);
+        weechat_string_dyn_concat (all_signals, ptr_signals_refresh, -1);
     }
     if (weechat_config_boolean (buflist_config_look_nick_prefix))
     {
-        weechat_string_dyn_concat (all_signals, ",");
+        weechat_string_dyn_concat (all_signals, ",", -1);
         weechat_string_dyn_concat (
             all_signals,
-            BUFLIST_CONFIG_SIGNALS_REFRESH_NICK_PREFIX);
+            BUFLIST_CONFIG_SIGNALS_REFRESH_NICK_PREFIX,
+            -1);
     }
 
-    signals = weechat_string_split (*all_signals, ",", 0, 0, &count);
+    signals = weechat_string_split (*all_signals, ",", NULL,
+                                    WEECHAT_STRING_SPLIT_STRIP_LEFT
+                                    | WEECHAT_STRING_SPLIT_STRIP_RIGHT
+                                    | WEECHAT_STRING_SPLIT_COLLAPSE_SEPS,
+                                    0, &count);
     if (signals)
     {
         signals_list = weechat_arraylist_new (
@@ -180,13 +184,13 @@ buflist_config_hook_signals_refresh ()
             {
                 weechat_arraylist_add (signals_list, signals[i]);
             }
+            list_size = weechat_arraylist_size (signals_list);
             buflist_config_signals_refresh = malloc (
-                weechat_arraylist_size (signals_list) *
-                sizeof (*buflist_config_signals_refresh));
+                list_size * sizeof (*buflist_config_signals_refresh));
             if (buflist_config_signals_refresh)
             {
-                buflist_config_num_signals_refresh = count;
-                for (i = 0; i < weechat_arraylist_size (signals_list); i++)
+                buflist_config_num_signals_refresh = list_size;
+                for (i = 0; i < list_size; i++)
                 {
                     buflist_config_signals_refresh[i] = weechat_hook_signal (
                         weechat_arraylist_get (signals_list, i),
@@ -194,10 +198,8 @@ buflist_config_hook_signals_refresh ()
                 }
                 if (weechat_buflist_plugin->debug >= 1)
                 {
-                    weechat_printf (NULL,
-                                    _("%s: %d signals hooked"),
-                                    BUFLIST_PLUGIN_NAME,
-                                    weechat_arraylist_size (signals_list));
+                    weechat_printf (NULL, _("%s: %d signals hooked"),
+                                    BUFLIST_PLUGIN_NAME, list_size);
                 }
             }
             weechat_arraylist_free (signals_list);
@@ -246,17 +248,58 @@ void
 buflist_config_change_sort (const void *pointer, void *data,
                             struct t_config_option *option)
 {
+    int i;
+    struct t_hashtable *hashtable_pointers;
+    char *sort;
+
     /* make C compiler happy */
     (void) pointer;
     (void) data;
     (void) option;
 
-    if (buflist_config_sort_fields)
-        weechat_string_free_split (buflist_config_sort_fields);
+    for (i = 0; i < BUFLIST_BAR_NUM_ITEMS; i++)
+    {
+        if (buflist_config_sort_fields[i])
+        {
+            weechat_string_free_split (buflist_config_sort_fields[i]);
+            buflist_config_sort_fields[i] = NULL;
+            buflist_config_sort_fields_count[i] = 0;
+        }
+    }
 
-    buflist_config_sort_fields = weechat_string_split (
-        weechat_config_string (buflist_config_look_sort),
-        ",", 0, 0, &buflist_config_sort_fields_count);
+    hashtable_pointers = weechat_hashtable_new (
+        32,
+        WEECHAT_HASHTABLE_STRING,
+        WEECHAT_HASHTABLE_POINTER,
+        NULL,
+        NULL);
+    if (!hashtable_pointers)
+        return;
+
+    for (i = 0; i < BUFLIST_BAR_NUM_ITEMS; i++)
+    {
+        weechat_hashtable_set (hashtable_pointers,
+                               "bar_item", buflist_bar_item_buflist[i]);
+
+        sort = weechat_string_eval_expression (
+            weechat_config_string (buflist_config_look_sort),
+            hashtable_pointers,
+            NULL,
+            NULL);
+
+        buflist_config_sort_fields[i] = weechat_string_split (
+            (sort) ? sort : "",
+            ",",
+            NULL,
+            WEECHAT_STRING_SPLIT_STRIP_LEFT
+            | WEECHAT_STRING_SPLIT_STRIP_RIGHT
+            | WEECHAT_STRING_SPLIT_COLLAPSE_SEPS,
+            0,
+            &buflist_config_sort_fields_count[i]);
+
+        if (sort)
+            free (sort);
+    }
 
     buflist_bar_item_update (0);
 }
@@ -295,6 +338,22 @@ buflist_config_change_nick_prefix (const void *pointer, void *data,
     buflist_bar_item_update (0);
 }
 
+/*
+ * Callback for changes on option "buflist.look.use_items".
+ */
+
+void
+buflist_config_change_use_items (const void *pointer, void *data,
+                                 struct t_config_option *option)
+{
+    /* make C compiler happy */
+    (void) pointer;
+    (void) data;
+    (void) option;
+
+    buflist_bar_item_update (2);
+}
+
 /*
  * Callback for changes on options needing bar item refresh.
  */
@@ -401,6 +460,7 @@ buflist_config_init ()
     if (!ptr_section)
     {
         weechat_config_free (buflist_config_file);
+        buflist_config_file = NULL;
         return 0;
     }
 
@@ -466,6 +526,15 @@ buflist_config_init ()
         NULL, NULL, NULL,
         NULL, NULL, NULL,
         NULL, NULL, NULL);
+    buflist_config_look_mouse_wheel = weechat_config_new_option (
+        buflist_config_file, ptr_section,
+        "mouse_wheel", "boolean",
+        N_("if enabled, mouse wheel up/down actions jump to previous/next "
+           "buffer in list"),
+        NULL, 0, 0, "on", NULL, 0,
+        NULL, NULL, NULL,
+        NULL, NULL, NULL,
+        NULL, NULL, NULL);
     buflist_config_look_nick_prefix = weechat_config_new_option (
         buflist_config_file, ptr_section,
         "nick_prefix", "boolean",
@@ -486,15 +555,6 @@ buflist_config_init ()
         NULL, NULL, NULL,
         &buflist_config_change_buflist, NULL, NULL,
         NULL, NULL, NULL);
-    buflist_config_look_mouse_wheel = weechat_config_new_option (
-        buflist_config_file, ptr_section,
-        "mouse_wheel", "boolean",
-        N_("if enabled, mouse wheel up/down actions jump to previous/next "
-           "buffer in list"),
-        NULL, 0, 0, "on", NULL, 0,
-        NULL, NULL, NULL,
-        NULL, NULL, NULL,
-        NULL, NULL, NULL);
     buflist_config_look_signals_refresh = weechat_config_new_option (
         buflist_config_file, ptr_section,
         "signals_refresh", "string",
@@ -515,11 +575,23 @@ buflist_config_init ()
            "char \"-\" can be used before field to reverse order, "
            "char \"~\" can be used to do a case insensitive comparison; "
            "example: \"-~short_name\" for case insensitive and reverse "
-           "sort on buffer short name"),
+           "sort on buffer short name "
+           "(note: content is evaluated, only the pointer to bar_item can be "
+           "used, for example \"bar_item.name\")"),
         NULL, 0, 0, "number,-active", NULL, 0,
         NULL, NULL, NULL,
         &buflist_config_change_sort, NULL, NULL,
         NULL, NULL, NULL);
+    buflist_config_look_use_items = weechat_config_new_option (
+        buflist_config_file, ptr_section,
+        "use_items", "integer",
+        N_("number of buflist bar items that can be used; the item names are: "
+           "\"buflist\", \"buflist2\", \"buflist3\"; be careful, using more "
+           "than one bar item slows down the display of buffers list"),
+        NULL, 1, BUFLIST_BAR_NUM_ITEMS, "1", NULL, 0,
+        NULL, NULL, NULL,
+        &buflist_config_change_use_items, NULL, NULL,
+        NULL, NULL, NULL);
 
     /* format */
     ptr_section = weechat_config_new_section (buflist_config_file, "format",
@@ -532,6 +604,7 @@ buflist_config_init ()
     if (!ptr_section)
     {
         weechat_config_free (buflist_config_file);
+        buflist_config_file = NULL;
         return 0;
     }
 
@@ -539,7 +612,13 @@ buflist_config_init ()
         buflist_config_file, ptr_section,
         "buffer", "string",
         N_("format of each line with a buffer "
-           "(note: content is evaluated, see /help buflist)"),
+           "(note: content is evaluated, see /help buflist); "
+           "example: standard format for bar item \"buflist\" and only the "
+           "buffer number between square brackets for other bar items "
+           "(\"buflist2\" and \"buflist3\"): "
+           "\"${if:${bar_item.name}==buflist?${format_number}${indent}"
+           "${format_nick_prefix}${color_hotlist}${format_name}:"
+           "[${number}]}\""),
         NULL, 0, 0,
         "${format_number}${indent}${format_nick_prefix}${color_hotlist}"
         "${format_name}",
@@ -567,6 +646,15 @@ buflist_config_init ()
         NULL, NULL, NULL,
         &buflist_config_change_format, NULL, NULL,
         NULL, NULL, NULL);
+    buflist_config_format_hotlist_level[3] = weechat_config_new_option (
+        buflist_config_file, ptr_section,
+        "hotlist_highlight", "string",
+        N_("format for a buffer with hotlist level \"highlight\" "
+           "(note: content is evaluated, see /help buflist)"),
+        NULL, 0, 0, "${color:magenta}", NULL, 0,
+        NULL, NULL, NULL,
+        &buflist_config_change_buflist, NULL, NULL,
+        NULL, NULL, NULL);
     buflist_config_format_hotlist_level[0] = weechat_config_new_option (
         buflist_config_file, ptr_section,
         "hotlist_low", "string",
@@ -585,24 +673,6 @@ buflist_config_init ()
         NULL, NULL, NULL,
         &buflist_config_change_buflist, NULL, NULL,
         NULL, NULL, NULL);
-    buflist_config_format_hotlist_level[2] = weechat_config_new_option (
-        buflist_config_file, ptr_section,
-        "hotlist_private", "string",
-        N_("format for a buffer with hotlist level \"private\" "
-           "(note: content is evaluated, see /help buflist)"),
-        NULL, 0, 0, "${color:green}", NULL, 0,
-        NULL, NULL, NULL,
-        &buflist_config_change_buflist, NULL, NULL,
-        NULL, NULL, NULL);
-    buflist_config_format_hotlist_level[3] = weechat_config_new_option (
-        buflist_config_file, ptr_section,
-        "hotlist_highlight", "string",
-        N_("format for a buffer with hotlist level \"highlight\" "
-           "(note: content is evaluated, see /help buflist)"),
-        NULL, 0, 0, "${color:magenta}", NULL, 0,
-        NULL, NULL, NULL,
-        &buflist_config_change_buflist, NULL, NULL,
-        NULL, NULL, NULL);
     buflist_config_format_hotlist_level_none = weechat_config_new_option (
         buflist_config_file, ptr_section,
         "hotlist_none", "string",
@@ -612,6 +682,15 @@ buflist_config_init ()
         NULL, NULL, NULL,
         &buflist_config_change_buflist, NULL, NULL,
         NULL, NULL, NULL);
+    buflist_config_format_hotlist_level[2] = weechat_config_new_option (
+        buflist_config_file, ptr_section,
+        "hotlist_private", "string",
+        N_("format for a buffer with hotlist level \"private\" "
+           "(note: content is evaluated, see /help buflist)"),
+        NULL, 0, 0, "${color:green}", NULL, 0,
+        NULL, NULL, NULL,
+        &buflist_config_change_buflist, NULL, NULL,
+        NULL, NULL, NULL);
     buflist_config_format_hotlist_separator = weechat_config_new_option (
         buflist_config_file, ptr_section,
         "hotlist_separator", "string",
@@ -712,16 +791,21 @@ buflist_config_write ()
 void
 buflist_config_free ()
 {
+    int i;
+
     weechat_config_free (buflist_config_file);
 
     if (buflist_config_signals_refresh)
         buflist_config_free_signals_refresh ();
 
-    if (buflist_config_sort_fields)
+    for (i = 0; i < BUFLIST_BAR_NUM_ITEMS; i++)
     {
-        weechat_string_free_split (buflist_config_sort_fields);
-        buflist_config_sort_fields = NULL;
-        buflist_config_sort_fields_count = 0;
+        if (buflist_config_sort_fields[i])
+        {
+            weechat_string_free_split (buflist_config_sort_fields[i]);
+            buflist_config_sort_fields[i] = NULL;
+            buflist_config_sort_fields_count[i] = 0;
+        }
     }
 
     if (buflist_config_format_buffer_eval)
