@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2017 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -14,12 +14,14 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef WEECHAT_PLUGIN_BUFLIST_CONFIG_H
 #define WEECHAT_PLUGIN_BUFLIST_CONFIG_H
 
+#include "buflist-bar-item.h"
+
 #define BUFLIST_CONFIG_NAME "buflist"
 
 #define BUFLIST_CONFIG_SIGNALS_REFRESH                                  \
@@ -43,6 +45,7 @@ extern struct t_config_option *buflist_config_look_nick_prefix;
 extern struct t_config_option *buflist_config_look_nick_prefix_empty;
 extern struct t_config_option *buflist_config_look_signals_refresh;
 extern struct t_config_option *buflist_config_look_sort;
+extern struct t_config_option *buflist_config_look_use_items;
 
 extern struct t_config_option *buflist_config_format_buffer;
 extern struct t_config_option *buflist_config_format_buffer_current;
@@ -56,12 +59,14 @@ extern struct t_config_option *buflist_config_format_name;
 extern struct t_config_option *buflist_config_format_nick_prefix;
 extern struct t_config_option *buflist_config_format_number;
 
-extern char **buflist_config_sort_fields;
-extern int buflist_config_sort_fields_count;
+extern char **buflist_config_sort_fields[BUFLIST_BAR_NUM_ITEMS];
+extern int buflist_config_sort_fields_count[BUFLIST_BAR_NUM_ITEMS];
 extern char *buflist_config_format_buffer_eval;
 extern char *buflist_config_format_buffer_current_eval;
 extern char *buflist_config_format_hotlist_eval;
 
+extern void buflist_config_change_sort (const void *pointer, void *data,
+                                        struct t_config_option *option);
 extern int buflist_config_init ();
 extern int buflist_config_read ();
 extern int buflist_config_write ();
