@@ -1,7 +1,7 @@
 /*
  * irc-bar-item.c - bar items for IRC plugin
  *
- * Copyright (C) 2003-2020 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -472,7 +472,7 @@ irc_bar_item_nick_host (const void *pointer, void *data,
 
     snprintf (buf, sizeof (buf), "%s%s%s",
               server->nick,
-              (server->host) ? "@" : "",
+              (server->host) ? "!" : "",
               (server->host) ? server->host : "");
 
     return strdup (buf);
@@ -643,6 +643,57 @@ irc_bar_item_nick_modes (const void *pointer, void *data,
     return buf;
 }
 
+/*
+ * Returns content of bar item "nick_prefix": bar item with nick prefix.
+ */
+
+char *
+irc_bar_item_nick_prefix (const void *pointer, void *data,
+                          struct t_gui_bar_item *item,
+                          struct t_gui_window *window,
+                          struct t_gui_buffer *buffer,
+                          struct t_hashtable *extra_info)
+{
+    struct t_irc_server *server;
+    struct t_irc_channel *channel;
+    struct t_irc_nick *ptr_nick;
+    char str_prefix[64];
+
+    /* make C compiler happy */
+    (void) pointer;
+    (void) data;
+    (void) item;
+    (void) window;
+    (void) extra_info;
+
+    if (!buffer)
+        return NULL;
+
+    irc_buffer_get_server_and_channel (buffer, &server, &channel);
+    if (!server || !server->nick)
+        return NULL;
+
+    str_prefix[0] = '\0';
+    if (channel && (channel->type == IRC_CHANNEL_TYPE_CHANNEL))
+    {
+        ptr_nick = irc_nick_search (server, channel, server->nick);
+        if (ptr_nick)
+        {
+            if (weechat_config_boolean (irc_config_look_nick_mode_empty)
+                || (ptr_nick->prefix[0] != ' '))
+            {
+                snprintf (str_prefix, sizeof (str_prefix), "%s%s",
+                          weechat_color (
+                              irc_nick_get_prefix_color_name (
+                                  server, ptr_nick->prefix[0])),
+                          ptr_nick->prefix);
+            }
+        }
+    }
+
+    return (str_prefix[0]) ? strdup (str_prefix) : NULL;
+}
+
 /*
  * Focus on nicklist.
  */
@@ -656,6 +707,7 @@ irc_bar_item_focus_buffer_nicklist (const void *pointer, void *data,
     struct t_gui_buffer *buffer;
     struct t_irc_nick *ptr_nick;
     const char *str_buffer, *nick;
+    char str_value[128];
 
     str_buffer = weechat_hashtable_get (info, "_buffer");
     if (!str_buffer || !str_buffer[0])
@@ -679,9 +731,15 @@ irc_bar_item_focus_buffer_nicklist (const void *pointer, void *data,
         if (nick)
         {
             ptr_nick = irc_nick_search (ptr_server, ptr_channel, nick);
-            if (ptr_nick && ptr_nick->host)
+            if (ptr_nick)
             {
-                weechat_hashtable_set (info, "irc_host", ptr_nick->host);
+                snprintf (str_value, sizeof (str_value),
+                          "0x%lx", (unsigned long)ptr_nick);
+                weechat_hashtable_set (info, "irc_nick", str_value);
+
+                if (ptr_nick->host)
+                    weechat_hashtable_set (info, "irc_host", ptr_nick->host);
+
                 return info;
             }
         }
@@ -762,6 +820,8 @@ irc_bar_item_init ()
                           &irc_bar_item_input_prompt, NULL, NULL);
     weechat_bar_item_new ("irc_nick_modes",
                           &irc_bar_item_nick_modes, NULL, NULL);
+    weechat_bar_item_new ("irc_nick_prefix",
+                          &irc_bar_item_nick_prefix, NULL, NULL);
 
     weechat_hook_focus ("buffer_nicklist",
                         &irc_bar_item_focus_buffer_nicklist, NULL, NULL);
