@@ -1,7 +1,7 @@
 /*
  * irc-sasl.c - SASL authentication with IRC server
  *
- * Copyright (C) 2003-2019 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -26,12 +26,10 @@
 #include <arpa/inet.h>
 #include <gcrypt.h>
 
-#ifdef HAVE_GNUTLS
 #include <gnutls/gnutls.h>
 #if LIBGNUTLS_VERSION_NUMBER >= 0x020a01 /* 2.10.1 */
 #include <gnutls/abstract.h>
 #endif /* LIBGNUTLS_VERSION_NUMBER >= 0x020a01 */
-#endif /* HAVE_GNUTLS */
 
 #include "../weechat-plugin.h"
 #include "irc.h"
@@ -96,8 +94,7 @@ irc_sasl_mechanism_plain (const char *sasl_username, const char *sasl_password)
 char *
 irc_sasl_get_key_content (struct t_irc_server *server, const char *sasl_key)
 {
-    const char *weechat_dir;
-    char *key_path1, *key_path2, *content;
+    char *weechat_dir, *key_path1, *key_path2, *content;
 
     if (!sasl_key)
         return NULL;
@@ -122,6 +119,8 @@ irc_sasl_get_key_content (struct t_irc_server *server, const char *sasl_key)
             (key_path2) ? key_path2 : ((key_path1) ? key_path1 : sasl_key));
     }
 
+    if (weechat_dir)
+        free (weechat_dir);
     if (key_path1)
         free (key_path1);
     if (key_path2)
@@ -143,7 +142,7 @@ irc_sasl_mechanism_ecdsa_nist256p_challenge (struct t_irc_server *server,
                                              const char *sasl_username,
                                              const char *sasl_key)
 {
-#if defined(HAVE_GNUTLS) && (LIBGNUTLS_VERSION_NUMBER >= 0x030015) /* 3.0.21 */
+#if LIBGNUTLS_VERSION_NUMBER >= 0x030015 /* 3.0.21 */
     char *data, *string, *answer_base64;
     int length_data, length_username, length, ret;
     char *str_privkey;
@@ -315,7 +314,7 @@ irc_sasl_mechanism_ecdsa_nist256p_challenge (struct t_irc_server *server,
 
     return answer_base64;
 
-#else /* no gnutls or gnutls < 3.0.21 */
+#else /* GnuTLS < 3.0.21 */
 
     /* make C compiler happy */
     (void) data_base64;
@@ -328,7 +327,7 @@ irc_sasl_mechanism_ecdsa_nist256p_challenge (struct t_irc_server *server,
                     weechat_prefix ("error"));
 
     return NULL;
-#endif /* defined(HAVE_GNUTLS) && (LIBGNUTLS_VERSION_NUMBER >= 0x030015) */
+#endif /* LIBGNUTLS_VERSION_NUMBER >= 0x030015 */
 }
 
 /*
@@ -479,10 +478,8 @@ irc_sasl_mechanism_dh_blowfish (const char *data_base64,
     /* create password buffers (clear and crypted) */
     length_password = strlen (sasl_password) +
         ((8 - (strlen (sasl_password) % 8)) % 8);
-    password_clear = malloc (length_password);
-    password_crypted = malloc (length_password);
-    memset (password_clear, 0, length_password);
-    memset (password_crypted, 0, length_password);
+    password_clear = calloc (1, length_password);
+    password_crypted = calloc (1, length_password);
     memcpy (password_clear, sasl_password, strlen (sasl_password));
 
     /* crypt password using blowfish */
@@ -605,10 +602,9 @@ irc_sasl_mechanism_dh_aes (const char *data_base64,
     length_password = strlen (sasl_password) + 1;
     length_userpass = length_username + length_password +
         ((16 - ((length_username + length_password) % 16)) % 16);
-    ptr_userpass = userpass_clear = malloc (length_userpass);
-    userpass_crypted = malloc (length_userpass);
-    memset (userpass_clear, 0, length_password);
-    memset (userpass_crypted, 0, length_password);
+    userpass_clear = calloc (1, length_userpass);
+    ptr_userpass = userpass_clear;
+    userpass_crypted = calloc (1, length_userpass);
     memcpy (ptr_userpass, sasl_username, length_username);
     ptr_userpass += length_username;
     memcpy (ptr_userpass, sasl_password, length_password);
