@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2019 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
  * Copyright (C) 2012 Simon Arlott
  *
  * This file is part of WeeChat, the extensible chat client.
@@ -25,9 +25,7 @@
 #include <sys/time.h>
 #include <regex.h>
 
-#ifdef HAVE_GNUTLS
 #include <gnutls/gnutls.h>
-#endif /* HAVE_GNUTLS */
 
 #ifndef NI_MAXHOST
 #define NI_MAXHOST 256
@@ -42,6 +40,13 @@ enum t_irc_server_sasl_fail
     IRC_SERVER_NUM_SASL_FAIL,
 };
 
+enum t_irc_server_charset_message
+{
+    IRC_SERVER_CHARSET_MESSAGE_MESSAGE = 0,
+    IRC_SERVER_CHARSET_MESSAGE_CHANNEL,
+    IRC_SERVER_CHARSET_MESSAGE_TEXT,
+};
+
 enum t_irc_server_option
 {
     IRC_SERVER_OPTION_ADDRESSES = 0, /* server addresses (IP/name with port) */
@@ -49,6 +54,7 @@ enum t_irc_server_option
     IRC_SERVER_OPTION_IPV6,          /* use IPv6 protocol                    */
     IRC_SERVER_OPTION_SSL,           /* SSL protocol                         */
     IRC_SERVER_OPTION_SSL_CERT,      /* client ssl certificate file          */
+    IRC_SERVER_OPTION_SSL_PASSWORD,   /* client ssl certificate key password */
     IRC_SERVER_OPTION_SSL_PRIORITIES, /* gnutls priorities                   */
     IRC_SERVER_OPTION_SSL_DHKEY_SIZE, /* Diffie Hellman key size             */
     IRC_SERVER_OPTION_SSL_FINGERPRINT, /* SHA1 fingerprint of certificate    */
@@ -85,6 +91,8 @@ enum t_irc_server_option
     IRC_SERVER_OPTION_MSG_QUIT,             /* default quit message          */
     IRC_SERVER_OPTION_NOTIFY,               /* notify list                   */
     IRC_SERVER_OPTION_SPLIT_MSG_MAX_LENGTH, /* max length of messages        */
+    IRC_SERVER_OPTION_CHARSET_MESSAGE,      /* what to decode/encode in msg  */
+    IRC_SERVER_OPTION_DEFAULT_CHANTYPES,    /* chantypes if not received     */
     /* number of server options */
     IRC_SERVER_NUM_OPTIONS,
 };
@@ -135,6 +143,15 @@ enum t_irc_server_casemapping
     IRC_SERVER_NUM_CASEMAPPING,
 };
 
+/* utf8mapping (string comparisons for nicks/channels) */
+enum t_irc_server_utf8mapping
+{
+    IRC_SERVER_UTF8MAPPING_NONE = 0,
+    IRC_SERVER_UTF8MAPPING_RFC8265,
+    /* number of utf8mapping */
+    IRC_SERVER_NUM_UTF8MAPPING,
+};
+
 /* output queue of messages to server (for sending slowly to server) */
 
 struct t_irc_outqueue
@@ -157,6 +174,7 @@ struct t_irc_server
 
     /* internal vars */
     int temp_server;                /* temporary server (not saved)          */
+    int fake_server;                /* fake server (no I/O, for tests)       */
     int reloading_from_config;      /* 1 if reloading from config file       */
     int reloaded_from_config;       /* 1 if reloaded from config file        */
     char *addresses_eval;           /* evaluation of addresses               */
@@ -178,11 +196,9 @@ struct t_irc_server
     int is_connected;               /* 1 if WeeChat is connected to server   */
     int ssl_connected;              /* = 1 if connected with SSL             */
     int disconnected;               /* 1 if server has been disconnected     */
-#ifdef HAVE_GNUTLS
     gnutls_session_t gnutls_sess;   /* gnutls session (only if SSL is used)  */
     gnutls_x509_crt_t tls_cert;     /* certificate used if ssl_cert is set   */
     gnutls_x509_privkey_t tls_cert_key; /* key used if ssl_cert is set       */
-#endif /* HAVE_GNUTLS */
     char *unterminated_message;     /* beginning of a message in input buf   */
     int nicks_count;                /* number of nicknames                   */
     char **nicks_array;             /* nicknames (after split)               */
@@ -204,6 +220,7 @@ struct t_irc_server
     int user_max_length;            /* max length of user (from msg 005)     */
     int host_max_length;            /* max length of host (from msg 005)     */
     int casemapping;                /* casemapping from msg 005              */
+    int utf8mapping;                /* utf8mapping from msg 005              */
     char *chantypes;                /* chantypes from msg 005 (eg "&#")      */
     char *chanmodes;                /* chanmodes from msg 005                */
                                     /* (eg "beI,k,l,imnpstaqr")              */
@@ -257,7 +274,6 @@ struct t_irc_message
 
 /* digest algorithms for fingerprint */
 
-#ifdef HAVE_GNUTLS
 enum t_irc_fingerprint_digest_algo
 {
     IRC_FINGERPRINT_ALGO_SHA1 = 0,
@@ -266,13 +282,13 @@ enum t_irc_fingerprint_digest_algo
     /* number of digest algorithms */
     IRC_FINGERPRINT_NUM_ALGOS,
 };
-#endif /* HAVE_GNUTLS */
 
+extern char *irc_server_prefix_modes_default;
+extern char *irc_server_prefix_chars_default;
+extern char *irc_server_chanmodes_default;
 extern struct t_irc_server *irc_servers;
-#ifdef HAVE_GNUTLS
 extern const int gnutls_cert_type_prio[];
 extern const int gnutls_prot_prio[];
-#endif /* HAVE_GNUTLS */
 extern struct t_irc_message *irc_recv_msgq, *irc_msgq_last_msg;
 extern char *irc_server_sasl_fail_string[];
 extern char *irc_server_options[][2];
@@ -282,6 +298,7 @@ extern struct t_irc_server *irc_server_search (const char *server_name);
 extern struct t_irc_server *irc_server_casesearch (const char *server_name);
 extern int irc_server_search_option (const char *option_name);
 extern int irc_server_search_casemapping (const char *casemapping);
+extern int irc_server_search_utf8mapping (const char *utf8mapping);
 extern int irc_server_strcasecmp (struct t_irc_server *server,
                                   const char *string1, const char *string2);
 extern int irc_server_strncasecmp (struct t_irc_server *server,
@@ -302,6 +319,7 @@ extern void irc_server_set_host (struct t_irc_server *server,
 extern const char *irc_server_get_alternate_nick (struct t_irc_server *server);
 extern const char *irc_server_get_isupport_value (struct t_irc_server *server,
                                                   const char *feature);
+extern const char *irc_server_get_chantypes (struct t_irc_server *server);
 extern void irc_server_set_prefix_modes_chars (struct t_irc_server *server,
                                                const char *prefix);
 extern void irc_server_set_lag (struct t_irc_server *server);
@@ -345,10 +363,8 @@ extern void irc_server_msgq_add_buffer (struct t_irc_server *server,
 extern void irc_server_msgq_flush ();
 extern void irc_server_set_buffer_title (struct t_irc_server *server);
 extern struct t_gui_buffer *irc_server_create_buffer (struct t_irc_server *server);
-#ifdef HAVE_GNUTLS
 int irc_server_fingerprint_search_algo_with_size (int size);
 char *irc_server_fingerprint_str_sizes ();
-#endif /* HAVE_GNUTLS */
 extern int irc_server_connect (struct t_irc_server *server);
 extern void irc_server_auto_connect (int auto_connect);
 extern void irc_server_autojoin_channels (struct t_irc_server *server);
