@@ -1,7 +1,7 @@
 /*
  * logger.c - logger plugin for WeeChat: save buffer lines to disk files
  *
- * Copyright (C) 2003-2017 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -16,14 +16,9 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
-/* this define is needed for strptime() (not on OpenBSD/Sun) */
-#if !defined(__OpenBSD__) && !defined(__sun)
-#define _XOPEN_SOURCE 700
-#endif
-
 #include <stdlib.h>
 #include <unistd.h>
 #include <errno.h>
@@ -39,7 +34,9 @@
 
 #include "../weechat-plugin.h"
 #include "logger.h"
+#include "logger-backlog.h"
 #include "logger-buffer.h"
+#include "logger-command.h"
 #include "logger-config.h"
 #include "logger-info.h"
 #include "logger-tail.h"
@@ -54,7 +51,8 @@ WEECHAT_PLUGIN_PRIORITY(14000);
 
 struct t_weechat_plugin *weechat_logger_plugin = NULL;
 
-struct t_hook *logger_timer = NULL;    /* timer to flush log files          */
+struct t_hook *logger_hook_timer = NULL;    /* timer to flush log files     */
+struct t_hook *logger_hook_print = NULL;
 
 
 /*
@@ -296,8 +294,7 @@ logger_get_mask_for_buffer (struct t_gui_buffer *buffer)
 char *
 logger_get_mask_expanded (struct t_gui_buffer *buffer, const char *mask)
 {
-    char *mask2, *mask3, *mask4, *mask5, *mask6, *mask7;
-    const char *dir_separator;
+    char *mask2, *mask3, *mask4, *mask5, *mask6, *mask7, *dir_separator;
     int length;
     time_t seconds;
     struct tm *date_tmp;
@@ -373,6 +370,7 @@ logger_get_mask_expanded (struct t_gui_buffer *buffer, const char *mask)
     }
 
 end:
+    free (dir_separator);
     if (mask2)
         free (mask2);
     if (mask3)
@@ -396,9 +394,8 @@ end:
 char *
 logger_get_filename (struct t_gui_buffer *buffer)
 {
-    char *res, *mask_expanded, *file_path;
+    char *res, *mask_expanded, *file_path, *dir_separator, *weechat_dir;
     const char *mask;
-    const char *dir_separator, *weechat_dir;
     int length;
 
     res = NULL;
@@ -410,7 +407,10 @@ logger_get_filename (struct t_gui_buffer *buffer)
         return NULL;
     weechat_dir = weechat_info_get ("weechat_dir", "");
     if (!weechat_dir)
+    {
+        free (dir_separator);
         return NULL;
+    }
 
     /* get filename mask for buffer */
     mask = logger_get_mask_for_buffer (buffer);
@@ -422,6 +422,8 @@ logger_get_filename (struct t_gui_buffer *buffer)
               "\"%s\", logging is disabled for this buffer"),
             weechat_prefix ("error"), LOGGER_PLUGIN_NAME,
             weechat_buffer_get_string (buffer, "name"));
+        free (dir_separator);
+        free (weechat_dir);
         return NULL;
     }
 
@@ -446,6 +448,8 @@ logger_get_filename (struct t_gui_buffer *buffer)
     }
 
 end:
+    free (dir_separator);
+    free (weechat_dir);
     if (mask_expanded)
         free (mask_expanded);
     if (file_path)
@@ -462,7 +466,7 @@ void
 logger_set_log_filename (struct t_logger_buffer *logger_buffer)
 {
     char *log_filename, *pos_last_sep;
-    const char *dir_separator;
+    char *dir_separator;
     struct t_logger_buffer *ptr_logger_buffer;
 
     /* get log filename for buffer */
@@ -504,12 +508,130 @@ logger_set_log_filename (struct t_logger_buffer *logger_buffer)
             weechat_mkdir_parents (log_filename, 0700);
             pos_last_sep[0] = dir_separator[0];
         }
+        free (dir_separator);
     }
 
     /* set log filename */
     logger_buffer->log_filename = log_filename;
 }
 
+/*
+ * Creates a log file.
+ *
+ * Returns:
+ *   1: OK
+ *   0: error
+ */
+
+int
+logger_create_log_file (struct t_logger_buffer *logger_buffer)
+{
+    char *charset, *message, buf_time[256], buf_beginning[1024];
+    int log_level, rc;
+    time_t seconds;
+    struct tm *date_tmp;
+    struct stat statbuf;
+
+    if (logger_buffer->log_file)
+    {
+        /*
+         * check that the inode has not changed, otherwise that means the file
+         * was deleted, and we must reopen it
+         */
+        rc = stat (logger_buffer->log_filename, &statbuf);
+        if ((rc == 0) && (statbuf.st_ino == logger_buffer->log_file_inode))
+        {
+            /* inode has not changed, we can write in this file */
+            return 1;
+        }
+        fclose (logger_buffer->log_file);
+        logger_buffer->log_file = NULL;
+        logger_buffer->log_file_inode = 0;
+    }
+
+    /* get log level */
+    log_level = logger_get_level_for_buffer (logger_buffer->buffer);
+    if (log_level == 0)
+        return 0;
+
+    /* create directory */
+    if (!logger_create_directory ())
+    {
+        weechat_printf_date_tags (
+            NULL, 0, "no_log",
+            _("%s%s: unable to create directory for logs "
+              "(\"%s\")"),
+            weechat_prefix ("error"), LOGGER_PLUGIN_NAME,
+            weechat_config_string (logger_config_file_path));
+        return 0;
+    }
+    if (!logger_buffer->log_filename)
+        logger_set_log_filename (logger_buffer);
+    if (!logger_buffer->log_filename)
+        return 0;
+
+    /* create or append to log file */
+    logger_buffer->log_file =
+        fopen (logger_buffer->log_filename, "a");
+    if (!logger_buffer->log_file)
+    {
+        weechat_printf_date_tags (
+            NULL, 0, "no_log",
+            _("%s%s: unable to write log file \"%s\": %s"),
+            weechat_prefix ("error"), LOGGER_PLUGIN_NAME,
+            logger_buffer->log_filename, strerror (errno));
+        return 0;
+    }
+
+    /* get file inode */
+    rc = stat (logger_buffer->log_filename, &statbuf);
+    if (rc != 0)
+    {
+        weechat_printf_date_tags (
+            NULL, 0, "no_log",
+            _("%s%s: unable to get file status of log file \"%s\": %s"),
+            weechat_prefix ("error"), LOGGER_PLUGIN_NAME,
+            logger_buffer->log_filename, strerror (errno));
+        fclose (logger_buffer->log_file);
+        logger_buffer->log_file = NULL;
+        logger_buffer->log_file_inode = 0;
+        return 0;
+    }
+    logger_buffer->log_file_inode = statbuf.st_ino;
+
+    /* write info line */
+    if (weechat_config_boolean (logger_config_file_info_lines)
+        && logger_buffer->write_start_info_line)
+    {
+        buf_time[0] = '\0';
+        seconds = time (NULL);
+        date_tmp = localtime (&seconds);
+        if (date_tmp)
+        {
+            if (strftime (buf_time, sizeof (buf_time) - 1,
+                          weechat_config_string (logger_config_file_time_format),
+                          date_tmp) == 0)
+                buf_time[0] = '\0';
+        }
+        snprintf (buf_beginning, sizeof (buf_beginning),
+                  _("%s\t****  Beginning of log  ****"),
+                  buf_time);
+        charset = weechat_info_get ("charset_terminal", "");
+        message = (charset) ?
+            weechat_iconv_from_internal (charset, buf_beginning) : NULL;
+        fprintf (logger_buffer->log_file,
+                 "%s\n", (message) ? message : buf_beginning);
+        if (charset)
+            free (charset);
+        if (message)
+            free (message);
+        logger_buffer->flush_needed = 1;
+    }
+    logger_buffer->write_start_info_line = 0;
+
+    return 1;
+}
+
 /*
  * Writes a line to log file.
  */
@@ -518,94 +640,32 @@ void
 logger_write_line (struct t_logger_buffer *logger_buffer,
                    const char *format, ...)
 {
-    char *message, buf_time[256], buf_beginning[1024];
-    const char *charset;
-    time_t seconds;
-    struct tm *date_tmp;
-    int log_level;
+    char *charset, *message;
 
-    charset = weechat_info_get ("charset_terminal", "");
+    if (!logger_create_log_file (logger_buffer))
+        return;
 
     if (!logger_buffer->log_file)
-    {
-        log_level = logger_get_level_for_buffer (logger_buffer->buffer);
-        if (log_level == 0)
-        {
-            logger_buffer_free (logger_buffer);
-            return;
-        }
-        if (!logger_create_directory ())
-        {
-            weechat_printf_date_tags (
-                NULL, 0, "no_log",
-                _("%s%s: unable to create directory for logs "
-                  "(\"%s\")"),
-                weechat_prefix ("error"), LOGGER_PLUGIN_NAME,
-                weechat_config_string (logger_config_file_path));
-            logger_buffer_free (logger_buffer);
-            return;
-        }
-        if (!logger_buffer->log_filename)
-            logger_set_log_filename (logger_buffer);
-        if (!logger_buffer->log_filename)
-        {
-            logger_buffer_free (logger_buffer);
-            return;
-        }
-
-        logger_buffer->log_file =
-            fopen (logger_buffer->log_filename, "a");
-        if (!logger_buffer->log_file)
-        {
-            weechat_printf_date_tags (
-                NULL, 0, "no_log",
-                _("%s%s: unable to write log file \"%s\": %s"),
-                weechat_prefix ("error"), LOGGER_PLUGIN_NAME,
-                logger_buffer->log_filename, strerror (errno));
-            logger_buffer_free (logger_buffer);
-            return;
-        }
-
-        if (weechat_config_boolean (logger_config_file_info_lines)
-            && logger_buffer->write_start_info_line)
-        {
-            buf_time[0] = '\0';
-            seconds = time (NULL);
-            date_tmp = localtime (&seconds);
-            if (date_tmp)
-            {
-                if (strftime (buf_time, sizeof (buf_time) - 1,
-                              weechat_config_string (logger_config_file_time_format),
-                              date_tmp) == 0)
-                    buf_time[0] = '\0';
-            }
-            snprintf (buf_beginning, sizeof (buf_beginning),
-                      _("%s\t****  Beginning of log  ****"),
-                      buf_time);
-            message = (charset) ?
-                weechat_iconv_from_internal (charset, buf_beginning) : NULL;
-            fprintf (logger_buffer->log_file,
-                     "%s\n", (message) ? message : buf_beginning);
-            if (message)
-                free (message);
-            logger_buffer->flush_needed = 1;
-        }
-        logger_buffer->write_start_info_line = 0;
-    }
+        return;
 
     weechat_va_format (format);
     if (vbuffer)
     {
+        charset = weechat_info_get ("charset_terminal", "");
         message = (charset) ?
             weechat_iconv_from_internal (charset, vbuffer) : NULL;
         fprintf (logger_buffer->log_file,
                  "%s\n", (message) ? message : vbuffer);
+        if (charset)
+            free (charset);
         if (message)
             free (message);
         logger_buffer->flush_needed = 1;
-        if (!logger_timer)
+        if (!logger_hook_timer)
         {
             fflush (logger_buffer->log_file);
+            if (weechat_config_boolean (logger_config_file_fsync))
+                fsync (fileno (logger_buffer->log_file));
             logger_buffer->flush_needed = 0;
         }
         free (vbuffer);
@@ -644,9 +704,8 @@ logger_stop (struct t_logger_buffer *logger_buffer, int write_info_line)
                                _("%s\t****  End of log  ****"),
                                buf_time);
         }
-        fclose (logger_buffer->log_file);
-        logger_buffer->log_file = NULL;
     }
+
     logger_buffer_free (logger_buffer);
 }
 
@@ -706,6 +765,7 @@ logger_start_buffer (struct t_gui_buffer *buffer, int write_info_line)
                     {
                         fclose (ptr_logger_buffer->log_file);
                         ptr_logger_buffer->log_file = NULL;
+                        ptr_logger_buffer->log_file_inode = 0;
                     }
                 }
             }
@@ -737,91 +797,6 @@ logger_start_buffer_all (int write_info_line)
     }
 }
 
-/*
- * Displays logging status for buffers.
- */
-
-void
-logger_list ()
-{
-    struct t_infolist *ptr_infolist;
-    struct t_logger_buffer *ptr_logger_buffer;
-    struct t_gui_buffer *ptr_buffer;
-    char status[128];
-
-    weechat_printf (NULL, "");
-    weechat_printf (NULL, _("Logging on buffers:"));
-
-    ptr_infolist = weechat_infolist_get ("buffer", NULL, NULL);
-    if (ptr_infolist)
-    {
-        while (weechat_infolist_next (ptr_infolist))
-        {
-            ptr_buffer = weechat_infolist_pointer (ptr_infolist, "pointer");
-            if (ptr_buffer)
-            {
-                ptr_logger_buffer = logger_buffer_search_buffer (ptr_buffer);
-                if (ptr_logger_buffer)
-                {
-                    snprintf (status, sizeof (status),
-                              _("logging (level: %d)"),
-                              ptr_logger_buffer->log_level);
-                }
-                else
-                {
-                    snprintf (status, sizeof (status), "%s", _("not logging"));
-                }
-                weechat_printf (NULL,
-                                "  %s[%s%d%s]%s (%s) %s%s%s: %s%s%s%s",
-                                weechat_color("chat_delimiters"),
-                                weechat_color("chat"),
-                                weechat_infolist_integer (ptr_infolist, "number"),
-                                weechat_color("chat_delimiters"),
-                                weechat_color("chat"),
-                                weechat_infolist_string (ptr_infolist, "plugin_name"),
-                                weechat_color("chat_buffer"),
-                                weechat_infolist_string (ptr_infolist, "name"),
-                                weechat_color("chat"),
-                                status,
-                                (ptr_logger_buffer) ? " (" : "",
-                                (ptr_logger_buffer) ?
-                                ((ptr_logger_buffer->log_filename) ?
-                                 ptr_logger_buffer->log_filename : _("log not started")) : "",
-                                (ptr_logger_buffer) ? ")" : "");
-            }
-        }
-        weechat_infolist_free (ptr_infolist);
-    }
-}
-
-/*
- * Enables/disables logging on a buffer.
- */
-
-void
-logger_set_buffer (struct t_gui_buffer *buffer, const char *value)
-{
-    char *name;
-    struct t_config_option *ptr_option;
-
-    name = logger_build_option_name (buffer);
-    if (!name)
-        return;
-
-    if (logger_config_set_level (name, value) != WEECHAT_CONFIG_OPTION_SET_ERROR)
-    {
-        ptr_option = logger_config_get_level (name);
-        if (ptr_option)
-        {
-            weechat_printf (NULL, _("%s: \"%s\" => level %d"),
-                            LOGGER_PLUGIN_NAME, name,
-                            weechat_config_integer (ptr_option));
-        }
-    }
-
-    free (name);
-}
-
 /*
  * Flushes all log files.
  */
@@ -844,54 +819,13 @@ logger_flush ()
                                           ptr_logger_buffer->log_filename);
             }
             fflush (ptr_logger_buffer->log_file);
+            if (weechat_config_boolean (logger_config_file_fsync))
+                fsync (fileno (ptr_logger_buffer->log_file));
             ptr_logger_buffer->flush_needed = 0;
         }
     }
 }
 
-/*
- * Callback for command "/logger".
- */
-
-int
-logger_command_cb (const void *pointer, void *data,
-                   struct t_gui_buffer *buffer,
-                   int argc, char **argv, char **argv_eol)
-{
-    /* make C compiler happy */
-    (void) pointer;
-    (void) data;
-    (void) argv_eol;
-
-    if ((argc == 1)
-        || ((argc == 2) && (weechat_strcasecmp (argv[1], "list") == 0)))
-    {
-        logger_list ();
-        return WEECHAT_RC_OK;
-    }
-
-    if (weechat_strcasecmp (argv[1], "set") == 0)
-    {
-        if (argc > 2)
-            logger_set_buffer (buffer, argv[2]);
-        return WEECHAT_RC_OK;
-    }
-
-    if (weechat_strcasecmp (argv[1], "flush") == 0)
-    {
-        logger_flush ();
-        return WEECHAT_RC_OK;
-    }
-
-    if (weechat_strcasecmp (argv[1], "disable") == 0)
-    {
-        logger_set_buffer (buffer, "0");
-        return WEECHAT_RC_OK;
-    }
-
-    WEECHAT_COMMAND_ERROR;
-}
-
 /*
  * Callback for signal "buffer_opened".
  */
@@ -953,130 +887,6 @@ logger_buffer_renamed_signal_cb (const void *pointer, void *data,
     return WEECHAT_RC_OK;
 }
 
-/*
- * Displays backlog for a buffer (by reading end of log file).
- */
-
-void
-logger_backlog (struct t_gui_buffer *buffer, const char *filename, int lines)
-{
-    const char *charset;
-    struct t_logger_line *last_lines, *ptr_lines;
-    char *pos_message, *pos_tab, *error, *message;
-    time_t datetime, time_now;
-    struct tm tm_line;
-    int num_lines;
-
-    charset = weechat_info_get ("charset_terminal", "");
-
-    weechat_buffer_set (buffer, "print_hooks_enabled", "0");
-
-    num_lines = 0;
-    last_lines = logger_tail_file (filename, lines);
-    ptr_lines = last_lines;
-    while (ptr_lines)
-    {
-        datetime = 0;
-        pos_message = strchr (ptr_lines->data, '\t');
-        if (pos_message)
-        {
-            /* initialize structure, because strptime does not do it */
-            memset (&tm_line, 0, sizeof (struct tm));
-            /*
-             * we get current time to initialize daylight saving time in
-             * structure tm_line, otherwise printed time will be shifted
-             * and will not use DST used on machine
-             */
-            time_now = time (NULL);
-            localtime_r (&time_now, &tm_line);
-            pos_message[0] = '\0';
-            error = strptime (ptr_lines->data,
-                              weechat_config_string (logger_config_file_time_format),
-                              &tm_line);
-            if (error && !error[0] && (tm_line.tm_year > 0))
-                datetime = mktime (&tm_line);
-            pos_message[0] = '\t';
-        }
-        pos_message = (pos_message && (datetime != 0)) ?
-            pos_message + 1 : ptr_lines->data;
-        message = (charset) ?
-            weechat_iconv_to_internal (charset, pos_message) : strdup (pos_message);
-        if (message)
-        {
-            pos_tab = strchr (message, '\t');
-            if (pos_tab)
-                pos_tab[0] = '\0';
-            weechat_printf_date_tags (buffer, datetime,
-                                      "no_highlight,notify_none,logger_backlog",
-                                      "%s%s%s%s%s",
-                                      weechat_color (weechat_config_string (logger_config_color_backlog_line)),
-                                      message,
-                                      (pos_tab) ? "\t" : "",
-                                      (pos_tab) ? weechat_color (weechat_config_string (logger_config_color_backlog_line)) : "",
-                                      (pos_tab) ? pos_tab + 1 : "");
-            if (pos_tab)
-                pos_tab[0] = '\t';
-            free (message);
-        }
-        num_lines++;
-        ptr_lines = ptr_lines->next_line;
-    }
-    if (last_lines)
-        logger_tail_free (last_lines);
-    if (num_lines > 0)
-    {
-        weechat_printf_date_tags (buffer, datetime,
-                                  "no_highlight,notify_none,logger_backlog_end",
-                                  _("%s===\t%s========== End of backlog (%d lines) =========="),
-                                  weechat_color (weechat_config_string (logger_config_color_backlog_end)),
-                                  weechat_color (weechat_config_string (logger_config_color_backlog_end)),
-                                  num_lines);
-        weechat_buffer_set (buffer, "unread", "");
-    }
-    weechat_buffer_set (buffer, "print_hooks_enabled", "1");
-}
-
-/*
- * Callback for signal "logger_backlog".
- */
-
-int
-logger_backlog_signal_cb (const void *pointer, void *data,
-                          const char *signal,
-                          const char *type_data, void *signal_data)
-{
-    struct t_logger_buffer *ptr_logger_buffer;
-
-    /* make C compiler happy */
-    (void) pointer;
-    (void) data;
-    (void) signal;
-    (void) type_data;
-
-    if (weechat_config_integer (logger_config_look_backlog) >= 0)
-    {
-        ptr_logger_buffer = logger_buffer_search_buffer (signal_data);
-        if (ptr_logger_buffer && ptr_logger_buffer->log_enabled)
-        {
-            if (!ptr_logger_buffer->log_filename)
-                logger_set_log_filename (ptr_logger_buffer);
-
-            if (ptr_logger_buffer->log_filename)
-            {
-                ptr_logger_buffer->log_enabled = 0;
-
-                logger_backlog (signal_data,
-                                ptr_logger_buffer->log_filename,
-                                weechat_config_integer (logger_config_look_backlog));
-
-                ptr_logger_buffer->log_enabled = 1;
-            }
-        }
-    }
-
-    return WEECHAT_RC_OK;
-}
-
 /*
  * Callback for signal "logger_start".
  */
@@ -1248,8 +1058,9 @@ logger_print_cb (const void *pointer, void *data,
 {
     struct t_logger_buffer *ptr_logger_buffer;
     struct tm *date_tmp;
-    char buf_time[256];
-    int line_log_level, prefix_is_nick;
+    char buf_time[256], *prefix_ansi, *message_ansi;
+    const char *ptr_prefix, *ptr_message;
+    int line_log_level, prefix_is_nick, color_lines;
 
     /* make C compiler happy */
     (void) pointer;
@@ -1259,32 +1070,56 @@ logger_print_cb (const void *pointer, void *data,
 
     logger_get_line_tag_info (tags_count, tags, &line_log_level,
                               &prefix_is_nick);
-    if (line_log_level >= 0)
-    {
-        ptr_logger_buffer = logger_buffer_search_buffer (buffer);
-        if (ptr_logger_buffer
-            && ptr_logger_buffer->log_enabled
-            && (date > 0)
-            && (line_log_level <= ptr_logger_buffer->log_level))
-        {
-            buf_time[0] = '\0';
-            date_tmp = localtime (&date);
-            if (date_tmp)
-            {
-                if (strftime (buf_time, sizeof (buf_time) - 1,
-                              weechat_config_string (logger_config_file_time_format),
-                              date_tmp) == 0)
-                    buf_time[0] = '\0';
-            }
+    if (line_log_level < 0)
+        return WEECHAT_RC_OK;
 
-            logger_write_line (ptr_logger_buffer,
-                               "%s\t%s%s%s\t%s",
-                               buf_time,
-                               (prefix && prefix_is_nick) ? weechat_config_string (logger_config_file_nick_prefix) : "",
-                               (prefix) ? prefix : "",
-                               (prefix && prefix_is_nick) ? weechat_config_string (logger_config_file_nick_suffix) : "",
-                               message);
+    ptr_logger_buffer = logger_buffer_search_buffer (buffer);
+    if (ptr_logger_buffer
+        && ptr_logger_buffer->log_enabled
+        && (date > 0)
+        && (line_log_level <= ptr_logger_buffer->log_level))
+    {
+        prefix_ansi = NULL;
+        message_ansi = NULL;
+        color_lines = weechat_config_boolean (logger_config_file_color_lines);
+        if (color_lines)
+        {
+            prefix_ansi = weechat_hook_modifier_exec ("color_encode_ansi",
+                                                      NULL, prefix);
+            message_ansi = weechat_hook_modifier_exec ("color_encode_ansi",
+                                                      NULL, message);
+            ptr_prefix = prefix_ansi;
+            ptr_message = message_ansi;
         }
+        else
+        {
+            ptr_prefix = prefix;
+            ptr_message = message;
+        }
+        buf_time[0] = '\0';
+        date_tmp = localtime (&date);
+        if (date_tmp)
+        {
+            if (strftime (buf_time, sizeof (buf_time) - 1,
+                          weechat_config_string (logger_config_file_time_format),
+                          date_tmp) == 0)
+                buf_time[0] = '\0';
+        }
+
+        logger_write_line (
+            ptr_logger_buffer,
+            "%s\t%s%s%s\t%s%s",
+            buf_time,
+            (ptr_prefix && prefix_is_nick) ? weechat_config_string (logger_config_file_nick_prefix) : "",
+            (ptr_prefix) ? ptr_prefix : "",
+            (ptr_prefix && prefix_is_nick) ? weechat_config_string (logger_config_file_nick_suffix) : "",
+            (color_lines) ? "\x1B[0m" : "",
+            ptr_message);
+
+        if (prefix_ansi)
+            free (prefix_ansi);
+        if (message_ansi)
+            free (message_ansi);
     }
 
     return WEECHAT_RC_OK;
@@ -1325,47 +1160,7 @@ weechat_plugin_init (struct t_weechat_plugin *plugin, int argc, char *argv[])
 
     logger_config_read ();
 
-    /* command /logger */
-    weechat_hook_command (
-        "logger",
-        N_("logger plugin configuration"),
-        N_("list"
-           " || set <level>"
-           " || flush"
-           " || disable"),
-        N_("   list: show logging status for opened buffers\n"
-           "    set: set logging level on current buffer\n"
-           "  level: level for messages to be logged (0 = logging disabled, "
-           "1 = a few messages (most important) .. 9 = all messages)\n"
-           "  flush: write all log files now\n"
-           "disable: disable logging on current buffer (set level to 0)\n"
-           "\n"
-           "Options \"logger.level.*\" and \"logger.mask.*\" can be used to set "
-           "level or mask for a buffer, or buffers beginning with name.\n"
-           "\n"
-           "Log levels used by IRC plugin:\n"
-           "  1: user message, notice, private\n"
-           "  2: nick change\n"
-           "  3: server message\n"
-           "  4: join/part/quit\n"
-           "  9: all other messages\n"
-           "\n"
-           "Examples:\n"
-           "  set level to 5 for current buffer:\n"
-           "    /logger set 5\n"
-           "  disable logging for current buffer:\n"
-           "    /logger disable\n"
-           "  set level to 3 for all IRC buffers:\n"
-           "    /set logger.level.irc 3\n"
-           "  disable logging for main WeeChat buffer:\n"
-           "    /set logger.level.core.weechat 0\n"
-           "  use a directory per IRC server and a file per channel inside:\n"
-           "    /set logger.mask.irc \"$server/$channel.weechatlog\""),
-        "list"
-        " || set 1|2|3|4|5|6|7|8|9"
-        " || flush"
-        " || disable",
-        &logger_command_cb, NULL, NULL);
+    logger_command_init ();
 
     logger_start_buffer_all (1);
 
@@ -1384,7 +1179,7 @@ weechat_plugin_init (struct t_weechat_plugin *plugin, int argc, char *argv[])
     weechat_hook_signal ("day_changed",
                          &logger_day_changed_signal_cb, NULL, NULL);
 
-    weechat_hook_print (NULL, NULL, NULL, 1, &logger_print_cb, NULL, NULL);
+    logger_config_color_lines_change (NULL, NULL, NULL);
 
     logger_info_init ();
 
@@ -1401,10 +1196,16 @@ weechat_plugin_end (struct t_weechat_plugin *plugin)
     /* make C compiler happy */
     (void) plugin;
 
-    if (logger_timer)
+    if (logger_hook_print)
     {
-        weechat_unhook (logger_timer);
-        logger_timer = NULL;
+        weechat_unhook (logger_hook_print);
+        logger_hook_print = NULL;
+    }
+
+    if (logger_hook_timer)
+    {
+        weechat_unhook (logger_hook_timer);
+        logger_hook_timer = NULL;
     }
 
     logger_config_write ();
