@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2017 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -14,7 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef WEECHAT_PLUGIN_PLUGIN_API_H
@@ -29,6 +29,16 @@ extern const char *plugin_api_gettext (const char *string);
 extern const char *plugin_api_ngettext (const char *single, const char *plural,
                                         int count);
 
+/* crypto */
+extern int plugin_api_crypto_hash (const void *data, int data_size,
+                                   const char *hash_algo,
+                                   void *hash, int *hash_size);
+extern int plugin_api_crypto_hash_pbkdf2 (const void *data, int data_size,
+                                          const char *hash_algo,
+                                          const void *salt, int salt_size,
+                                          int iterations,
+                                          void *hash, int *hash_size);
+
 /* config */
 extern void plugin_api_config_file_option_free (struct t_config_option *option);
 extern struct t_config_option *plugin_api_config_get (const char *option_name);
@@ -50,6 +60,10 @@ extern const char *plugin_api_prefix (const char *prefix);
 extern const char *plugin_api_color (const char *color_name);
 
 /* command */
+extern int plugin_api_command_options (struct t_weechat_plugin *plugin,
+                                       struct t_gui_buffer *buffer,
+                                       const char *command,
+                                       struct t_hashtable *options);
 extern int plugin_api_command (struct t_weechat_plugin *plugin,
                                struct t_gui_buffer *buffer,
                                const char *command);
