@@ -1,7 +1,7 @@
 /*
  * plugin-script.c - common functions used by script plugins
  *
- * Copyright (C) 2003-2017 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <stdlib.h>
@@ -33,56 +33,106 @@
 
 #include "weechat-plugin.h"
 #include "plugin-script.h"
-
-
-#define SCRIPT_OPTION_CHECK_LICENSE "check_license"
-
-int script_option_check_license = 0;
+#include "plugin-script-config.h"
 
 
 /*
- * Reads script configuration.
+ * Displays name and version of interpreter used.
  */
 
 void
-plugin_script_config_read (struct t_weechat_plugin *weechat_plugin)
+plugin_script_display_interpreter (struct t_weechat_plugin *weechat_plugin,
+                                   int indent)
 {
-    const char *string;
+    const char *ptr_name, *ptr_version;
 
-    string = weechat_config_get_plugin (SCRIPT_OPTION_CHECK_LICENSE);
-    if (!string)
+    ptr_name = weechat_hashtable_get (weechat_plugin->variables,
+                                      "interpreter_name");
+    ptr_version = weechat_hashtable_get (weechat_plugin->variables,
+                                         "interpreter_version");
+    if (ptr_name)
     {
-        weechat_config_set_plugin (SCRIPT_OPTION_CHECK_LICENSE, "off");
-        string = weechat_config_get_plugin (SCRIPT_OPTION_CHECK_LICENSE);
+        weechat_printf (NULL,
+                        "%s%s: %s",
+                        (indent) ? "  " : "",
+                        ptr_name,
+                        (ptr_version && ptr_version[0]) ? ptr_version : "(?)");
     }
-    if (string && (weechat_config_string_to_boolean (string) > 0))
-        script_option_check_license = 1;
-    else
-        script_option_check_license = 0;
 }
 
 /*
- * Callback for changes on configuration option.
+ * Callback for signal "debug_libs".
  */
 
 int
-plugin_script_config_cb (const void *pointer, void *data,
-                         const char *option, const char *value)
+plugin_script_signal_debug_libs_cb (const void *pointer, void *data,
+                                    const char *signal,
+                                    const char *type_data,
+                                    void *signal_data)
 {
     struct t_weechat_plugin *plugin;
 
     /* make C compiler happy */
     (void) data;
-    (void) option;
-    (void) value;
+    (void) signal;
+    (void) type_data;
+    (void) signal_data;
 
     plugin = (struct t_weechat_plugin *)pointer;
 
-    plugin_script_config_read (plugin);
+    plugin_script_display_interpreter (plugin, 1);
 
     return WEECHAT_RC_OK;
 }
 
+/*
+ * Callback for info "xxx_interpreter".
+ */
+
+char *
+plugin_script_info_interpreter_cb (const void *pointer, void *data,
+                                   const char *info_name,
+                                   const char *arguments)
+{
+    struct t_weechat_plugin *weechat_plugin;
+    const char *ptr_interpreter;
+
+    /* make C compiler happy */
+    (void) data;
+    (void) info_name;
+    (void) arguments;
+
+    weechat_plugin = (struct t_weechat_plugin *)pointer;
+
+    ptr_interpreter = weechat_hashtable_get (weechat_plugin->variables,
+                                             "interpreter_name");
+    return (ptr_interpreter) ? strdup (ptr_interpreter) : NULL;
+}
+
+/*
+ * Callback for info "xxx_version".
+ */
+
+char *
+plugin_script_info_version_cb (const void *pointer, void *data,
+                               const char *info_name,
+                               const char *arguments)
+{
+    struct t_weechat_plugin *weechat_plugin;
+    const char *ptr_version;
+
+    /* make C compiler happy */
+    (void) data;
+    (void) info_name;
+    (void) arguments;
+
+    weechat_plugin = (struct t_weechat_plugin *)pointer;
+
+    ptr_version = weechat_hashtable_get (weechat_plugin->variables,
+                                         "interpreter_version");
+    return (ptr_version) ? strdup (ptr_version) : NULL;
+}
+
 /*
  * Creates directories for plugin in WeeChat home:
  * - ~/.weechat/XXX/
@@ -119,54 +169,43 @@ plugin_script_create_dirs (struct t_weechat_plugin *weechat_plugin)
 void
 plugin_script_init (struct t_weechat_plugin *weechat_plugin,
                     int argc, char *argv[],
-                    struct t_plugin_script_init *init)
+                    struct t_plugin_script_data *plugin_data)
 {
-    char *string, *completion, signal_name[128];
+    char string[512], *completion;
     char *action_signals[] = { "install", "remove", "autoload", NULL };
-    int length, i, auto_load_scripts;
+    int i, auto_load_scripts;
 
-    /* read script configuration */
-    plugin_script_config_read (weechat_plugin);
+    /* initialize script configuration file (file: "<language>.conf") */
+    plugin_script_config_init (weechat_plugin, plugin_data);
 
-    /* add hook for configuration option */
-    length = strlen (weechat_plugin->name) + 64;
-    string = malloc (length);
-    if (string)
-    {
-        snprintf (string, length, "plugins.var.%s.%s",
-                  weechat_plugin->name, SCRIPT_OPTION_CHECK_LICENSE);
-        weechat_hook_config (string,
-                             &plugin_script_config_cb, weechat_plugin, NULL);
-        free (string);
-    }
+    /* read configuration file */
+    weechat_config_read (*plugin_data->config_file);
 
     /* create directories in WeeChat home */
     plugin_script_create_dirs (weechat_plugin);
 
     /* add command */
     completion = NULL;
-    length = strlen (weechat_plugin->name) + 16;
-    string = malloc (length);
-    if (string)
-    {
-        snprintf (string, length, "%%(%s_script)",
-                  weechat_plugin->name);
-        completion = weechat_string_replace ("list %s"
-                                             " || listfull %s"
-                                             " || load %(filename)"
-                                             " || autoload"
-                                             " || reload %s"
-                                             " || unload %s",
-                                             "%s",
-                                             string);
-    }
+    snprintf (string, sizeof (string), "%%(%s_script)", weechat_plugin->name);
+    completion = weechat_string_replace ("list %s"
+                                         " || listfull %s"
+                                         " || load %(filename)"
+                                         " || autoload"
+                                         " || reload %s"
+                                         " || unload %s"
+                                         " || eval"
+                                         " || version",
+                                         "%s",
+                                         string);
     weechat_hook_command (
         weechat_plugin->name,
         N_("list/load/unload scripts"),
         N_("list|listfull [<name>]"
            " || load [-q] <filename>"
            " || autoload"
-           " || reload|unload [-q] [<name>]"),
+           " || reload|unload [-q] [<name>]"
+           " || eval [-o|-oc] <code>"
+           " || version"),
         N_("    list: list loaded scripts\n"
            "listfull: list loaded scripts (verbose)\n"
            "    load: load a script\n"
@@ -175,60 +214,75 @@ plugin_script_init (struct t_weechat_plugin *weechat_plugin,
            "then load all scripts in \"autoload\" directory)\n"
            "  unload: unload a script (if no name given, unload all scripts)\n"
            "filename: script (file) to load\n"
+           "      -q: quiet mode: do not display messages\n"
            "    name: a script name (name used in call to \"register\" "
            "function)\n"
-           "      -q: quiet mode: do not display messages\n"
+           "    eval: evaluate source code and display result on current "
+           "buffer\n"
+           "      -o: send evaluation result to the buffer without executing "
+           "commands\n"
+           "     -oc: send evaluation result to the buffer and execute "
+           "commands\n"
+           "    code: source code to evaluate\n"
+           " version: display the version of interpreter used\n"
            "\n"
            "Without argument, this command lists all loaded scripts."),
         completion,
-        init->callback_command, NULL, NULL);
-    if (string)
-        free (string);
+        plugin_data->callback_command, NULL, NULL);
     if (completion)
         free (completion);
 
     /* add completion, hdata and infolist */
-    length = strlen (weechat_plugin->name) + 64;
-    string = malloc (length);
-    if (string)
-    {
-        snprintf (string, length, "%s_script", weechat_plugin->name);
-        weechat_hook_completion (string, N_("list of scripts"),
-                                 init->callback_completion, NULL, NULL);
-        weechat_hook_hdata (string, N_("list of scripts"),
-                            init->callback_hdata, weechat_plugin, NULL);
-        weechat_hook_infolist (string, N_("list of scripts"),
-                               N_("script pointer (optional)"),
-                               N_("script name (wildcard \"*\" is allowed) "
-                                  "(optional)"),
-                               init->callback_infolist, NULL, NULL);
-        free (string);
-    }
+    snprintf (string, sizeof (string), "%s_script", weechat_plugin->name);
+    weechat_hook_completion (string, N_("list of scripts"),
+                             plugin_data->callback_completion, NULL, NULL);
+    weechat_hook_hdata (string, N_("list of scripts"),
+                        plugin_data->callback_hdata, weechat_plugin, NULL);
+    weechat_hook_infolist (string, N_("list of scripts"),
+                           N_("script pointer (optional)"),
+                           N_("script name (wildcard \"*\" is allowed) "
+                              "(optional)"),
+                           plugin_data->callback_infolist, NULL, NULL);
+    snprintf (string, sizeof (string), "%s_eval", weechat_plugin->name);
+    weechat_hook_info (string, N_("evaluation of source code"),
+                       N_("source code to execute"),
+                       plugin_data->callback_info_eval, NULL, NULL);
 
     /* add signal for "debug_dump" */
     weechat_hook_signal ("debug_dump",
-                         init->callback_signal_debug_dump, NULL, NULL);
+                         plugin_data->callback_signal_debug_dump, NULL, NULL);
 
     /* add signal for "debug_libs" */
     weechat_hook_signal ("debug_libs",
-                         init->callback_signal_debug_libs, NULL, NULL);
+                         plugin_script_signal_debug_libs_cb,
+                         weechat_plugin, NULL);
 
     /* add signals for script actions (install/remove/autoload) */
     for (i = 0; action_signals[i]; i++)
     {
-        snprintf (signal_name, sizeof (signal_name), "%s_script_%s",
-                  weechat_plugin->name,
-                  action_signals[i]);
-        weechat_hook_signal (signal_name,
-                             init->callback_signal_script_action, NULL, NULL);
+        snprintf (string, sizeof (string),
+                  "%s_script_%s", weechat_plugin->name, action_signals[i]);
+        weechat_hook_signal (
+            string,
+            plugin_data->callback_signal_script_action, NULL, NULL);
     }
 
+    /* add infos */
+    snprintf (string, sizeof (string), "%s_interpreter", weechat_plugin->name);
+    weechat_hook_info (string, N_("name of the interpreter used"), NULL,
+                       &plugin_script_info_interpreter_cb,
+                       weechat_plugin, NULL);
+    snprintf (string, sizeof (string), "%s_version", weechat_plugin->name);
+    weechat_hook_info (string, N_("version of the interpreter used"), NULL,
+                       &plugin_script_info_version_cb,
+                       weechat_plugin, NULL);
+
     /* parse arguments */
     auto_load_scripts = 1;
     for (i = 0; i < argc; i++)
     {
         if ((strcmp (argv[i], "-s") == 0)
-                 || (strcmp (argv[i], "--no-script") == 0))
+            || (strcmp (argv[i], "--no-script") == 0))
         {
             auto_load_scripts = 0;
         }
@@ -237,7 +291,8 @@ plugin_script_init (struct t_weechat_plugin *weechat_plugin,
     /* autoload scripts */
     if (auto_load_scripts)
     {
-        plugin_script_auto_load (weechat_plugin, init->callback_load_file);
+        plugin_script_auto_load (weechat_plugin,
+                                 plugin_data->callback_load_file);
     }
 }
 
@@ -273,22 +328,24 @@ plugin_script_valid (struct t_plugin_script *scripts,
  * Converts a pointer to a string for usage in a script.
  *
  * Returns string with format "0x12345678".
- *
- * Note: result must be freed after use.
  */
 
-char *
+const char *
 plugin_script_ptr2str (void *pointer)
 {
-    char pointer_str[128];
+    static char str_pointer[32][32];
+    static int index_pointer = 0;
+
+    index_pointer = (index_pointer + 1) % 32;
+    str_pointer[index_pointer][0] = '\0';
 
     if (!pointer)
-        return strdup ("");
+        return str_pointer[index_pointer];
 
-    snprintf (pointer_str, sizeof (pointer_str),
-              "0x%lx", (long unsigned int)pointer);
+    snprintf (str_pointer[index_pointer], sizeof (str_pointer[index_pointer]),
+              "0x%lx", (unsigned long)pointer);
 
-    return strdup (pointer_str);
+    return str_pointer[index_pointer];
 }
 
 /*
@@ -302,7 +359,7 @@ plugin_script_str2ptr (struct t_weechat_plugin *weechat_plugin,
                        const char *script_name, const char *function_name,
                        const char *str_pointer)
 {
-    long unsigned int value;
+    unsigned long value;
     int rc;
     struct t_gui_buffer *ptr_buffer;
 
@@ -404,8 +461,7 @@ plugin_script_auto_load (struct t_weechat_plugin *weechat_plugin,
                          void (*callback)(void *data,
                                           const char *filename))
 {
-    const char *dir_home;
-    char *dir_name;
+    char *dir_home, *dir_name;
     int dir_length;
 
     /* build directory, adding WeeChat home */
@@ -415,12 +471,16 @@ plugin_script_auto_load (struct t_weechat_plugin *weechat_plugin,
     dir_length = strlen (dir_home) + strlen (weechat_plugin->name) + 16;
     dir_name = malloc (dir_length);
     if (!dir_name)
+    {
+        free (dir_home);
         return;
+    }
 
     snprintf (dir_name, dir_length,
               "%s/%s/autoload", dir_home, weechat_plugin->name);
     weechat_exec_on_files (dir_name, 0, 0, callback, NULL);
 
+    free (dir_home);
     free (dir_name);
 }
 
@@ -482,8 +542,7 @@ char *
 plugin_script_search_path (struct t_weechat_plugin *weechat_plugin,
                            const char *filename)
 {
-    char *final_name;
-    const char *dir_home, *dir_system;
+    char *final_name, *dir_home, *dir_system;
     int length;
     struct stat st;
 
@@ -503,7 +562,10 @@ plugin_script_search_path (struct t_weechat_plugin *weechat_plugin,
                       "%s/%s/autoload/%s",
                       dir_home, weechat_plugin->name, filename);
             if ((stat (final_name, &st) == 0) && (st.st_size > 0))
+            {
+                free (dir_home);
                 return final_name;
+            }
             free (final_name);
         }
 
@@ -516,7 +578,10 @@ plugin_script_search_path (struct t_weechat_plugin *weechat_plugin,
             snprintf (final_name, length,
                       "%s/%s/%s", dir_home, weechat_plugin->name, filename);
             if ((stat (final_name, &st) == 0) && (st.st_size > 0))
+            {
+                free (dir_home);
                 return final_name;
+            }
             free (final_name);
         }
 
@@ -528,9 +593,13 @@ plugin_script_search_path (struct t_weechat_plugin *weechat_plugin,
             snprintf (final_name, length,
                       "%s/%s", dir_home, filename);
             if ((stat (final_name, &st) == 0) && (st.st_size > 0))
+            {
+                free (dir_home);
                 return final_name;
+            }
             free (final_name);
         }
+        free (dir_home);
     }
 
     /* try WeeChat system dir */
@@ -545,9 +614,13 @@ plugin_script_search_path (struct t_weechat_plugin *weechat_plugin,
             snprintf (final_name,length,
                       "%s/%s/%s", dir_system, weechat_plugin->name, filename);
             if ((stat (final_name, &st) == 0) && (st.st_size > 0))
+            {
+                free (dir_system);
                 return final_name;
+            }
             free (final_name);
         }
+        free (dir_system);
     }
 
     return strdup (filename);
@@ -618,6 +691,41 @@ plugin_script_insert_sorted (struct t_weechat_plugin *weechat_plugin,
     }
 }
 
+/*
+ * Allocates a new script.
+ *
+ * Returns pointer to new script, NULL if error.
+ */
+
+struct t_plugin_script *
+plugin_script_alloc (const char *filename, const char *name,
+                     const char *author, const char *version,
+                     const char *license, const char *description,
+                     const char *shutdown_func, const char *charset)
+{
+    struct t_plugin_script *new_script;
+
+    new_script = malloc (sizeof (*new_script));
+    if (!new_script)
+        return NULL;
+
+    new_script->filename = strdup (filename);
+    new_script->interpreter = NULL;
+    new_script->name = strdup (name);
+    new_script->author = strdup (author);
+    new_script->version = strdup (version);
+    new_script->license = strdup (license);
+    new_script->description = strdup (description);
+    new_script->shutdown_func = (shutdown_func) ?
+        strdup (shutdown_func) : NULL;
+    new_script->charset = (charset) ? strdup (charset) : NULL;
+    new_script->unloading = 0;
+    new_script->prev_script = NULL;
+    new_script->next_script = NULL;
+
+    return new_script;
+}
+
 /*
  * Adds a script to list of scripts.
  *
@@ -626,8 +734,7 @@ plugin_script_insert_sorted (struct t_weechat_plugin *weechat_plugin,
 
 struct t_plugin_script *
 plugin_script_add (struct t_weechat_plugin *weechat_plugin,
-                   struct t_plugin_script **scripts,
-                   struct t_plugin_script **last_script,
+                   struct t_plugin_script_data *plugin_data,
                    const char *filename, const char *name, const char *author,
                    const char *version, const char *license,
                    const char *description, const char *shutdown_func,
@@ -644,7 +751,7 @@ plugin_script_add (struct t_weechat_plugin *weechat_plugin,
         return NULL;
     }
 
-    if (script_option_check_license
+    if (weechat_config_boolean (*(plugin_data->config_look_check_license))
         && (weechat_strcmp_ignore_chars (weechat_plugin->license, license,
                                          "0123456789-.,/\\()[]{}", 0) != 0))
     {
@@ -655,32 +762,27 @@ plugin_script_add (struct t_weechat_plugin *weechat_plugin,
                         license, name, weechat_plugin->license);
     }
 
-    new_script = malloc (sizeof (*new_script));
-    if (new_script)
+    new_script = plugin_script_alloc (filename, name, author, version, license,
+                                      description, shutdown_func, charset);
+    if (!new_script)
     {
-        new_script->filename = strdup (filename);
-        new_script->interpreter = NULL;
-        new_script->name = strdup (name);
-        new_script->author = strdup (author);
-        new_script->version = strdup (version);
-        new_script->license = strdup (license);
-        new_script->description = strdup (description);
-        new_script->shutdown_func = (shutdown_func) ?
-            strdup (shutdown_func) : NULL;
-        new_script->charset = (charset) ? strdup (charset) : NULL;
-        new_script->unloading = 0;
-
-        plugin_script_insert_sorted (weechat_plugin, scripts, last_script,
-                                     new_script);
-
-        return new_script;
+        weechat_printf (NULL,
+                        _("%s: error loading script \"%s\" "
+                          "(not enough memory)"),
+                        weechat_plugin->name, name);
+        return NULL;
     }
 
-    weechat_printf (NULL,
-                    _("%s: error loading script \"%s\" (not enough memory)"),
-                    weechat_plugin->name, name);
+    /* add script to the list (except the internal "eval" fake script) */
+    if (strcmp (new_script->name, WEECHAT_SCRIPT_EVAL_NAME) != 0)
+    {
+        plugin_script_insert_sorted (weechat_plugin,
+                                     plugin_data->scripts,
+                                     plugin_data->last_script,
+                                     new_script);
+    }
 
-    return NULL;
+    return new_script;
 }
 
 /*
@@ -909,6 +1011,33 @@ plugin_script_remove_configs (struct t_weechat_plugin *weechat_plugin,
     }
 }
 
+/*
+ * Frees a script.
+ */
+
+void
+plugin_script_free (struct t_plugin_script *script)
+{
+    if (script->filename)
+        free (script->filename);
+    if (script->name)
+        free (script->name);
+    if (script->author)
+        free (script->author);
+    if (script->version)
+        free (script->version);
+    if (script->license)
+        free (script->license);
+    if (script->description)
+        free (script->description);
+    if (script->shutdown_func)
+        free (script->shutdown_func);
+    if (script->charset)
+        free (script->charset);
+
+    free (script);
+}
+
 /*
  * Removes a script from list of scripts.
  */
@@ -930,24 +1059,6 @@ plugin_script_remove (struct t_weechat_plugin *weechat_plugin,
     /* remove all hooks created by this script */
     weechat_unhook_all (script->name);
 
-    /* free data */
-    if (script->filename)
-        free (script->filename);
-    if (script->name)
-        free (script->name);
-    if (script->author)
-        free (script->author);
-    if (script->version)
-        free (script->version);
-    if (script->license)
-        free (script->license);
-    if (script->description)
-        free (script->description);
-    if (script->shutdown_func)
-        free (script->shutdown_func);
-    if (script->charset)
-        free (script->charset);
-
     /* remove script from list */
     if (script->prev_script)
         (script->prev_script)->next_script = script->next_script;
@@ -958,8 +1069,8 @@ plugin_script_remove (struct t_weechat_plugin *weechat_plugin,
     if (*last_script == script)
         *last_script = script->prev_script;
 
-    /* free script */
-    free (script);
+    /* free data and script */
+    plugin_script_free (script);
 }
 
 /*
@@ -976,8 +1087,8 @@ plugin_script_completion (struct t_weechat_plugin *weechat_plugin,
     for (ptr_script = scripts; ptr_script;
          ptr_script = ptr_script->next_script)
     {
-        weechat_hook_completion_list_add (completion, ptr_script->name,
-                                          0, WEECHAT_LIST_POS_SORT);
+        weechat_completion_list_add (completion, ptr_script->name,
+                                     0, WEECHAT_LIST_POS_SORT);
     }
 }
 
@@ -1101,13 +1212,13 @@ void
 plugin_script_action_install (struct t_weechat_plugin *weechat_plugin,
                               struct t_plugin_script *scripts,
                               void (*script_unload)(struct t_plugin_script *script),
-                              int (*script_load)(const char *filename),
+                              struct t_plugin_script *(*script_load)(const char *filename,
+                                                                     const char *code),
                               int *quiet,
                               char **list)
 {
     char **argv, *name, *ptr_base_name, *base_name, *new_path, *autoload_path;
-    char *symlink_path, str_signal[128], *ptr_list;
-    const char *dir_home, *dir_separator;
+    char *symlink_path, str_signal[128], *ptr_list, *dir_home, *dir_separator;
     int argc, i, length, rc, autoload, existing_script, script_loaded;
     struct t_plugin_script *ptr_script;
 
@@ -1140,7 +1251,11 @@ plugin_script_action_install (struct t_weechat_plugin *weechat_plugin,
         }
     }
 
-    argv = weechat_string_split (ptr_list, ",", 0, 0, &argc);
+    argv = weechat_string_split (ptr_list, ",", NULL,
+                                 WEECHAT_STRING_SPLIT_STRIP_LEFT
+                                 | WEECHAT_STRING_SPLIT_STRIP_RIGHT
+                                 | WEECHAT_STRING_SPLIT_COLLAPSE_SEPS,
+                                 0, &argc);
     if (argv)
     {
         for (i = 0; i < argc; i++)
@@ -1204,6 +1319,8 @@ plugin_script_action_install (struct t_weechat_plugin *weechat_plugin,
                                         free (symlink_path);
                                     }
                                     free (autoload_path);
+                                    if (dir_separator)
+                                        free (dir_separator);
                                 }
                             }
 
@@ -1214,7 +1331,7 @@ plugin_script_action_install (struct t_weechat_plugin *weechat_plugin,
                              * - script was loaded
                              */
                             if ((!existing_script && autoload) || script_loaded)
-                                (*script_load) (new_path);
+                                (*script_load) (new_path, NULL);
                         }
                         else
                         {
@@ -1230,6 +1347,8 @@ plugin_script_action_install (struct t_weechat_plugin *weechat_plugin,
                         free (new_path);
                     }
                     free (base_name);
+                    if (dir_home)
+                        free (dir_home);
                 }
                 free (name);
             }
@@ -1281,7 +1400,11 @@ plugin_script_action_remove (struct t_weechat_plugin *weechat_plugin,
         ptr_list += 3;
     }
 
-    argv = weechat_string_split (ptr_list, ",", 0, 0, &argc);
+    argv = weechat_string_split (ptr_list, ",", NULL,
+                                 WEECHAT_STRING_SPLIT_STRIP_LEFT
+                                 | WEECHAT_STRING_SPLIT_STRIP_RIGHT
+                                 | WEECHAT_STRING_SPLIT_COLLAPSE_SEPS,
+                                 0, &argc);
     if (argv)
     {
         for (i = 0; i < argc; i++)
@@ -1319,8 +1442,7 @@ plugin_script_action_autoload (struct t_weechat_plugin *weechat_plugin,
                                char **list)
 {
     char **argv, *name, *ptr_base_name, *base_name, *autoload_path;
-    char *symlink_path, *ptr_list;
-    const char *dir_home, *dir_separator;
+    char *symlink_path, *ptr_list, *dir_home, *dir_separator;
     int argc, i, length, rc, autoload;
 
     if (!*list)
@@ -1352,7 +1474,11 @@ plugin_script_action_autoload (struct t_weechat_plugin *weechat_plugin,
         }
     }
 
-    argv = weechat_string_split (ptr_list, ",", 0, 0, &argc);
+    argv = weechat_string_split (ptr_list, ",", NULL,
+                                 WEECHAT_STRING_SPLIT_STRIP_LEFT
+                                 | WEECHAT_STRING_SPLIT_STRIP_RIGHT
+                                 | WEECHAT_STRING_SPLIT_COLLAPSE_SEPS,
+                                 0, &argc);
     if (argv)
     {
         for (i = 0; i < argc; i++)
@@ -1389,6 +1515,8 @@ plugin_script_action_autoload (struct t_weechat_plugin *weechat_plugin,
                                 (void) rc;
                                 free (symlink_path);
                             }
+                            if (dir_separator)
+                                free (dir_separator);
                         }
                         else
                         {
@@ -1397,6 +1525,8 @@ plugin_script_action_autoload (struct t_weechat_plugin *weechat_plugin,
                         free (autoload_path);
                     }
                     free (base_name);
+                    if (dir_home)
+                        free (dir_home);
                 }
                 free (name);
             }
@@ -1648,20 +1778,22 @@ plugin_script_infolist_list_scripts (struct t_weechat_plugin *weechat_plugin,
 
 void
 plugin_script_end (struct t_weechat_plugin *weechat_plugin,
-                   struct t_plugin_script **scripts,
-                   void (*callback_unload_all)())
+                   struct t_plugin_script_data *plugin_data)
 {
     int scripts_loaded;
 
-    scripts_loaded = (*scripts) ? 1 : 0;
-
-    (void)(callback_unload_all) ();
-
+    /* unload all scripts */
+    scripts_loaded = (*(plugin_data->scripts)) ? 1 : 0;
+    (void)(plugin_data->unload_all) ();
     if (scripts_loaded)
     {
         weechat_printf (NULL, _("%s: scripts unloaded"),
                         weechat_plugin->name);
     }
+
+    /* write config file (file: "<language>.conf") */
+    weechat_config_write (*(plugin_data->config_file));
+    weechat_config_free (*(plugin_data->config_file));
 }
 
 /*
