@@ -1,7 +1,7 @@
 /*
  * weechat-python.c - python plugin for WeeChat
  *
- * Copyright (C) 2003-2020 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
  * Copyright (C) 2005-2007 Emmanuel Bouthenot <kolter@openics.org>
  * Copyright (C) 2012 Simon Arlott
  *
@@ -444,13 +444,13 @@ weechat_python_output (PyObject *self, PyObject *args)
         ptr_msg = msg;
         while ((ptr_newline = strchr (ptr_msg, '\n')) != NULL)
         {
-            ptr_newline[0] = '\0';
-            weechat_string_dyn_concat (python_buffer_output, ptr_msg);
+            weechat_string_dyn_concat (python_buffer_output,
+                                       ptr_msg,
+                                       ptr_newline - ptr_msg);
             weechat_python_output_flush ();
-            ptr_newline[0] = '\n';
             ptr_msg = ++ptr_newline;
         }
-        weechat_string_dyn_concat (python_buffer_output, ptr_msg);
+        weechat_string_dyn_concat (python_buffer_output, ptr_msg, -1);
     }
 
     Py_INCREF(Py_None);
