@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2017 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -14,7 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef WEECHAT_PLUGIN_RELAY_SERVER_H
@@ -22,9 +22,7 @@
 
 #include <time.h>
 
-#ifdef HAVE_GNUTLS
 #define RELAY_SERVER_GNUTLS_DH_BITS 1024
-#endif /* HAVE_GNUTLS */
 
 struct t_relay_server
 {
@@ -33,9 +31,14 @@ struct t_relay_server
     char *protocol_args;               /* arguments used for protocol       */
                                        /* example: server for irc protocol  */
     int port;                          /* listening on this port            */
+                                       /* or UNIX socket, if negative.      */
+    char *path;                        /* listening on this path (UNIX),    */
+                                       /* contains string representation of */
+                                       /* port if IP                        */
     int ipv4;                          /* IPv4 protocol enabled             */
     int ipv6;                          /* IPv6 protocol enabled             */
     int ssl;                           /* 1 if SSL is enabled               */
+    int unix_socket;                   /* 1 if UNIX socket                  */
     int sock;                          /* socket for connection             */
     struct t_hook *hook_fd;            /* hook for socket                   */
     time_t start_time;                 /* start time                        */
@@ -49,18 +52,22 @@ extern struct t_relay_server *last_relay_server;
 
 extern void relay_server_get_protocol_args (const char *protocol_and_string,
                                             int *ipv4, int *ipv6,
-                                            int *ssl,
+                                            int *ssl, int *unix_socket,
                                             char **protocol,
                                             char **protocol_args);
 extern struct t_relay_server *relay_server_search (const char *protocol_and_args);
 extern struct t_relay_server *relay_server_search_port (int port);
+extern struct t_relay_server *relay_server_search_path (const char *path);
 extern void relay_server_close_socket (struct t_relay_server *server);
 extern int relay_server_create_socket (struct t_relay_server *server);
 extern struct t_relay_server *relay_server_new (const char *protocol_string,
                                                 enum t_relay_protocol protocol,
                                                 const char *protocol_args,
-                                                int port, int ipv4, int ipv6,
-                                                int ssl);
+                                                int port, const char *path,
+                                                int ipv4, int ipv6,
+                                                int ssl, int unix_socket);
+extern void relay_server_update_path (struct t_relay_server *server,
+                                      const char *path);
 extern void relay_server_update_port (struct t_relay_server *server, int port);
 extern void relay_server_free (struct t_relay_server *server);
 extern void relay_server_free_all ();
