@@ -1,7 +1,7 @@
 /*
  * relay-weechat-protocol.c - WeeChat protocol for relay to client
  *
- * Copyright (C) 2003-2019 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -31,6 +31,7 @@
 #include "relay-weechat-nicklist.h"
 #include "../relay-buffer.h"
 #include "../relay-client.h"
+#include "../relay-auth.h"
 #include "../relay-config.h"
 #include "../relay-raw.h"
 
@@ -158,24 +159,213 @@ relay_weechat_protocol_is_sync (struct t_relay_client *ptr_client,
     return 0;
 }
 
+/*
+ * Replies to a client handshake command.
+ */
+
+void
+relay_weechat_protocol_handshake_reply (struct t_relay_client *client,
+                                        const char *id)
+{
+    struct t_relay_weechat_msg *msg;
+    struct t_hashtable *hashtable;
+    char *totp_secret, string[64];
+
+    totp_secret = weechat_string_eval_expression (
+        weechat_config_string (relay_config_network_totp_secret),
+        NULL, NULL, NULL);
+
+    hashtable = weechat_hashtable_new (32,
+                                       WEECHAT_HASHTABLE_STRING,
+                                       WEECHAT_HASHTABLE_STRING,
+                                       NULL, NULL);
+    if (hashtable)
+    {
+        weechat_hashtable_set (
+            hashtable,
+            "password_hash_algo",
+            (client->password_hash_algo >= 0) ?
+            relay_auth_password_hash_algo_name[client->password_hash_algo] : "");
+        snprintf (string, sizeof (string), "%d", client->password_hash_iterations);
+        weechat_hashtable_set (
+            hashtable,
+            "password_hash_iterations",
+            string);
+        weechat_hashtable_set (
+            hashtable,
+            "nonce",
+            client->nonce);
+        weechat_hashtable_set (
+            hashtable,
+            "totp",
+            (totp_secret && totp_secret[0]) ? "on" : "off");
+        weechat_hashtable_set (
+            hashtable,
+            "compression",
+            relay_weechat_compression_string[RELAY_WEECHAT_DATA(client, compression)]);
+
+        msg = relay_weechat_msg_new (id);
+        if (msg)
+        {
+            relay_weechat_msg_add_type (msg, RELAY_WEECHAT_MSG_OBJ_HASHTABLE);
+            relay_weechat_msg_add_hashtable (msg, hashtable);
+
+            /* send message */
+            relay_weechat_msg_send (client, msg);
+            relay_weechat_msg_free (msg);
+        }
+
+        weechat_hashtable_free (hashtable);
+    }
+
+    if (totp_secret)
+        free (totp_secret);
+}
+
+
+/*
+ * Callback for command "handshake" (from client).
+ */
+
+RELAY_WEECHAT_PROTOCOL_CALLBACK(handshake)
+{
+    char **options, **auths, *pos;
+    int i, j, index_hash_algo, hash_algo_found, auth_allowed, compression;
+    int password_received, plain_text_password;
+
+    RELAY_WEECHAT_PROTOCOL_MIN_ARGS(0);
+
+    if (client->status != RELAY_STATUS_WAITING_AUTH)
+        return WEECHAT_RC_OK;
+
+    /* only one handshake is allowed */
+    if (RELAY_WEECHAT_DATA(client, handshake_done))
+    {
+        relay_client_set_status (client, RELAY_STATUS_AUTH_FAILED);
+        return WEECHAT_RC_OK;
+    }
+
+    hash_algo_found = -1;
+    password_received = 0;
+
+    options = (argc > 0) ?
+        weechat_string_split_command (argv_eol[0], ',') : NULL;
+    if (options)
+    {
+        for (i = 0; options[i]; i++)
+        {
+            pos = strchr (options[i], '=');
+            if (pos)
+            {
+                pos[0] = '\0';
+                pos++;
+                if (strcmp (options[i], "password_hash_algo") == 0)
+                {
+                    password_received = 1;
+                    auths = weechat_string_split (
+                        pos,
+                        ":",
+                        NULL,
+                        WEECHAT_STRING_SPLIT_STRIP_LEFT
+                        | WEECHAT_STRING_SPLIT_STRIP_RIGHT
+                        | WEECHAT_STRING_SPLIT_COLLAPSE_SEPS,
+                        0,
+                        NULL);
+                    if (auths)
+                    {
+                        for (j = 0; auths[j]; j++)
+                        {
+                            index_hash_algo = relay_auth_password_hash_algo_search (
+                                auths[j]);
+                            if ((index_hash_algo >= 0) && (index_hash_algo > hash_algo_found))
+                            {
+                                auth_allowed = weechat_string_match_list (
+                                    relay_auth_password_hash_algo_name[index_hash_algo],
+                                    (const char **)relay_config_network_password_hash_algo_list,
+                                    1);
+                                if (auth_allowed)
+                                    hash_algo_found = index_hash_algo;
+                            }
+                        }
+                        weechat_string_free_split (auths);
+                    }
+                }
+                else if (strcmp (options[i], "compression") == 0)
+                {
+                    compression = relay_weechat_compression_search (pos);
+                    if (compression >= 0)
+                        RELAY_WEECHAT_DATA(client, compression) = compression;
+                }
+            }
+        }
+        weechat_string_free_split_command (options);
+    }
+
+    if (!password_received)
+    {
+        plain_text_password = weechat_string_match_list (
+            relay_auth_password_hash_algo_name[0],
+            (const char **)relay_config_network_password_hash_algo_list,
+            1);
+        if (plain_text_password)
+            hash_algo_found = 0;
+    }
+
+    client->password_hash_algo = hash_algo_found;
+
+    relay_weechat_protocol_handshake_reply (client, id);
+
+    RELAY_WEECHAT_DATA(client, handshake_done) = 1;
+
+    /* if no algo was found, we close the connection immediately */
+    if (client->password_hash_algo < 0)
+        relay_client_set_status (client, RELAY_STATUS_AUTH_FAILED);
+
+    return WEECHAT_RC_OK;
+}
+
 /*
  * Callback for command "init" (from client).
  *
+ * Format is:  init arg1=value1,arg2=value2
+ *
+ * Allowed arguments:
+ *   password       plain text password (recommended with SSL only)
+ *   password_hash  hashed password, value is: algorithm:[parameters:]hash
+ *                  supported algorithms: sha256, sha512 and pbkdf2
+ *                  for pbkdf2, parameters are: algorithm, salt, iterations
+ *                  hash is given in hexadecimal
+ *   totp           time-based one time password used as secondary
+ *                  authentication factor
+ *   compression    zlib (default) or off
+ *
  * Message looks like:
  *   init password=mypass
  *   init password=mypass,compression=zlib
  *   init password=mypass,compression=off
+ *   init password_hash=sha256:71c480df93d6ae2f1efad1447c66c9…,totp=123456
+ *   init password_hash=pbkdf2:sha256:414232…:100000:01757d53157c…,totp=123456
  */
 
 RELAY_WEECHAT_PROTOCOL_CALLBACK(init)
 {
-    char **options, *pos, *password, *totp_secret, *info_totp_args;
-    const char *info_totp;
-    int i, compression, length;
+    char **options, *pos, *relay_password, *totp_secret, *info_totp_args, *info_totp;
+    int i, compression, length, password_received, totp_received;
 
-    RELAY_WEECHAT_PROTOCOL_MIN_ARGS(1);
+    RELAY_WEECHAT_PROTOCOL_MIN_ARGS(0);
 
-    options = weechat_string_split_command (argv_eol[0], ',');
+    relay_password = weechat_string_eval_expression (
+        weechat_config_string (relay_config_network_password),
+        NULL, NULL, NULL);
+    totp_secret = weechat_string_eval_expression (
+        weechat_config_string (relay_config_network_totp_secret),
+        NULL, NULL, NULL);
+
+    password_received = 0;
+    totp_received = 0;
+
+    options = (argc > 0) ?
+        weechat_string_split_command (argv_eol[0], ',') : NULL;
     if (options)
     {
         for (i = 0; options[i]; i++)
@@ -187,21 +377,19 @@ RELAY_WEECHAT_PROTOCOL_CALLBACK(init)
                 pos++;
                 if (strcmp (options[i], "password") == 0)
                 {
-                    password = weechat_string_eval_expression (
-                        weechat_config_string (relay_config_network_password),
-                        NULL, NULL, NULL);
-                    if (password)
-                    {
-                        if (strcmp (password, pos) == 0)
-                            RELAY_WEECHAT_DATA(client, password_ok) = 1;
-                        free (password);
-                    }
+                    password_received = 1;
+                    if (relay_auth_password (client, pos, relay_password))
+                        RELAY_WEECHAT_DATA(client, password_ok) = 1;
+                }
+                else if (strcmp (options[i], "password_hash") == 0)
+                {
+                    password_received = 1;
+                    if (relay_auth_password_hash (client, pos, relay_password))
+                        RELAY_WEECHAT_DATA(client, password_ok) = 1;
                 }
                 else if (strcmp (options[i], "totp") == 0)
                 {
-                    totp_secret = weechat_string_eval_expression (
-                        weechat_config_string (relay_config_network_totp_secret),
-                        NULL, NULL, NULL);
+                    totp_received = 1;
                     if (totp_secret)
                     {
                         length = strlen (totp_secret) + strlen (pos) + 16 + 1;
@@ -217,9 +405,10 @@ RELAY_WEECHAT_PROTOCOL_CALLBACK(init)
                             info_totp = weechat_info_get ("totp_validate", info_totp_args);
                             if (info_totp && (strcmp (info_totp, "1") == 0))
                                 RELAY_WEECHAT_DATA(client, totp_ok) = 1;
+                            if (info_totp)
+                                free (info_totp);
                             free (info_totp_args);
                         }
-                        free (totp_secret);
                     }
                 }
                 else if (strcmp (options[i], "compression") == 0)
@@ -233,18 +422,31 @@ RELAY_WEECHAT_PROTOCOL_CALLBACK(init)
         weechat_string_free_split_command (options);
     }
 
-    if (RELAY_WEECHAT_DATA(client, password_ok)
-        && RELAY_WEECHAT_DATA(client, totp_ok))
+    /* if no password received and password is empty, it's OK */
+    if (!password_received && (!relay_password || !relay_password[0]))
+        RELAY_WEECHAT_DATA(client, password_ok) = 1;
+
+    /* if no TOTP received and totp_secret is empty, it's OK */
+    if (!totp_received && (!totp_secret || !totp_secret[0]))
+        RELAY_WEECHAT_DATA(client, totp_ok) = 1;
+
+    if (RELAY_WEECHAT_AUTH_OK(client))
     {
         weechat_hook_signal_send ("relay_client_auth_ok",
                                   WEECHAT_HOOK_SIGNAL_POINTER,
                                   client);
+        relay_client_set_status (client, RELAY_STATUS_CONNECTED);
     }
     else
     {
         relay_client_set_status (client, RELAY_STATUS_AUTH_FAILED);
     }
 
+    if (relay_password)
+        free (relay_password);
+    if (totp_secret)
+        free (totp_secret);
+
     return WEECHAT_RC_OK;
 }
 
@@ -290,7 +492,7 @@ RELAY_WEECHAT_PROTOCOL_CALLBACK(hdata)
 RELAY_WEECHAT_PROTOCOL_CALLBACK(info)
 {
     struct t_relay_weechat_msg *msg;
-    const char *info;
+    char *info;
 
     RELAY_WEECHAT_PROTOCOL_MIN_ARGS(1);
 
@@ -304,6 +506,8 @@ RELAY_WEECHAT_PROTOCOL_CALLBACK(info)
         relay_weechat_msg_add_string (msg, info);
         relay_weechat_msg_send (client, msg);
         relay_weechat_msg_free (msg);
+        if (info)
+            free (info);
     }
 
     return WEECHAT_RC_OK;
@@ -371,8 +575,7 @@ RELAY_WEECHAT_PROTOCOL_CALLBACK(nicklist)
             if (weechat_relay_plugin->debug >= 1)
             {
                 weechat_printf (NULL,
-                                _("%s: invalid buffer pointer in message: "
-                                  "\"%s %s\""),
+                                _("%s: invalid buffer in message: \"%s %s\""),
                                 RELAY_PLUGIN_NAME,
                                 command,
                                 argv_eol[0]);
@@ -392,45 +595,6 @@ RELAY_WEECHAT_PROTOCOL_CALLBACK(nicklist)
     return WEECHAT_RC_OK;
 }
 
-/*
- * Timer callback for input command.
- */
-
-int
-relay_weechat_protocol_input_timer_cb (const void *pointer,
-                                       void *data,
-                                       int remaining_calls)
-{
-    char **timer_args;
-    int i;
-    struct t_gui_buffer *ptr_buffer;
-
-    /* make C compiler happy */
-    (void) data;
-    (void) remaining_calls;
-
-    timer_args = (char **)pointer;
-
-    if (!timer_args)
-        return WEECHAT_RC_ERROR;
-
-    if (timer_args[0] && timer_args[1])
-    {
-        ptr_buffer = weechat_buffer_search ("==", timer_args[0]);
-        if (ptr_buffer)
-            weechat_command (ptr_buffer, timer_args[1]);
-    }
-
-    for (i = 0; i < 2; i++)
-    {
-        if (timer_args[i])
-            free (timer_args[i]);
-    }
-    free (timer_args);
-
-    return WEECHAT_RC_OK;
-}
-
 /*
  * Callback for command "input" (from client).
  *
@@ -443,7 +607,9 @@ relay_weechat_protocol_input_timer_cb (const void *pointer,
 RELAY_WEECHAT_PROTOCOL_CALLBACK(input)
 {
     struct t_gui_buffer *ptr_buffer;
-    char *pos, **timer_args;
+    struct t_hashtable *options;
+    const char *ptr_weechat_commands;
+    char *pos;
 
     RELAY_WEECHAT_PROTOCOL_MIN_ARGS(1);
 
@@ -453,8 +619,7 @@ RELAY_WEECHAT_PROTOCOL_CALLBACK(input)
         if (weechat_relay_plugin->debug >= 1)
         {
             weechat_printf (NULL,
-                            _("%s: invalid buffer pointer in message: "
-                              "\"%s %s\""),
+                            _("%s: invalid buffer in message: \"%s %s\""),
                             RELAY_PLUGIN_NAME,
                             command,
                             argv[0]);
@@ -463,24 +628,213 @@ RELAY_WEECHAT_PROTOCOL_CALLBACK(input)
     }
 
     pos = strchr (argv_eol[0], ' ');
-    if (pos)
+    if (!pos)
+        return WEECHAT_RC_OK;
+
+    pos++;
+    options = weechat_hashtable_new (8,
+                                     WEECHAT_HASHTABLE_STRING,
+                                     WEECHAT_HASHTABLE_STRING,
+                                     NULL, NULL);
+    if (!options)
     {
-        /*
-         * use a timer to execute the command after we go back in the
-         * WeeChat main loop (some commands like /upgrade executed now can
-         * cause a crash)
-         */
-        timer_args = malloc (2 * sizeof (*timer_args));
-        if (timer_args)
+        weechat_printf (NULL,
+                        _("%s%s: not enough memory"),
+                        weechat_prefix ("error"), RELAY_PLUGIN_NAME);
+        return WEECHAT_RC_OK;
+    }
+
+    ptr_weechat_commands = weechat_config_string (
+        relay_config_weechat_commands);
+    if (ptr_weechat_commands && ptr_weechat_commands[0])
+    {
+        weechat_hashtable_set (
+            options,
+            "commands",
+            weechat_config_string (relay_config_weechat_commands));
+    }
+    /*
+     * delay the execution of command after we go back in the WeeChat
+     * main loop (some commands like /upgrade executed now can cause
+     * a crash)
+     */
+    weechat_hashtable_set (options, "delay", "1");
+
+    /* execute the command, with the delay */
+    weechat_command_options (ptr_buffer, pos, options);
+
+    weechat_hashtable_free (options);
+
+    return WEECHAT_RC_OK;
+}
+
+/*
+ * Callback for command "completion" (from client).
+ *
+ * Message looks like:
+ *   completion core.weechat -1 /help fi
+ *   input irc.freenode.#weechat 5 /quernick
+ *   input 0x12345678 -1 nick
+ */
+
+RELAY_WEECHAT_PROTOCOL_CALLBACK(completion)
+{
+    struct t_gui_buffer *ptr_buffer;
+    struct t_gui_completion *completion;
+    struct t_gui_completion_word *word;
+    struct t_hdata *ptr_hdata_completion, *ptr_hdata_completion_word;
+    struct t_arraylist *ptr_list;
+    struct t_relay_weechat_msg *msg;
+    char *error, *pos_data;
+    int i, position, length_data, context, pos_start, size;
+
+    RELAY_WEECHAT_PROTOCOL_MIN_ARGS(0);
+
+    completion = NULL;
+
+    /* return an empty hdata as error if there are not enough arguments */
+    if (argc < 2)
+        goto error;
+
+    ptr_buffer = relay_weechat_protocol_get_buffer (argv[0]);
+    if (!ptr_buffer)
+    {
+        if (weechat_relay_plugin->debug >= 1)
         {
-            timer_args[0] = strdup (weechat_buffer_get_string (ptr_buffer,
-                                                               "full_name"));
-            timer_args[1] = strdup (pos + 1);
-            weechat_hook_timer (1, 0, 1,
-                                &relay_weechat_protocol_input_timer_cb,
-                                timer_args,
-                                NULL);
+            weechat_printf (NULL,
+                            _("%s: invalid buffer in message: \"%s %s\""),
+                            RELAY_PLUGIN_NAME,
+                            command,
+                            argv[0]);
         }
+        goto error;
+    }
+
+    error = NULL;
+    position = (int)strtol (argv[1], &error, 10);
+    if (!error || error[0])
+        goto error;
+
+    pos_data = strchr (argv_eol[1], ' ');
+    if (pos_data)
+        pos_data++;
+
+    length_data = strlen ((pos_data) ? pos_data : "");
+    if ((position < 0) || (position > length_data))
+        position = length_data;
+
+    completion = weechat_completion_new (ptr_buffer);
+    if (!completion)
+        goto error;
+
+    if (!weechat_completion_search (completion,
+                                    (pos_data) ? pos_data : "",
+                                    position,
+                                    1))
+    {
+        goto error;
+    }
+
+    ptr_hdata_completion = weechat_hdata_get ("completion");
+    if (!ptr_hdata_completion)
+        goto error;
+
+    ptr_hdata_completion_word = weechat_hdata_get ("completion_word");
+    if (!ptr_hdata_completion_word)
+        goto error;
+
+    ptr_list = weechat_hdata_pointer (ptr_hdata_completion, completion, "list");
+    if (!ptr_list)
+        goto error;
+
+    msg = relay_weechat_msg_new (id);
+    if (msg)
+    {
+        relay_weechat_msg_add_type (msg, RELAY_WEECHAT_MSG_OBJ_HDATA);
+        relay_weechat_msg_add_string (msg, "completion");
+        relay_weechat_msg_add_string (msg,
+                                      "context:str,"
+                                      "base_word:str,"
+                                      "pos_start:int,"
+                                      "pos_end:int,"
+                                      "add_space:int,"
+                                      "list:arr");
+        relay_weechat_msg_add_int (msg, 1); /* count */
+        relay_weechat_msg_add_pointer (msg, completion);
+        /* context */
+        context = weechat_hdata_integer (ptr_hdata_completion, completion,
+                                         "context");
+        switch (context)
+        {
+            case 0:
+                relay_weechat_msg_add_string (msg, "null");
+                break;
+            case 1:
+                relay_weechat_msg_add_string (msg, "command");
+                break;
+            case 2:
+                relay_weechat_msg_add_string (msg, "command_arg");
+                break;
+            default:
+                relay_weechat_msg_add_string (msg, "auto");
+                break;
+        }
+        /* base_word */
+        relay_weechat_msg_add_string (
+            msg,
+            weechat_hdata_string (ptr_hdata_completion,
+                                  completion, "base_word"));
+        /* pos_start */
+        pos_start = weechat_hdata_integer (ptr_hdata_completion,
+                                           completion, "position_replace");
+        relay_weechat_msg_add_int (msg, pos_start);
+
+        /* pos_end */
+        relay_weechat_msg_add_int (
+            msg,
+            (position > pos_start) ? position - 1 : position);
+        /* add_space */
+        relay_weechat_msg_add_int (
+            msg,
+            weechat_hdata_integer (ptr_hdata_completion,
+                                   completion, "add_space"));
+        /* list */
+        relay_weechat_msg_add_type (msg, RELAY_WEECHAT_MSG_OBJ_STRING);
+        size = weechat_arraylist_size (ptr_list);
+        relay_weechat_msg_add_int (msg, size);
+        for (i = 0; i < size; i++)
+        {
+            word = (struct t_gui_completion_word *)weechat_arraylist_get (
+                ptr_list, i);
+            relay_weechat_msg_add_string (
+                msg,
+                weechat_hdata_string (ptr_hdata_completion_word, word, "word"));
+        }
+
+        /* send message */
+        relay_weechat_msg_send (client, msg);
+        relay_weechat_msg_free (msg);
+    }
+
+    weechat_completion_free (completion);
+
+    return WEECHAT_RC_OK;
+
+error:
+    if (completion)
+        weechat_completion_free (completion);
+
+    msg = relay_weechat_msg_new (id);
+    if (msg)
+    {
+        relay_weechat_msg_add_type (msg, RELAY_WEECHAT_MSG_OBJ_HDATA);
+        relay_weechat_msg_add_string (msg, "completion");
+        relay_weechat_msg_add_string (msg, NULL);  /* keys */
+        relay_weechat_msg_add_int (msg, 0);  /* count */
+
+        /* send message */
+        relay_weechat_msg_send (client, msg);
+        relay_weechat_msg_free (msg);
     }
 
     return WEECHAT_RC_OK;
@@ -503,6 +857,8 @@ relay_weechat_protocol_signal_buffer_cb (const void *pointer, void *data,
     struct t_gui_buffer *ptr_buffer;
     struct t_relay_weechat_msg *msg;
     char cmd_hdata[64], str_signal[128];
+    const char *ptr_old_full_name;
+    int *ptr_old_flags, flags;
 
     /* make C compiler happy */
     (void) data;
@@ -642,6 +998,27 @@ relay_weechat_protocol_signal_buffer_cb (const void *pointer, void *data,
         if (!ptr_buffer)
             return WEECHAT_RC_OK;
 
+        /* rename old buffer name if present in hashtable "buffers_sync" */
+        ptr_old_full_name = weechat_buffer_get_string (ptr_buffer,
+                                                       "old_full_name");
+        if (ptr_old_full_name && ptr_old_full_name[0])
+        {
+            ptr_old_flags = weechat_hashtable_get (
+                RELAY_WEECHAT_DATA(ptr_client, buffers_sync),
+                ptr_old_full_name);
+            if (ptr_old_flags)
+            {
+                flags = *ptr_old_flags;
+                weechat_hashtable_remove (
+                    RELAY_WEECHAT_DATA(ptr_client, buffers_sync),
+                    ptr_old_full_name);
+                weechat_hashtable_set (
+                    RELAY_WEECHAT_DATA(ptr_client, buffers_sync),
+                    weechat_buffer_get_string (ptr_buffer, "full_name"),
+                    &flags);
+            }
+        }
+
         /* send signal only if sync with flag "buffers" or "buffer" */
         if (relay_weechat_protocol_is_sync (ptr_client, ptr_buffer,
                                             RELAY_WEECHAT_PROTOCOL_SYNC_BUFFERS |
@@ -763,8 +1140,9 @@ relay_weechat_protocol_signal_buffer_cb (const void *pointer, void *data,
                           (unsigned long)ptr_line_data);
                 relay_weechat_msg_add_hdata (msg, cmd_hdata,
                                              "buffer,date,date_printed,"
-                                             "displayed,highlight,tags_array,"
-                                             "prefix,message");
+                                             "displayed,notify_level,"
+                                             "highlight,tags_array,prefix,"
+                                             "message");
                 relay_weechat_msg_send (ptr_client, msg);
                 relay_weechat_msg_free (msg);
             }
@@ -1047,7 +1425,13 @@ RELAY_WEECHAT_PROTOCOL_CALLBACK(sync)
 
     RELAY_WEECHAT_PROTOCOL_MIN_ARGS(0);
 
-    buffers = weechat_string_split ((argc > 0) ? argv[0] : "*", ",", 0, 0,
+    buffers = weechat_string_split ((argc > 0) ? argv[0] : "*",
+                                    ",",
+                                    NULL,
+                                    WEECHAT_STRING_SPLIT_STRIP_LEFT
+                                    | WEECHAT_STRING_SPLIT_STRIP_RIGHT
+                                    | WEECHAT_STRING_SPLIT_COLLAPSE_SEPS,
+                                    0,
                                     &num_buffers);
     if (buffers)
     {
@@ -1055,7 +1439,11 @@ RELAY_WEECHAT_PROTOCOL_CALLBACK(sync)
         if (argc > 1)
         {
             add_flags = 0;
-            flags = weechat_string_split (argv[1], ",", 0, 0, &num_flags);
+            flags = weechat_string_split (argv[1], ",", NULL,
+                                          WEECHAT_STRING_SPLIT_STRIP_LEFT
+                                          | WEECHAT_STRING_SPLIT_STRIP_RIGHT
+                                          | WEECHAT_STRING_SPLIT_COLLAPSE_SEPS,
+                                          0, &num_flags);
             if (flags)
             {
                 for (i = 0; i < num_flags; i++)
@@ -1126,7 +1514,13 @@ RELAY_WEECHAT_PROTOCOL_CALLBACK(desync)
 
     RELAY_WEECHAT_PROTOCOL_MIN_ARGS(0);
 
-    buffers = weechat_string_split ((argc > 0) ? argv[0] : "*", ",", 0, 0,
+    buffers = weechat_string_split ((argc > 0) ? argv[0] : "*",
+                                    ",",
+                                    NULL,
+                                    WEECHAT_STRING_SPLIT_STRIP_LEFT
+                                    | WEECHAT_STRING_SPLIT_STRIP_RIGHT
+                                    | WEECHAT_STRING_SPLIT_COLLAPSE_SEPS,
+                                    0,
                                     &num_buffers);
     if (buffers)
     {
@@ -1134,7 +1528,11 @@ RELAY_WEECHAT_PROTOCOL_CALLBACK(desync)
         if (argc > 1)
         {
             sub_flags = 0;
-            flags = weechat_string_split (argv[1], ",", 0, 0, &num_flags);
+            flags = weechat_string_split (argv[1], ",", NULL,
+                                          WEECHAT_STRING_SPLIT_STRIP_LEFT
+                                          | WEECHAT_STRING_SPLIT_STRIP_RIGHT
+                                          | WEECHAT_STRING_SPLIT_COLLAPSE_SEPS,
+                                          0, &num_flags);
             if (flags)
             {
                 for (i = 0; i < num_flags; i++)
@@ -1337,12 +1735,14 @@ relay_weechat_protocol_recv (struct t_relay_client *client, const char *data)
     char *pos, *id, *command, **argv, **argv_eol;
     int i, argc, return_code;
     struct t_relay_weechat_protocol_cb protocol_cb[] =
-        { { "init", &relay_weechat_protocol_cb_init },
+        { { "handshake", &relay_weechat_protocol_cb_handshake },
+          { "init", &relay_weechat_protocol_cb_init },
           { "hdata", &relay_weechat_protocol_cb_hdata },
           { "info", &relay_weechat_protocol_cb_info },
           { "infolist", &relay_weechat_protocol_cb_infolist },
           { "nicklist", &relay_weechat_protocol_cb_nicklist },
           { "input", &relay_weechat_protocol_cb_input },
+          { "completion", &relay_weechat_protocol_cb_completion },
           { "sync", &relay_weechat_protocol_cb_sync },
           { "desync", &relay_weechat_protocol_cb_desync },
           { "test", &relay_weechat_protocol_cb_test },
@@ -1405,21 +1805,29 @@ relay_weechat_protocol_recv (struct t_relay_client *client, const char *data)
         {
             pos++;
         }
-        argv = weechat_string_split (pos, " ", 0, 0, &argc);
-        argv_eol = weechat_string_split (pos, " ", 2, 0, NULL);
+        argv = weechat_string_split (pos, " ", NULL,
+                                     WEECHAT_STRING_SPLIT_STRIP_LEFT
+                                     | WEECHAT_STRING_SPLIT_STRIP_RIGHT
+                                     | WEECHAT_STRING_SPLIT_COLLAPSE_SEPS,
+                                     0, &argc);
+        argv_eol = weechat_string_split (pos, " ", NULL,
+                                         WEECHAT_STRING_SPLIT_STRIP_LEFT
+                                         | WEECHAT_STRING_SPLIT_COLLAPSE_SEPS
+                                         | WEECHAT_STRING_SPLIT_KEEP_EOL,
+                                         0, NULL);
     }
 
     for (i = 0; protocol_cb[i].name; i++)
     {
         if (strcmp (protocol_cb[i].name, command) == 0)
         {
-            if ((strcmp (protocol_cb[i].name, "init") != 0)
-                && (!RELAY_WEECHAT_DATA(client, password_ok)
-                    || !RELAY_WEECHAT_DATA(client, totp_ok)))
+            if ((strcmp (protocol_cb[i].name, "handshake") != 0)
+                && (strcmp (protocol_cb[i].name, "init") != 0)
+                && !RELAY_WEECHAT_AUTH_OK(client))
             {
                 /*
-                 * command is not "init" and password or totp are not set?
-                 * then close connection!
+                 * command is not handshake/init and password or totp are not
+                 * set? then close connection!
                  */
                 relay_client_set_status (client,
                                          RELAY_STATUS_AUTH_FAILED);
