@@ -1,7 +1,7 @@
 /*
  * script-completion.c - completions for script command
  *
- * Copyright (C) 2003-2019 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -49,9 +49,9 @@ script_completion_languages_cb (const void *pointer, void *data,
 
     for (i = 0; i < SCRIPT_NUM_LANGUAGES; i++)
     {
-        weechat_hook_completion_list_add (completion,
-                                          script_language[i],
-                                          0, WEECHAT_LIST_POS_SORT);
+        weechat_completion_list_add (completion,
+                                     script_language[i],
+                                     0, WEECHAT_LIST_POS_SORT);
     }
 
     return WEECHAT_RC_OK;
@@ -77,9 +77,9 @@ script_completion_extensions_cb (const void *pointer, void *data,
 
     for (i = 0; i < SCRIPT_NUM_LANGUAGES; i++)
     {
-        weechat_hook_completion_list_add (completion,
-                                          script_extension[i],
-                                          0, WEECHAT_LIST_POS_SORT);
+        weechat_completion_list_add (completion,
+                                     script_extension[i],
+                                     0, WEECHAT_LIST_POS_SORT);
     }
 
     return WEECHAT_RC_OK;
@@ -106,9 +106,9 @@ script_completion_scripts_cb (const void *pointer, void *data,
     for (ptr_script = scripts_repo; ptr_script;
          ptr_script = ptr_script->next_script)
     {
-        weechat_hook_completion_list_add (completion,
-                                          ptr_script->name_with_extension,
-                                          0, WEECHAT_LIST_POS_SORT);
+        weechat_completion_list_add (completion,
+                                     ptr_script->name_with_extension,
+                                     0, WEECHAT_LIST_POS_SORT);
     }
 
     return WEECHAT_RC_OK;
@@ -137,9 +137,9 @@ script_completion_scripts_installed_cb (const void *pointer, void *data,
     {
         if (ptr_script->status & SCRIPT_STATUS_INSTALLED)
         {
-            weechat_hook_completion_list_add (completion,
-                                              ptr_script->name_with_extension,
-                                              0, WEECHAT_LIST_POS_SORT);
+            weechat_completion_list_add (completion,
+                                         ptr_script->name_with_extension,
+                                         0, WEECHAT_LIST_POS_SORT);
         }
     }
 
@@ -172,9 +172,9 @@ script_completion_exec_file_cb (void *data, const char *filename)
     if (filename2)
     {
         ptr_base_name = basename (filename2);
-        weechat_hook_completion_list_add (completion,
-                                          ptr_base_name,
-                                          0, WEECHAT_LIST_POS_SORT);
+        weechat_completion_list_add (completion,
+                                     ptr_base_name,
+                                     0, WEECHAT_LIST_POS_SORT);
         free (filename2);
     }
 }
@@ -189,8 +189,7 @@ script_completion_scripts_files_cb (const void *pointer, void *data,
                                     struct t_gui_buffer *buffer,
                                     struct t_gui_completion *completion)
 {
-    const char *weechat_home;
-    char *directory;
+    char *weechat_home, *directory;
     int length, i;
     void *pointers[2];
 
@@ -228,6 +227,9 @@ script_completion_scripts_files_cb (const void *pointer, void *data,
         free (directory);
     }
 
+    if (weechat_home)
+        free (weechat_home);
+
     return WEECHAT_RC_OK;
 }
 
@@ -256,15 +258,22 @@ script_completion_tags_cb (const void *pointer, void *data,
     {
         if (ptr_script->tags)
         {
-            list_tags = weechat_string_split (ptr_script->tags, ",", 0, 0,
-                                              &num_tags);
+            list_tags = weechat_string_split (
+                ptr_script->tags,
+                ",",
+                NULL,
+                WEECHAT_STRING_SPLIT_STRIP_LEFT
+                | WEECHAT_STRING_SPLIT_STRIP_RIGHT
+                | WEECHAT_STRING_SPLIT_COLLAPSE_SEPS,
+                0,
+                &num_tags);
             if (list_tags)
             {
                 for (i = 0; i < num_tags; i++)
                 {
-                    weechat_hook_completion_list_add (completion,
-                                                      list_tags[i],
-                                                      0, WEECHAT_LIST_POS_SORT);
+                    weechat_completion_list_add (completion,
+                                                 list_tags[i],
+                                                 0, WEECHAT_LIST_POS_SORT);
                 }
                 weechat_string_free_split (list_tags);
             }
