@@ -1,7 +1,7 @@
 /*
  * script-config.c - script configuration options (file script.conf)
  *
- * Copyright (C) 2003-2019 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -79,9 +79,10 @@ struct t_config_option *script_config_color_text_version_selected;
 
 struct t_config_option *script_config_scripts_autoload;
 struct t_config_option *script_config_scripts_cache_expire;
+struct t_config_option *script_config_scripts_download_enabled;
 struct t_config_option *script_config_scripts_download_timeout;
-struct t_config_option *script_config_scripts_path;
 struct t_config_option *script_config_scripts_hold;
+struct t_config_option *script_config_scripts_path;
 struct t_config_option *script_config_scripts_url;
 
 
@@ -96,7 +97,8 @@ struct t_config_option *script_config_scripts_url;
 const char *
 script_config_get_diff_command ()
 {
-    const char *diff_command, *dir_separator;
+    const char *diff_command;
+    char *dir_separator;
     static char result[64];
     struct stat st;
     char *path, **paths, bin[4096];
@@ -113,7 +115,11 @@ script_config_get_diff_command ()
         result[0] = '\0';
         if (dir_separator && path)
         {
-            paths = weechat_string_split (path, ":", 0, 0, &num_paths);
+            paths = weechat_string_split (path, ":", NULL,
+                                          WEECHAT_STRING_SPLIT_STRIP_LEFT
+                                          | WEECHAT_STRING_SPLIT_STRIP_RIGHT
+                                          | WEECHAT_STRING_SPLIT_COLLAPSE_SEPS,
+                                          0, &num_paths);
             if (paths)
             {
                 for (i = 0; i < num_paths; i++)
@@ -131,6 +137,8 @@ script_config_get_diff_command ()
                 weechat_string_free_split (paths);
             }
         }
+        if (dir_separator)
+            free (dir_separator);
         if (!result[0])
             snprintf (result, sizeof (result), "diff");
         return result;
@@ -289,8 +297,15 @@ script_config_hold (const char *name_with_extension)
     if (hold)
     {
         hold[0] = '\0';
-        items = weechat_string_split (weechat_config_string (script_config_scripts_hold),
-                                      ",", 0, 0, &num_items);
+        items = weechat_string_split (
+            weechat_config_string (script_config_scripts_hold),
+            ",",
+            NULL,
+            WEECHAT_STRING_SPLIT_STRIP_LEFT
+            | WEECHAT_STRING_SPLIT_STRIP_RIGHT
+            | WEECHAT_STRING_SPLIT_COLLAPSE_SEPS,
+            0,
+            &num_items);
         if (items)
         {
             for (i = 0; i < num_items; i++)
@@ -332,8 +347,15 @@ script_config_unhold (const char *name_with_extension)
     if (hold)
     {
         hold[0] = '\0';
-        items = weechat_string_split (weechat_config_string (script_config_scripts_hold),
-                                      ",", 0, 0, &num_items);
+        items = weechat_string_split (
+            weechat_config_string (script_config_scripts_hold),
+            ",",
+            NULL,
+            WEECHAT_STRING_SPLIT_STRIP_LEFT
+            | WEECHAT_STRING_SPLIT_STRIP_RIGHT
+            | WEECHAT_STRING_SPLIT_COLLAPSE_SEPS,
+            0,
+            &num_items);
         if (items)
         {
             for (i = 0; i < num_items; i++)
@@ -732,20 +754,23 @@ script_config_init ()
            "0 = always expire)"),
         NULL, -1, 525600, "1440", NULL, 0,
         NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
+    script_config_scripts_download_enabled = weechat_config_new_option (
+        script_config_file, ptr_section,
+        "download_enabled", "boolean",
+        N_("enable download of files from the scripts repository when the "
+           "/script command is used (list of scripts and scripts themselves); "
+           "the list of scripts is downloaded from the URL specified in the "
+           "option script.scripts.url; WeeChat will sometimes download again "
+           "the list of scripts when you use the /script command, even if "
+           "you don't install a script"),
+        NULL, 0, 0, "off", NULL, 0,
+        NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
     script_config_scripts_download_timeout = weechat_config_new_option (
         script_config_file, ptr_section,
         "download_timeout", "integer",
         N_("timeout (in seconds) for download of scripts and list of scripts"),
         NULL, 1, 3600, "30", NULL, 0,
         NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
-    script_config_scripts_path = weechat_config_new_option (
-        script_config_file, ptr_section,
-        "path", "string",
-        N_("local cache directory for scripts; \"%h\" at beginning of string "
-           "is replaced by WeeChat home (\"~/.weechat\" by default) "
-           "(note: content is evaluated, see /help eval)"),
-        NULL, 0, 0, "%h/script", NULL, 0,
-        NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
     script_config_scripts_hold = weechat_config_new_option (
         script_config_file, ptr_section,
         "hold", "string",
@@ -756,6 +781,14 @@ script_config_init ()
         NULL, NULL, NULL,
         &script_config_change_hold_cb, NULL, NULL,
         NULL, NULL, NULL);
+    script_config_scripts_path = weechat_config_new_option (
+        script_config_file, ptr_section,
+        "path", "string",
+        N_("local cache directory for scripts; \"%h\" at beginning of string "
+           "is replaced by WeeChat home (\"~/.weechat\" by default) "
+           "(note: content is evaluated, see /help eval)"),
+        NULL, 0, 0, "%h/script", NULL, 0,
+        NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
     script_config_scripts_url = weechat_config_new_option (
         script_config_file, ptr_section,
         "url", "string",
