@@ -1,7 +1,7 @@
 /*
  * trigger-callback.c - callbacks for triggers
  *
- * Copyright (C) 2014-2020 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2014-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -33,9 +33,6 @@
 /* hashtable used to evaluate "conditions" */
 struct t_hashtable *trigger_callback_hashtable_options_conditions = NULL;
 
-/* hashtable used to replace with regex */
-struct t_hashtable *trigger_callback_hashtable_options_regex = NULL;
-
 
 /*
  * Parses an IRC message.
@@ -125,6 +122,21 @@ trigger_callback_get_irc_server_channel (const char *irc_server_name,
     }
 }
 
+/*
+ * Sets variables common to all triggers in a hashtable:
+ * - tg_trigger_name
+ */
+
+void
+trigger_callback_set_common_vars (struct t_trigger *trigger,
+                                  struct t_hashtable *hashtable)
+{
+    if (!trigger || !hashtable)
+        return;
+
+    weechat_hashtable_set (hashtable, "tg_trigger_name", trigger->name);
+}
+
 /*
  * Sets variables in "extra_vars" hashtable using tags from message.
  *
@@ -233,6 +245,7 @@ trigger_callback_replace_regex (struct t_trigger *trigger,
     char *value;
     const char *ptr_key, *ptr_value;
     int i, pointers_allocated;
+    struct t_hashtable *hashtable_options_regex;
 
     pointers_allocated = 0;
 
@@ -283,8 +296,14 @@ trigger_callback_replace_regex (struct t_trigger *trigger,
             ptr_value = weechat_hashtable_get (extra_vars, ptr_key);
         }
 
+        hashtable_options_regex = weechat_hashtable_new (
+            32,
+            WEECHAT_HASHTABLE_STRING,
+            WEECHAT_HASHTABLE_STRING,
+            NULL, NULL);
+
         weechat_hashtable_set (pointers, "regex", trigger->regex[i].regex);
-        weechat_hashtable_set (trigger_callback_hashtable_options_regex,
+        weechat_hashtable_set (hashtable_options_regex,
                                "regex_replace",
                                trigger->regex[i].replace_escaped);
 
@@ -292,7 +311,9 @@ trigger_callback_replace_regex (struct t_trigger *trigger,
             ptr_value,
             pointers,
             extra_vars,
-            trigger_callback_hashtable_options_regex);
+            hashtable_options_regex);
+
+        weechat_hashtable_free (hashtable_options_regex);
 
         if (value)
         {
@@ -508,8 +529,9 @@ trigger_callback_signal_cb (const void *pointer, void *data,
     }
 
     /* add data in hashtable used for conditions/replace/command */
-    ptr_signal_data = NULL;
+    trigger_callback_set_common_vars (trigger, extra_vars);
     weechat_hashtable_set (extra_vars, "tg_signal", signal);
+    ptr_signal_data = NULL;
     if (strcmp (type_data, WEECHAT_HOOK_SIGNAL_STRING) == 0)
     {
         ptr_signal_data = (const char *)signal_data;
@@ -583,6 +605,7 @@ trigger_callback_hsignal_cb (const void *pointer, void *data,
     }
 
     /* add data in hashtable used for conditions/replace/command */
+    trigger_callback_set_common_vars (trigger, extra_vars);
     weechat_hashtable_set (extra_vars, "tg_signal", signal);
 
     /* execute the trigger (conditions, regex, command) */
@@ -646,6 +669,7 @@ trigger_callback_modifier_cb (const void *pointer, void *data,
     }
 
     /* add data in hashtable used for conditions/replace/command */
+    trigger_callback_set_common_vars (trigger, extra_vars);
     weechat_hashtable_set (extra_vars, "tg_modifier", modifier);
     weechat_hashtable_set (extra_vars, "tg_modifier_data", modifier_data);
     weechat_hashtable_set (extra_vars, "tg_string", string);
@@ -818,6 +842,7 @@ trigger_callback_line_cb (const void *pointer, void *data,
     weechat_hashtable_remove (extra_vars, "tags");
 
     /* add data in hashtables used for conditions/replace/command */
+    trigger_callback_set_common_vars (trigger, extra_vars);
     ptr_value = weechat_hashtable_get (line, "buffer");
     if (!ptr_value || (ptr_value[0] != '0') || (ptr_value[1] != 'x'))
         goto end;
@@ -954,6 +979,7 @@ trigger_callback_print_cb  (const void *pointer, void *data,
     TRIGGER_CALLBACK_CB_NEW_EXTRA_VARS;
 
     /* add data in hashtables used for conditions/replace/command */
+    trigger_callback_set_common_vars (trigger, extra_vars);
     weechat_hashtable_set (pointers, "buffer", buffer);
     date_tmp = localtime (&date);
     if (date_tmp)
@@ -1016,7 +1042,7 @@ trigger_callback_command_cb  (const void *pointer, void *data,
                               struct t_gui_buffer *buffer,
                               int argc, char **argv, char **argv_eol)
 {
-    char str_name[32];
+    char str_name[32], str_temp[128];
     int i;
 
     TRIGGER_CALLBACK_CB_INIT(WEECHAT_RC_OK);
@@ -1025,7 +1051,10 @@ trigger_callback_command_cb  (const void *pointer, void *data,
     TRIGGER_CALLBACK_CB_NEW_EXTRA_VARS;
 
     /* add data in hashtables used for conditions/replace/command */
+    trigger_callback_set_common_vars (trigger, extra_vars);
     weechat_hashtable_set (pointers, "buffer", buffer);
+    snprintf (str_temp, sizeof (str_temp), "%d", argc);
+    weechat_hashtable_set (extra_vars, "tg_argc", str_temp);
     for (i = 0; i < argc; i++)
     {
         snprintf (str_name, sizeof (str_name), "tg_argv%d", i);
@@ -1057,6 +1086,7 @@ trigger_callback_command_run_cb  (const void *pointer, void *data,
     TRIGGER_CALLBACK_CB_NEW_EXTRA_VARS;
 
     /* add data in hashtables used for conditions/replace/command */
+    trigger_callback_set_common_vars (trigger, extra_vars);
     weechat_hashtable_set (pointers, "buffer", buffer);
     weechat_hashtable_set (extra_vars, "tg_command", command);
 
@@ -1099,6 +1129,7 @@ trigger_callback_timer_cb  (const void *pointer, void *data,
     TRIGGER_CALLBACK_CB_NEW_EXTRA_VARS;
 
     /* add data in hashtable used for conditions/replace/command */
+    trigger_callback_set_common_vars (trigger, extra_vars);
     snprintf (str_temp, sizeof (str_temp), "%d", remaining_calls);
     weechat_hashtable_set (extra_vars, "tg_remaining_calls", str_temp);
     date = time (NULL);
@@ -1132,6 +1163,7 @@ trigger_callback_config_cb  (const void *pointer, void *data,
     TRIGGER_CALLBACK_CB_NEW_EXTRA_VARS;
 
     /* add data in hashtable used for conditions/replace/command */
+    trigger_callback_set_common_vars (trigger, extra_vars);
     weechat_hashtable_set (extra_vars, "tg_option", option);
     weechat_hashtable_set (extra_vars, "tg_value", value);
 
@@ -1160,6 +1192,7 @@ trigger_callback_focus_cb (const void *pointer, void *data,
     TRIGGER_CALLBACK_CB_NEW_POINTERS;
 
     /* add data in hashtables used for conditions/replace/command */
+    trigger_callback_set_common_vars (trigger, info);
     ptr_value = weechat_hashtable_get (info, "_window");
     if (ptr_value && ptr_value[0] && (strncmp (ptr_value, "0x", 2) == 0))
     {
@@ -1198,6 +1231,7 @@ trigger_callback_info_cb (const void *pointer, void *data,
     TRIGGER_CALLBACK_CB_NEW_EXTRA_VARS;
 
     /* add data in hashtable used for conditions/replace/command */
+    trigger_callback_set_common_vars (trigger, extra_vars);
     weechat_hashtable_set (extra_vars, "tg_info_name", info_name);
     weechat_hashtable_set (extra_vars, "tg_arguments", arguments);
     weechat_hashtable_set (extra_vars, "tg_info", "");
@@ -1236,6 +1270,7 @@ trigger_callback_info_hashtable_cb (const void *pointer, void *data,
     extra_vars = weechat_hashtable_dup (hashtable);
 
     /* add data in hashtable used for conditions/replace/command */
+    trigger_callback_set_common_vars (trigger, extra_vars);
     weechat_hashtable_set (extra_vars, "tg_info_name", info_name);
 
     /* execute the trigger (conditions, regex, command) */
@@ -1284,12 +1319,6 @@ trigger_callback_init ()
         weechat_hashtable_set (trigger_callback_hashtable_options_conditions,
                                "type", "condition");
     }
-
-    trigger_callback_hashtable_options_regex = weechat_hashtable_new (
-        32,
-        WEECHAT_HASHTABLE_STRING,
-        WEECHAT_HASHTABLE_STRING,
-        NULL, NULL);
 }
 
 /*
@@ -1301,6 +1330,4 @@ trigger_callback_end ()
 {
     if (trigger_callback_hashtable_options_conditions)
         weechat_hashtable_free (trigger_callback_hashtable_options_conditions);
-    if (trigger_callback_hashtable_options_regex)
-        weechat_hashtable_free (trigger_callback_hashtable_options_regex);
 }
