@@ -1,7 +1,7 @@
 /*
  * trigger.c - trigger plugin for WeeChat
  *
- * Copyright (C) 2014-2017 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2014-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <stdlib.h>
@@ -49,20 +49,21 @@ char *trigger_option_default[TRIGGER_NUM_OPTIONS] =
 { "on", "signal", "", "", "", "", "ok", "none" };
 
 char *trigger_hook_type_string[TRIGGER_NUM_HOOK_TYPES] =
-{ "signal", "hsignal", "modifier", "print", "command", "command_run", "timer",
-  "config", "focus" };
+{ "signal", "hsignal", "modifier", "line", "print", "command", "command_run",
+  "timer", "config", "focus", "info", "info_hashtable" };
 char *trigger_hook_option_values =
-    "signal|hsignal|modifier|print|command|command_run|timer|config|focus";
+    "signal|hsignal|modifier|line|print|command|command_run|timer|config|"
+    "focus|info|info_hashtable";
 char *trigger_hook_default_arguments[TRIGGER_NUM_HOOK_TYPES] =
-{ "xxx", "xxx", "xxx", "", "cmd;desc;args;args_desc;%(buffers_names)", "/cmd",
-  "60000;0;0", "xxx", "chat" };
+{ "xxx", "xxx", "xxx", "", "", "cmd;desc;args;args_desc;%(buffers_names)",
+  "/cmd", "60000;0;0", "xxx", "chat", "xxx", "xxx" };
 char *trigger_hook_default_rc[TRIGGER_NUM_HOOK_TYPES] =
-{ "ok,ok_eat,error", "ok,ok_eat,error", "", "ok,error", "ok,error",
-  "ok,ok_eat,error", "ok", "ok", "" };
+{ "ok,ok_eat,error", "ok,ok_eat,error", "", "", "ok,error", "ok,error",
+  "ok,ok_eat,error", "ok", "ok", "", "", "" };
 
 char *trigger_hook_regex_default_var[TRIGGER_NUM_HOOK_TYPES] =
-{ "tg_signal_data", "", "tg_string", "tg_message", "tg_argv_eol1", "tg_command",
-  "tg_remaining_calls", "tg_value", "" };
+{ "tg_signal_data", "", "tg_string", "message", "tg_message", "tg_argv_eol1",
+  "tg_command", "tg_remaining_calls", "tg_value", "", "tg_info", "" };
 
 char *trigger_return_code_string[TRIGGER_NUM_RETURN_CODES] =
 { "ok", "ok_eat", "error" };
@@ -262,19 +263,32 @@ trigger_unhook (struct t_trigger *trigger)
 void
 trigger_hook (struct t_trigger *trigger)
 {
-    char **argv, **argv_eol, *tags, *message, *error1, *error2, *error3;
+    char **argv, **argv_eol, *buffer_type, *buffer_name, *tags, *message;
+    char *error1, *error2, *error3;
+    char *eval_desc, *eval_args, *eval_desc_args, *eval_completion;
     int i, argc, strip_colors;
     long interval, align_second, max_calls;
+    struct t_hashtable *extra_vars;
 
     if (!weechat_config_boolean (trigger->options[TRIGGER_OPTION_ENABLED]))
         return;
 
     trigger_unhook (trigger);
 
-    argv = weechat_string_split (weechat_config_string (trigger->options[TRIGGER_OPTION_ARGUMENTS]),
-                                 ";", 0, 0, &argc);
-    argv_eol = weechat_string_split (weechat_config_string (trigger->options[TRIGGER_OPTION_ARGUMENTS]),
-                                     ";", 1, 0, NULL);
+    argv = weechat_string_split (
+        weechat_config_string (trigger->options[TRIGGER_OPTION_ARGUMENTS]),
+        ";",
+        NULL,
+        0,
+        0,
+        &argc);
+    argv_eol = weechat_string_split (
+        weechat_config_string (trigger->options[TRIGGER_OPTION_ARGUMENTS]),
+        ";",
+        NULL,
+        WEECHAT_STRING_SPLIT_KEEP_EOL,
+        0,
+        NULL);
 
     switch (weechat_config_integer (trigger->options[TRIGGER_OPTION_HOOK]))
     {
@@ -329,6 +343,30 @@ trigger_hook (struct t_trigger *trigger)
                 }
             }
             break;
+        case TRIGGER_HOOK_LINE:
+            buffer_type = NULL;
+            buffer_name = NULL;
+            tags = NULL;
+            if (argv && (argc >= 1))
+            {
+                buffer_type = argv[0];
+                if ((argc >= 2) && (strcmp (argv[1], "*") != 0))
+                    buffer_name = argv[1];
+                if ((argc >= 3) && (strcmp (argv[2], "*") != 0))
+                    tags = argv[2];
+            }
+            trigger->hooks = malloc (sizeof (trigger->hooks[0]));
+            if (trigger->hooks)
+            {
+                trigger->hooks_count = 1;
+                trigger->hooks[0] = weechat_hook_line (
+                    buffer_type,
+                    buffer_name,
+                    tags,
+                    &trigger_callback_line_cb,
+                    trigger, NULL);
+            }
+            break;
         case TRIGGER_HOOK_PRINT:
             tags = NULL;
             message = NULL;
@@ -363,15 +401,42 @@ trigger_hook (struct t_trigger *trigger)
                 trigger->hooks = malloc (sizeof (trigger->hooks[0]));
                 if (trigger->hooks)
                 {
+                    extra_vars = weechat_hashtable_new (32,
+                                                        WEECHAT_HASHTABLE_STRING,
+                                                        WEECHAT_HASHTABLE_STRING,
+                                                        NULL, NULL);
+                    if (extra_vars)
+                    {
+                        weechat_hashtable_set (extra_vars, "tg_trigger_name",
+                                               trigger->name);
+                    }
                     trigger->hooks_count = 1;
+                    eval_desc = (argc > 1) ? weechat_string_eval_expression (
+                        argv[1], NULL, extra_vars, NULL) : NULL;
+                    eval_args = (argc > 2) ? weechat_string_eval_expression (
+                        argv[2], NULL, extra_vars, NULL) : NULL;
+                    eval_desc_args = (argc > 3) ? weechat_string_eval_expression (
+                        argv[3], NULL, extra_vars, NULL) : NULL;
+                    eval_completion = (argc > 4) ? weechat_string_eval_expression (
+                        argv[4], NULL, extra_vars, NULL) : NULL;
                     trigger->hooks[0] = weechat_hook_command (
                         argv[0],  /* command */
-                        (argc > 1) ? argv[1] : "",  /* description */
-                        (argc > 2) ? argv[2] : "",  /* arguments */
-                        (argc > 3) ? argv[3] : "",  /* description of args */
-                        (argc > 4) ? argv[4] : "",  /* completion */
+                        (eval_desc) ? eval_desc : "",
+                        (eval_args) ? eval_args : "",
+                        (eval_desc_args) ? eval_desc_args : "",
+                        (eval_completion) ? eval_completion : "",
                         &trigger_callback_command_cb,
                         trigger, NULL);
+                    if (eval_desc)
+                        free (eval_desc);
+                    if (eval_args)
+                        free (eval_args);
+                    if (eval_desc_args)
+                        free (eval_desc_args);
+                    if (eval_completion)
+                        free (eval_completion);
+                    if (extra_vars)
+                        weechat_hashtable_free (extra_vars);
                 }
             }
             break;
@@ -456,6 +521,45 @@ trigger_hook (struct t_trigger *trigger)
                 }
             }
             break;
+        case TRIGGER_HOOK_INFO:
+            if (argv && (argc >= 1))
+            {
+                trigger->hooks = malloc (argc * sizeof (trigger->hooks[0]));
+                if (trigger->hooks)
+                {
+                    trigger->hooks_count = argc;
+                    for (i = 0; i < argc; i++)
+                    {
+                        trigger->hooks[i] = weechat_hook_info (
+                            argv[i],
+                            NULL,
+                            NULL,
+                            &trigger_callback_info_cb,
+                            trigger, NULL);
+                    }
+                }
+            }
+            break;
+        case TRIGGER_HOOK_INFO_HASHTABLE:
+            if (argv && (argc >= 1))
+            {
+                trigger->hooks = malloc (argc * sizeof (trigger->hooks[0]));
+                if (trigger->hooks)
+                {
+                    trigger->hooks_count = argc;
+                    for (i = 0; i < argc; i++)
+                    {
+                        trigger->hooks[i] = weechat_hook_info_hashtable (
+                            argv[i],
+                            NULL,
+                            NULL,
+                            NULL,
+                            &trigger_callback_info_hashtable_cb,
+                            trigger, NULL);
+                    }
+                }
+            }
+            break;
     }
 
     if (!trigger->hooks)
@@ -1226,7 +1330,9 @@ trigger_debug_dump_cb (const void *pointer, void *data,
 int
 weechat_plugin_init (struct t_weechat_plugin *plugin, int argc, char *argv[])
 {
-    int i, upgrading;
+    /* make C compiler happy */
+    (void) argc;
+    (void) argv;
 
     weechat_plugin = plugin;
 
@@ -1245,17 +1351,7 @@ weechat_plugin_init (struct t_weechat_plugin *plugin, int argc, char *argv[])
     /* hook completions */
     trigger_completion_init ();
 
-    /* look at arguments */
-    upgrading = 0;
-    for (i = 0; i < argc; i++)
-    {
-        if (weechat_strcasecmp (argv[i], "--upgrade") == 0)
-        {
-            upgrading = 1;
-        }
-    }
-
-    if (upgrading)
+    if (weechat_trigger_plugin->upgrading)
         trigger_buffer_set_callbacks ();
 
     return WEECHAT_RC_OK;
