@@ -1,7 +1,7 @@
 /*
  * xfer.c - file transfer and direct chat plugin for WeeChat
  *
- * Copyright (C) 2003-2017 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <stdlib.h>
@@ -296,8 +296,6 @@ xfer_close (struct t_xfer *xfer, enum t_xfer_status status)
 
     if (XFER_HAS_ENDED(xfer->status))
     {
-        xfer_send_signal (xfer, "xfer_ended");
-
         if (xfer->hook_fd)
         {
             weechat_unhook (xfer->hook_fd);
@@ -346,17 +344,26 @@ xfer_close (struct t_xfer *xfer, enum t_xfer_status status)
          || (xfer->status == XFER_STATUS_ABORTED))
         && XFER_IS_FILE(xfer->type)
         && XFER_IS_RECV(xfer->type)
-        && xfer->local_filename
+        && xfer->temp_local_filename
         && xfer->pos == 0)
     {
         /* erase file only if really empty on disk */
-        if (stat (xfer->local_filename, &st) != -1)
+        if (stat (xfer->temp_local_filename, &st) != -1)
         {
             if ((unsigned long long) st.st_size == 0)
-                unlink (xfer->local_filename);
+                unlink (xfer->temp_local_filename);
         }
     }
 
+    /* rename received file if it has a suffix */
+    if ((xfer->status == XFER_STATUS_DONE)
+        && XFER_IS_FILE(xfer->type)
+        && XFER_IS_RECV(xfer->type)
+        && (strcmp (xfer->local_filename, xfer->temp_local_filename) != 0))
+    {
+        rename (xfer->temp_local_filename, xfer->local_filename);
+    }
+
     if (XFER_IS_FILE(xfer->type))
         xfer_file_calculate_speed (xfer, 1);
 
@@ -370,6 +377,9 @@ xfer_close (struct t_xfer *xfer, enum t_xfer_status status)
         close (xfer->file);
         xfer->file = -1;
     }
+
+    if (XFER_HAS_ENDED(xfer->status))
+        xfer_send_signal (xfer, "xfer_ended");
 }
 
 /*
@@ -465,9 +475,16 @@ xfer_alloc ()
 
     time_now = time (NULL);
 
-    /* default values */
+    new_xfer->plugin_id = NULL;
+    new_xfer->plugin_name = NULL;
+    new_xfer->type = 0;
+    new_xfer->protocol = 0;
+    new_xfer->remote_nick = NULL;
+    new_xfer->local_nick = NULL;
+    new_xfer->charset_modifier = NULL;
     new_xfer->filename = NULL;
     new_xfer->size = 0;
+    new_xfer->proxy = NULL;
     new_xfer->local_address = NULL;
     new_xfer->local_address_length = 0;
     new_xfer->local_address_str = NULL;
@@ -475,16 +492,11 @@ xfer_alloc ()
     new_xfer->remote_address_length = 0;
     new_xfer->remote_address_str = NULL;
     new_xfer->port = 0;
-    new_xfer->remote_nick = NULL;
-    new_xfer->local_nick = NULL;
-    new_xfer->charset_modifier = NULL;
-
-    new_xfer->type = 0;
-    new_xfer->protocol = 0;
     new_xfer->status = 0;
     new_xfer->buffer = NULL;
     new_xfer->remote_nick_color = NULL;
     new_xfer->fast_send = weechat_config_boolean (xfer_config_network_fast_send);
+    new_xfer->send_ack = weechat_config_boolean (xfer_config_network_send_ack);
     new_xfer->blocksize = weechat_config_integer (xfer_config_network_blocksize);
     new_xfer->start_time = time_now;
     new_xfer->start_transfer = time_now;
@@ -498,6 +510,7 @@ xfer_alloc ()
     new_xfer->unterminated_message = NULL;
     new_xfer->file = -1;
     new_xfer->local_filename = NULL;
+    new_xfer->temp_local_filename = NULL;
     new_xfer->filename_suffix = -1;
     new_xfer->pos = 0;
     new_xfer->ack = 0;
@@ -507,6 +520,9 @@ xfer_alloc ()
     new_xfer->last_activity = 0;
     new_xfer->bytes_per_sec = 0;
     new_xfer->eta = 0;
+    new_xfer->hash_handle = NULL;
+    new_xfer->hash_target = NULL;
+    new_xfer->hash_status = XFER_HASH_STATUS_UNKNOWN;
 
     new_xfer->prev_xfer = NULL;
     new_xfer->next_xfer = xfer_list;
@@ -537,8 +553,15 @@ xfer_nick_auto_accepted (const char *server, const char *nick)
 
     rc = 0;
 
-    nicks = weechat_string_split (weechat_config_string (xfer_config_file_auto_accept_nicks),
-                                  ",", 0, 0, &num_nicks);
+    nicks = weechat_string_split (
+        weechat_config_string (xfer_config_file_auto_accept_nicks),
+        ",",
+        NULL,
+        WEECHAT_STRING_SPLIT_STRIP_LEFT
+        | WEECHAT_STRING_SPLIT_STRIP_RIGHT
+        | WEECHAT_STRING_SPLIT_COLLAPSE_SEPS,
+        0,
+        &num_nicks);
     if (nicks)
     {
         for (i = 0; i < num_nicks; i++)
@@ -637,8 +660,8 @@ xfer_new (const char *plugin_name, const char *plugin_id,
           const char *local_filename)
 {
     struct t_xfer *new_xfer;
-    const char *ptr_color, *ptr_crc32;
-    char str_address[NI_MAXHOST];
+    const char *ptr_crc32;
+    char str_address[NI_MAXHOST], *color;
     int rc;
 
     new_xfer = xfer_alloc ();
@@ -662,8 +685,10 @@ xfer_new (const char *plugin_name, const char *plugin_id,
     new_xfer->type = type;
     new_xfer->protocol = protocol;
     new_xfer->remote_nick = strdup (remote_nick);
-    ptr_color = weechat_info_get ("irc_nick_color_name", remote_nick);
-    new_xfer->remote_nick_color = (ptr_color) ? strdup (ptr_color) : NULL;
+    color = weechat_info_get ("irc_nick_color_name", remote_nick);
+    new_xfer->remote_nick_color = (color) ? strdup (color) : NULL;
+    if (color)
+        free (color);
     new_xfer->local_nick = (local_nick) ? strdup (local_nick) : NULL;
     new_xfer->charset_modifier = (charset_modifier) ? strdup (charset_modifier) : NULL;
     if (XFER_IS_FILE(type))
@@ -688,11 +713,11 @@ xfer_new (const char *plugin_name, const char *plugin_id,
     if (XFER_IS_RECV(type))
     {
         new_xfer->local_address_str = strdup ("");
-        xfer_set_remote_address(new_xfer, address, address_length, str_address);
+        xfer_set_remote_address (new_xfer, address, address_length, str_address);
     }
     else
     {
-        xfer_set_local_address(new_xfer, address, address_length, str_address);
+        xfer_set_local_address (new_xfer, address, address_length, str_address);
         new_xfer->remote_address_str = strdup ("");
     }
 
@@ -907,6 +932,8 @@ xfer_free (struct t_xfer *xfer)
     /* free data */
     if (xfer->plugin_id)
         free (xfer->plugin_id);
+    if (xfer->plugin_name)
+        free (xfer->plugin_name);
     if (xfer->remote_nick)
         free (xfer->remote_nick);
     if (xfer->local_nick)
@@ -915,6 +942,8 @@ xfer_free (struct t_xfer *xfer)
         free (xfer->charset_modifier);
     if (xfer->filename)
         free (xfer->filename);
+    if (xfer->proxy)
+        free (xfer->proxy);
     if (xfer->local_address)
         free (xfer->local_address);
     if (xfer->local_address_str)
@@ -923,12 +952,22 @@ xfer_free (struct t_xfer *xfer)
         free (xfer->remote_address);
     if (xfer->remote_address_str)
         free (xfer->remote_address_str);
+    if (xfer->buffer)
+        free (xfer->buffer);
     if (xfer->remote_nick_color)
         free (xfer->remote_nick_color);
+    if (xfer->hook_fd)
+        weechat_unhook (xfer->hook_fd);
+    if (xfer->hook_timer)
+        weechat_unhook (xfer->hook_timer);
+    if (xfer->hook_connect)
+        weechat_unhook (xfer->hook_connect);
     if (xfer->unterminated_message)
         free (xfer->unterminated_message);
     if (xfer->local_filename)
         free (xfer->local_filename);
+    if (xfer->temp_local_filename)
+        free (xfer->temp_local_filename);
     if (xfer->hash_handle)
     {
         gcry_md_close (*xfer->hash_handle);
@@ -946,6 +985,18 @@ xfer_free (struct t_xfer *xfer)
         xfer_buffer_selected_line = (xfer_count == 0) ? 0 : xfer_count - 1;
 }
 
+/*
+ * Frees all xfers.
+ */
+
+void
+xfer_free_all ()
+{
+    while (xfer_list)
+    {
+        xfer_free (xfer_list);
+    }
+}
 
 /*
  * Resolves address.
@@ -1184,7 +1235,7 @@ xfer_add_cb (const void *pointer, void *data,
     }
     else
     {
-        memset(&bind_addr, 0, sizeof(bind_addr));
+        memset (&bind_addr, 0, sizeof (bind_addr));
 
         /* determine bind_addr family from either own_ip or default */
         if (weechat_config_string (xfer_config_network_own_ip)
@@ -1650,6 +1701,8 @@ xfer_add_to_infolist (struct t_infolist *infolist, struct t_xfer *xfer)
         return 0;
     if (!weechat_infolist_new_var_string (ptr_item, "local_filename", xfer->local_filename))
         return 0;
+    if (!weechat_infolist_new_var_string (ptr_item, "temp_local_filename", xfer->temp_local_filename))
+        return 0;
     if (!weechat_infolist_new_var_integer (ptr_item, "filename_suffix", xfer->filename_suffix))
         return 0;
     snprintf (value, sizeof (value), "%llu", xfer->pos);
@@ -1738,6 +1791,7 @@ xfer_print_log ()
         weechat_log_printf ("  unterminated_message. . : '%s'",  ptr_xfer->unterminated_message);
         weechat_log_printf ("  file. . . . . . . . . . : %d",    ptr_xfer->file);
         weechat_log_printf ("  local_filename. . . . . : '%s'",  ptr_xfer->local_filename);
+        weechat_log_printf ("  temp_local_filename . . : '%s'",  ptr_xfer->temp_local_filename);
         weechat_log_printf ("  filename_suffix . . . . : %d",    ptr_xfer->filename_suffix);
         weechat_log_printf ("  pos . . . . . . . . . . : %llu",  ptr_xfer->pos);
         weechat_log_printf ("  ack . . . . . . . . . . : %llu",  ptr_xfer->ack);
@@ -1795,7 +1849,9 @@ xfer_debug_dump_cb (const void *pointer, void *data,
 int
 weechat_plugin_init (struct t_weechat_plugin *plugin, int argc, char *argv[])
 {
-    int i, upgrading;
+    /* make C compiler happy */
+    (void) argc;
+    (void) argv;
 
     weechat_plugin = plugin;
 
@@ -1825,15 +1881,7 @@ weechat_plugin_init (struct t_weechat_plugin *plugin, int argc, char *argv[])
 
     xfer_info_init ();
 
-    /* look at arguments */
-    upgrading = 0;
-    for (i = 0; i < argc; i++)
-    {
-        if (weechat_strcasecmp (argv[i], "--upgrade") == 0)
-            upgrading = 1;
-    }
-
-    if (upgrading)
+    if (weechat_xfer_plugin->upgrading)
         xfer_upgrade_load ();
 
     return WEECHAT_RC_OK;
@@ -1856,5 +1904,7 @@ weechat_plugin_end (struct t_weechat_plugin *plugin)
     else
         xfer_disconnect_all ();
 
+    xfer_free_all ();
+
     return WEECHAT_RC_OK;
 }
