@@ -1,7 +1,7 @@
 /*
  * tests.cpp - run WeeChat tests
  *
- * Copyright (C) 2014-2019 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2014-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -59,8 +59,9 @@ extern "C"
 
 /* import tests from libs */
 /* core */
-IMPORT_TEST_GROUP(Plugins);
 IMPORT_TEST_GROUP(CoreArraylist);
+IMPORT_TEST_GROUP(CoreCalc);
+IMPORT_TEST_GROUP(CoreCrypto);
 IMPORT_TEST_GROUP(CoreEval);
 IMPORT_TEST_GROUP(CoreHashtable);
 IMPORT_TEST_GROUP(CoreHdata);
@@ -73,7 +74,9 @@ IMPORT_TEST_GROUP(CoreUrl);
 IMPORT_TEST_GROUP(CoreUtf8);
 IMPORT_TEST_GROUP(CoreUtil);
 /* GUI */
+IMPORT_TEST_GROUP(GuiColor);
 IMPORT_TEST_GROUP(GuiLine);
+IMPORT_TEST_GROUP(GuiNick);
 /* scripts */
 IMPORT_TEST_GROUP(Scripts);
 
@@ -162,7 +165,7 @@ void
 run_cmd (const char *command)
 {
     printf (">>> Running command: %s\n", command);
-    input_data (ptr_core_buffer, command);
+    input_data (ptr_core_buffer, command, NULL);
 }
 
 /*
@@ -217,7 +220,6 @@ main (int argc, char *argv[])
     printf ("WeeChat arguments: \"%s\"\n", args);
 
     /* init WeeChat */
-    printf ("------------------------------------------------------------\n");
     weechat_init_gettext ();
     weechat_init (weechat_argc, weechat_argv, &test_gui_init);
     if (weechat_argv)
@@ -256,20 +258,11 @@ main (int argc, char *argv[])
 
     /* run all tests */
     printf ("\n");
-    printf (">>>>>>>>>> TESTS >>>>>>>>>>\n");
     rc = CommandLineTestRunner::RunAllTests (argc, argv);
-    printf ("<<<<<<<<<< TESTS <<<<<<<<<<\n");
-    printf ("\n");
 
     /* end WeeChat */
+    gui_chat_mute = GUI_CHAT_MUTE_ALL_BUFFERS;
     weechat_end (&gui_main_end);
-    printf ("------------------------------------------------------------\n");
-
-    /* display status */
-    printf ("\n");
-    printf ("\33[%d;1m*** %s ***\33[0m\n",
-            (rc == 0) ? 32 : 31,  /* 32 = green (OK), 31 = red (error) */
-            (rc == 0) ? "OK" : "ERROR");
 
     dlclose (handle);
 
